package facets.core.app;
import static facets.core.app.AppConstants.*;
import static facets.core.app.AppSurface.ContentStyle.*;
import static facets.core.app.Dialogs.Response.*;
import static facets.util.app.AppValues.*;
import facets.core.app.AppSurface.ContentStyle;
import facets.core.app.AppSurface.Contenter;
import facets.core.app.Dialogs.ExceptionTexts;
import facets.core.app.Dialogs.Response;
import facets.core.app.Dialogs.Surfacer;
import facets.core.superficial.Notice;
import facets.core.superficial.STarget;
import facets.core.superficial.SToggling;
import facets.core.superficial.STrigger;
import facets.core.superficial.TargetCore;
import facets.core.superficial.Notifying.Impact;
import facets.core.superficial.STarget.Targeted;
import facets.core.superficial.app.SAreaTarget;
import facets.core.superficial.app.SSurface;
import facets.facet.app.FacetAppSurface;
import facets.util.Debug;
import facets.util.Objects;
import facets.util.Tracer;
import facets.util.app.WatchableOperation;
import facets.util.app.AppValues;
import facets.util.app.WatchableOperation.CancelStyle;
import facets.util.app.AppWatcher.WatcherCoupler;
import facets.util.datatree.ValueNode;
import java.io.File;
import java.util.List;
/**
	Can launch dialogs and create root target elements on behalf of a 
	{@link ActionAppSurface}. 
<p>{@link AppActions} defines a range
of commonly required top-level application actions, together with
{@link STarget}s to represent them in the surface (typically as menu items). 
 */
public abstract class AppActions extends Tracer{
	public static final String TITLE_CLOSE_CONTENT="Close this content?",
		KEY_EXTRAS_SPLIT="extrasSplit",KEY_SHOW_EXTRAS="showExtras",
		KEY_SHOW_PREFERENCES="showPreferences";
	public static final int BLOCKING_TITLE_0=0,BLOCKING_RUBRIC_1=1,
	CANCEL_TITLE_2=2,CANCEL_RUBRIC_3=3;
	public static final int 
		TARGETS_CORE=0,
		TARGETS_WINDOW=1,
		TARGETS_NEW=2,
		TARGETS_HELP=3,
		TARGETS_LAST=TARGETS_HELP;
	private final WatchableOperation newContent=new WatchableOperation(
			"ValueAppActions.newContent"){
		@Override
		public void doSimpleOperation(){
			if(app.contentStyle!=SINGLE)app.addContent(app.getInternalContentSource());
			else if(contentIsRemovable(TITLE_CLOSE_CONTENT,app.findActiveContent())==Yes)
				app.replaceSingleContent(app.getInternalContentSource());
			else return;
			app.notify(new Notice(app.surfaceRootTargeter(),Impact.DEFAULT));
		}};
	protected ActionAppSurface app;
	private STarget windowBasics[],
		windowClose=new STrigger(TITLE_WINDOW_CLOSE,new STrigger.Coupler(){
		  public void fired(STrigger t){
		  	windowTryClose();
		  }
		});
	private StatePreferences preferences;
	/**
	Called by the private {@link WatcherCoupler} in {@link FacetAppSurface}. 
	 */
	protected boolean handleWatchCall(List<WatchableOperation>ops,Exception e,
			boolean firstRetry,boolean inOpen){
		String title=app.title();
		Dialogs dialogs=app.dialogs();
		WatchableOperation op=ops.get(0);
		String[]texts=op.getBlockingCancelTexts();
		boolean error=e!=null,detail=true&&Debug.natureDebug,
			cancel=op.cancelStyle()==CancelStyle.Dialog;
		if(!error){
			return cancel?dialogs.confirmYesNo(texts[CANCEL_TITLE_2],
					texts[CANCEL_RUBRIC_3])!=No 
			:dialogs.warningYesNo("Operation Timed Out",
				(detail?("Operation " +op):(firstRetry?"An operation":"The operation"))
						+(firstRetry?(" has timed out. If you do not wait for it to complete,<br>" +
						" $appTitle may slow down or close without warning.<br>&nbsp;<br>" +
						"Wait for the operation to complete?")
								:" has still not completed. <br>Carry on waiting?")
				)!=No;
		}
		ExceptionTexts tt=new ExceptionTexts("Operation Failed",
			("An operation has failed" +(detail?" as shown below. <br>":". <br>")), 
			(detail?"":("If you do not exit now, "+title+" may slow down or close without warning.<br>")) +
					"Exit "+title+"?",
			!detail?"":("Operation failed in\n" +Objects.toString(ops.toArray(),"\n")+
					"\ndue to: \n"+e+"\n"));
		if(dialogs.warningException(tt,e,inOpen)==Yes)attemptClose();
		return false;
	}
	/**
	Cover for {@link AppSurface#removeActiveRoot()}. 
	 */
	final public boolean windowTryClose(){
		return app.removeActiveRoot();
	}
	final protected void setWindowCloseLiveState(){
		if(windowClose==null)return;
		STarget[]roots=((SAreaTarget)app.surfaceRootTargeter().target()).indexableTargets();
		windowClose.setLive(roots.length>1||app.contentStyle==DESKTOP);
	}
	/**
	Encapsulates reverting the current content. 
	 */
	final public void revertContent(){
		Contenter content=app.findActiveContent();
		if(false&&content.hasChanged()&&app.dialogs().confirmYesNo("Reload content?",
				"Reload " +content.title()+"?")!=Yes)
			return;
		app.revertActiveContent();
	}
	protected Response contentIsRemovable(String dialogTitle,Contenter content){
		return !app.spec.canSaveContent()||!content.hasChanged()?Yes 
				:app.dialogs().confirmYesNo(dialogTitle,
						"Abandon changes to " +content.title()+"?");
	}
	/**
	Create targets representing the actions defined by the instance type. 
	<p>Called from {@link ActionAppSurface#lazySurfaceRootElements()} when creating
	the {@link SAreaTarget} which in turn creates {@link AppSurface#surfaceRootTargeter()}.  
	Default returns triggers grouped as they are likely to be 
	required for application menus; some expose member methods.  
	<p>Targeter elements can be accessed 
	using the following indexes into <code>elements</code>:
	<ul>
	<li>{@link #TARGETS_CORE} - {@link #launchPreferences()} and  
	{@link #attemptClose()}
	<li>{@link #TARGETS_WINDOW} etc; 
	</ul>
	 */
	protected STarget[]newSurfaceRootElements(){
		final ValueNode stateApp=app.values.state(PATH_CORE_APP);
		STarget 
			about=new STrigger(TITLE_APP_ABOUT+" "+app.title()+"...",
				new STrigger.Coupler(){
					public void fired(STrigger t){
						launchAbout();
					}
				}),
			show=new SToggling(TITLE_APP_HELP,stateApp.getOrPutBoolean(KEY_SHOW_EXTRAS,false),
				new SToggling.Coupler(){
					public void stateSet(SToggling t){
						stateApp.put(KEY_SHOW_EXTRAS,t.isSet());
						updateLayout();
					}
				}),
			news=new TargetCore("New Menu",new STrigger(TITLE_NEW,
				new STrigger.Coupler(){
				  public void fired(STrigger t){
						if(false)app.runWatched(newContent);
						else newContent.doSimpleOperation();
				  }
				})),
			preferences=new STrigger(TITLE_APP_PREFERENCES+"...",
				new STrigger.Coupler(){
					public void fired(STrigger t){
						launchPreferences();
					}
				}),
			close=new STrigger(TITLE_APP_CLOSE,new STrigger.Coupler(){
				public void fired(STrigger t){
					attemptClose();
			}
			}),
			core=new TargetCore("Core Menu",!showPreferences()?new STarget[]{close}
				:new STarget[]{preferences,close}),
			help=new TargetCore("Help Menu",!app.spec.offersHelp()?new STarget[]{about}
					:new STarget[]{show,about}),
			window=app.contentStyle==ContentStyle.SINGLE?new TargetCore("No Window Menu")
					:new TargetCore("Window Menu",newWindowTargets());
		setWindowCloseLiveState();
		return new STarget[]{core,window,news,help};
	}
	/**
	Called from default implementation of {@link #newSurfaceRootElements()}. 
	 */
	final protected STarget[]newWindowTargets(){
		if(windowBasics==null)windowBasics=
			app.contentStyle==DESKTOP?
				new STarget[]{
			new STrigger(TITLE_WINDOW_NEW,new STrigger.Coupler(){
			  public void fired(STrigger t){
					app.cloneActiveRoot();			
					windowClose.setLive(true);
				}
			}),				
			windowClose,				
		}
		:new STarget[]{windowClose};
		return TargetCore.join(windowBasics,
			 ((Targeted)((SAreaTarget)app.surfaceRootTargeter().target()
					 ).attachedFacet()).targets());
	}
	/**
	Convenience method calling {@link AppHost#updateLayout(SSurface)}. 
	<p>Called by the {@link STrigger} with title {@link AppConstants#TITLE_APP_HELP}
	created by the default implementation of {@link #newSurfaceRootElements()}
	and other framework elements. 
	 */
	final public void updateLayout(){
		((AppHost)app.host()).updateLayout(app);
	}
	/**
	Called by the {@link STrigger} with title {@link AppConstants#TITLE_APP_CLOSE}
	created by the default implementation of {@link #newSurfaceRootElements()}. 
	 */
	protected abstract void attemptClose();
	/**
	Called from constructor of {@link ActionAppSurface}. 
	 */
	protected void attachToApp(ActionAppSurface app){
		if(this.app!=null)throw new IllegalArgumentException(
				"App already attached in "+Debug.info(this));
		else this.app=app;
	}
	/**
	Supplier of {@link DialogSurface.Contenter}s and matching {@link DialogSurface} for 
	{@link AppActions#launchPreferences()}. 
	 */
	public interface StatePreferences{
		/**
		Create {@link Contenter}s exposing session state values.
		<p>Called by {@link AppActions#launchPreferences()} 
		 */
		DialogSurface.Contenter[]newContenters();
		/**
		Return a {@link Surfacer} creating a suitable {@link DialogSurface}
		for the {@link DialogSurface.Contenter}s created by {@link #newContenters()}.   
		<p>Called by {@link AppActions#launchPreferences()} 
		 */
		Surfacer newSurfacer();
	}
	/**
	Show preferences in core menu? 
	@return by default <code>false</code> unless {@link #KEY_SHOW_PREFERENCES}
	has been set in {@link AppValues#nature(boolean)}
	 */
	final protected boolean showPreferences(){
		return app.values.nature(true).getBoolean(KEY_SHOW_PREFERENCES);
	}
	/**
	Launches a preferences dialog exposing session state values. 
	<p>Calls {@link Dialogs#launchSurfaced(Surfacer, String, DialogSurface.DialogActions,DialogSurface.Contenter...)}
	with parameters returned by the {@link StatePreferences} created with 
	{@link #lazyPreferences()}.  
	Typically called by one of {@link #newSurfaceRootElements()}; 
	declared <code>public</code> to enable calls from client code. 
	 */
	final public void launchPreferences(){
		if(preferences==null)preferences=lazyPreferences();
		DialogSurface.Contenter[]contenters=preferences.newContenters();
		if(contenters==null||contenters.length==0)throw new IllegalStateException(
				"Null or empty contenters in "+Debug.info(this));
		Surfacer surfacer=preferences.newSurfacer();
		if(surfacer==null)throw new IllegalStateException("Null surfacer in "+Debug.info(this));
		app.dialogs().launchSurfaced(surfacer,TITLE_APP_PREFERENCES,
				DialogActionDefaults.newApplyOkCancel(),contenters);
	}
	/**
	Supply a {@link StatePreferences} for used by {@link #launchPreferences()}. 
	 */
	protected abstract StatePreferences lazyPreferences();
	/**
	Launch an 'about' pane.
	<p>Called by one of {@link #newSurfaceRootElements()}; declared <code>public</code> to enable
	calls from client code. 
	 */
	public void launchAbout(){
		String appName=app.title();
		app.dialogs().htmlPane(TITLE_APP_ABOUT+" "+appName,appName+".html");
	}
	/**
	May return source file on behalf of surface.
	<p>Called by {@link ActionAppSurface#getOpeningContentSources()};
	default returns <code>null</code>. 
	 */
	public File getOpeningContentSourceFile(){
	  return null;
	}
	/**
	Called from {@link ActionAppSurface#surfaceRetargeted()}. 
	<p>Default is empty stub. 
	 */
	protected void surfaceRetargeted(){}
}/*/*
This file forms part of Version 0.8.79 of Facets, a framework demonstrating 
the Superficial approach to GUI-driven interactive applications.  
http://superficial.sourceforge.net 				
Copyright (C) 2011  David M Wright 
This library is free software; you can redistribute it and/or modify it under 
the terms of the GNU Lesser General Public License as published by the 
Free Software Foundation; either release 3 of the License, or (at your 
option) any later release.
This library is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR 
A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more 
details.
You should have received a copy of the GNU Lesser General Public License along 
with this library; if not, write to the Free Software Foundation, Inc., 
59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/
