package facets.core.app;
import facets.core.app.AppSurface.ContentStyle;
import facets.facet.app.FacetAppValues;
import facets.util.Debug;
import facets.util.Tracer;
import facets.util.app.AppValues;
/**
Defines policy for an {@link AppSurface}.
<p><b>Note</b> Some methods are only used by {@link ActionAppSurface}.
 */
public abstract class AppSpecifier extends Tracer{
	private static AppSpecifier oneOnly;
	/** Immutable {@link AppValues} as passed to the constructor. */
	public final AppValues values;
	public AppSpecifier(AppValues values){
		if(oneOnly!=null&&this!=oneOnly)throw new IllegalStateException(
			"Only one capabilities allowed in "+Debug.info(this));
		else oneOnly=this;
		if((this.values=values)==null)throw new IllegalArgumentException(
				"Null values in "+Debug.info(this));
	}
	private AppSurface app;
	void attachToApp(AppSurface app){
		this.app=app;
	}
	/**
	The {@link AppSurface} specified by this {@link AppSpecifier}. 
	 */
	final public AppSurface app(){
		if(app==null)throw new IllegalStateException(
				"Null app in "+Debug.info(this));
		else return app;
	}
	/**
	@return by default {@link ContentStyle#SINGLE}
	 */
	public ContentStyle contentStyle(){
		return ContentStyle.SINGLE;
	}
	/**
	Used to compose an {@link ActionAppSurface}. 
	 */
	public abstract AppActions newActions();
	/**
	Referenced by default of {@link #canEditContent()}
	@return by default <code>false</code>
	 */
	public boolean canCreateContent(){
		return false;
	}
	/**
	Referenced by default of {@link #canSaveContent()}
	@return by default {@link #canCreateContent()}
	 */
	public boolean canEditContent(){
		return canCreateContent();
	}
	/**
	Referenced by default of {@link #canOverwriteContent()}
	@return by default {@link #canEditContent()}
	 */
	public boolean canSaveContent(){
		return canEditContent();
	}
	/**
	@return by default {@link #canSaveContent()}
	 */
	public boolean canOverwriteContent(){
		return canSaveContent();
	}
	/**
	Are help features available? 
	@return by default false
	 */
	public boolean offersHelp(){
		return false;
	}
}/*/*
This file forms part of Version 0.8.79 of Facets, a framework demonstrating 
the Superficial approach to GUI-driven interactive applications.  
http://superficial.sourceforge.net 				
Copyright (C) 2011  David M Wright 
This library is free software; you can redistribute it and/or modify it under 
the terms of the GNU Lesser General Public License as published by the 
Free Software Foundation; either release 3 of the License, or (at your 
option) any later release.
This library is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR 
A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more 
details.
You should have received a copy of the GNU Lesser General Public License along 
with this library; if not, write to the Free Software Foundation, Inc., 
59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/
