package facets.core.app;
import static facets.core.app.AppSurface.ContentStyle.*;
import facets.core.app.ViewerContenter.ContentSource;
import facets.core.superficial.Facetable;
import facets.core.superficial.FacetedTarget;
import facets.core.superficial.Notice;
import facets.core.superficial.Notifiable;
import facets.core.superficial.Notifying;
import facets.core.superficial.SFacet;
import facets.core.superficial.SFrameTarget;
import facets.core.superficial.SIndexing;
import facets.core.superficial.STarget;
import facets.core.superficial.STargeter;
import facets.core.superficial.STrigger;
import facets.core.superficial.TargeterCore.TargetType;
import facets.core.superficial.app.AreaTargeter;
import facets.core.superficial.app.SAreaTarget;
import facets.core.superficial.app.SContentRootTargeter;
import facets.core.superficial.app.SContenter;
import facets.core.superficial.app.SHost;
import facets.core.superficial.app.SSurface;
import facets.core.superficial.app.SView;
import facets.core.superficial.app.ViewableAction;
import facets.core.superficial.app.ViewableFrame;
import facets.core.superficial.app.ViewerTarget;
import facets.core.superficial.app.SHost.FacetLayout;
import facets.util.Debug;
import facets.util.Strings;
import facets.util.app.AppValues;
/**
Allows an {@link AppSurface} to be run headless eg for debugging.
 */
final public class Headless{
	/**
	Empty {@link AppHost} for a headless {@link AppSurface}.
	 */
	static public class HeadlessHost implements AppHost{
		private final AppSurface surface;
		HeadlessHost(AppSurface surface){
			this.surface=surface;
		}
		public void runHostedApp(){
			surface.openApp();
		}
		public Object wrapped(){
			throw new RuntimeException("Not implemented in "+Debug.info(this));
		}
		public void setTitle(String title){}
		public void setLayout(FacetLayout layout){}
		public SurfaceServices activeServices(){
			throw new RuntimeException("Not implemented in "+Debug.info(this));
		}
		public FacetLayout newDesktopLayout(SFacet contentFacet, LayoutFeatures features){
			throw new RuntimeException("Not implemented in "+Debug.info(this));
		}
		public void updateLayout(SSurface surface){}
		public Dialogs newDialogs(ActionAppSurface app){
			return null;
		}
		public void splashDown(){}
		public void splashUp(String msg){}
	}
	private static int targets;
	SAreaTarget surfaceRoot;
	static FacetLayout newLayout(){
		FacetLayout layout=new FacetLayout(){};
		return layout;
	}
	MountFacet newRootFacet(SAreaTarget surfaceRoot){
		return newFacet();
	}
	static AreaTargeter newRootTargeter(){
		return new AreaTargeter(newAreaTarget()){};
	}
	static SContentRootTargeter newContentRootTargeter(){
		TargetType target=newAreaTarget();
		SContentRootTargeter t=new SContentRootTargeter(target);
		t.retarget((STarget)target,Notifying.Impact.DEFAULT);
		t.setMonitor(new Notifiable(){
			public void notify(Notice notice){
				throw new RuntimeException("Not implemented in "+Debug.info(this));
			}});
		return t;
	}
	private static MountFacet newFacet(){
		return new MountFacet(){
			public STarget target(){
				throw new RuntimeException("Not implemented in "+Debug.info(this));
			}
			public void retarget(STarget target,Notifying.Impact impact){}
			public void setFacets(SFacet...facets){}
			public STarget[]targets(){
				return new STarget[]{};
			}
		};
	}
	private static SAreaTarget newAreaTarget(){
		return new SAreaTarget("TargetType"+targets++,
			SIndexing.newDefault("SIndexing"+targets++,
					new STarget[]{new SFrameTarget("TargetType"+targets++)},0)){
				public STargeter newTargeter(){
					return newRootTargeter();
				}};
	}
	public void buildRetargeted(){
		throw new RuntimeException("Not implemented in "+Debug.info(this));
	}
	public String title(){
		return Headless.class.getSimpleName();
	}
	void doAppTest(AppSurface app){
		if(false)Debug.printStackTrace(10);
		else if(false)throw new RuntimeException("Debug");
		else for(int i=0;i<10;i++){
			app.replaceSingleContent(app.getOpeningContentSources()[0]);
			Debug.memCheck("Headless.doAppTest ",true);
		}
		app.trace("~.doAppTest ");
		System.exit(0);
	}
	private final static ContentSource contentSource=new ContentSource(){
		private int texts=1;
		public Object newContent(){
			return Strings.newHtmlText("HTML"+texts++,10*1000);
		}
	};
	static void main(String[]args){
		AppSurface app=new AppSurface(new AppSpecifier(new AppValues(Headless.class)){
			@Override
			public AppActions newActions(){
				throw new RuntimeException("Not implemented in "+Debug.info(this));
			}
		}){
			private SHost host;
			protected SContenter newContenter(Object source){
				return new ViewerContenter(source){
					protected ViewableFrame newContentViewable(final Object source){
						return new ViewableFrame("ViewableFrame",source){
						};
					}
					protected FacetedTarget[]newContentViewers(ViewableFrame viewable){
						STarget view=new SFrameTarget(new SView(){
							public String title(){
								return "View";
							}
							public boolean isLive(){
								throw new RuntimeException("Not implemented in "+Debug.info(this));
							}
							public boolean allowMultipleSelection(){
								throw new RuntimeException("Not implemented in "+Debug.info(this));
							}
						});
						return new FacetedTarget[]{new ViewerTarget(viewable.title(),viewable,view){
									protected ViewableAction getTriggerAction(STrigger trigger){
										throw new RuntimeException("Not implemented in "+Debug.info(this));
									}
									protected STrigger[]newActionTriggers(
											ViewableAction[]actions){
										return new STrigger[]{};
									}}};
					}
					protected void attachContentRootFacets(SAreaTarget rootArea){
						for(STarget each:rootArea.descendants())
							if(each instanceof Facetable)((Facetable)each).attachFacet(newFacet());
					}
					public FacetLayout newContentLayout(SHost host,SFacet appRoot,
							SContentRootTargeter t){
						throw new RuntimeException("Not implemented in "+Debug.info(this));
					}
				};
			}
			protected boolean isHeadless(){
				return true;
			}
			public boolean contentIsRemovable(Contenter content){
				return true;
			}
			protected Object[]getOpeningContentSources(){
				return new Object[]{contentSource.newContent()};
			}
			protected FacetLayout newEmptyDesktopLayout(SHost host,
					SContentRootTargeter rootTargeter,SFacet surfaceRootFacet){
				throw new RuntimeException("Not implemented in "+Debug.info(this));
			}
			public SHost host(){
				if(host!=null)return host;
				return host=newHeadlessHost();
			}
		};
		((AppHost)app.host()).runHostedApp();
	}
}/*/*
This file forms part of Version 0.8.79 of Facets, a framework demonstrating 
the Superficial approach to GUI-driven interactive applications.  
http://superficial.sourceforge.net 				
Copyright (C) 2011  David M Wright 
This library is free software; you can redistribute it and/or modify it under 
the terms of the GNU Lesser General Public License as published by the 
Free Software Foundation; either release 3 of the License, or (at your 
option) any later release.
This library is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR 
A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more 
details.
You should have received a copy of the GNU Lesser General Public License along 
with this library; if not, write to the Free Software Foundation, Inc., 
59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/
