package facets.core.app;
import facets.core.superficial.SFacet;
import facets.core.superficial.STargeter;
import facets.core.superficial.app.ViewerTarget;
import facets.util.Debug;
/**
Creates {@link SFacet}s for a top-level menu. 
<p>{@link MenuFacets} supplies facets for a pull-down or 
context menu, facets that may in turn manage single or multiple
menu items or sub-menus. The facets and the top-level menu itself must be created 
using an appropriate facet builder. 
 */
public abstract class MenuFacets{
	final public STargeter targeter;
	final private String title;
  /**
  Unique constructor. 
  @param title is returned by {@link #title} if non-empty and that method 
  is not overridden
  @param t is set as {@link #targeter}
   */
  public MenuFacets(String title,STargeter t){
		if((this.targeter=t)==null)throw new IllegalArgumentException(
				"Null targeter in "+Debug.info(this));
		if((this.title=title)==null)throw new IllegalArgumentException(
				"Null title in "+Debug.info(this));
	}
	public MenuFacets(MenuFacets src){
		this(src.title(),src.targeter);
	}
	/**
	Return facets managing items for a menu. 
	 */
	public SFacet[]getFacets(){
		throw new RuntimeException("Not implemented in "+Debug.info(this));
	}
	/**
	Allows context sensitivity and merging of facets. 
	@param viewer 
	@param viewerFacets 
	@return by default {@link #getFacets()}
	 */
	public SFacet[]getContextFacets(ViewerTarget viewer, SFacet[]viewerFacets){
		return getFacets();
	}
	/**
	 Returns the title for a pull-down menu. 
	 <p>Returns either any non-empty title passed during construction, 
	 or that of the targeter passed. 
	 */
	final public String title(){
		return !title.equals("")?title:targeter.title();
	}
}
/*/*
This file forms part of Version 0.8.79 of Facets, a framework demonstrating 
the Superficial approach to GUI-driven interactive applications.  
http://superficial.sourceforge.net 				
Copyright (C) 2011  David M Wright 
This library is free software; you can redistribute it and/or modify it under 
the terms of the GNU Lesser General Public License as published by the 
Free Software Foundation; either release 3 of the License, or (at your 
option) any later release.
This library is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR 
A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more 
details.
You should have received a copy of the GNU Lesser General Public License along 
with this library; if not, write to the Free Software Foundation, Inc., 
59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

