package facets.core.app;
import facets.util.Tracer;
import facets.util.datatree.TypedNode;
import facets.util.datatree.ValueNode;
/**
Encapsulates editing of a {@link ValueNode} using an input dialog.
 */
public abstract class ValueEdit extends Tracer{
	private final PathSelection selection;
	private final ValueNode node;
	private final String values[],valueLine,pairKey,proposal;
	private final int pathAt;
	public ValueEdit(PathSelection selection){
		this.selection=selection;
		NodePath path=(NodePath)selection.paths[0];
		node=(ValueNode)selection.selection();
		values=node.values();
		if(false)trace(".ValueEdit: node=",node);
		pathAt=path.valueAt();
		valueLine=values.length==0||pathAt<0?null:values[pathAt];
		String[]pair=valueLine==null?null:valueLine.split("=",2);
		pairKey=pair==null||pair.length==1?null:pair[0];
		proposal=valueLine==null?node.title():pairKey==null?valueLine:pair[1];
	}
	public final boolean dialogEdit(){
		String title=valueLine==null?"Rename":"Set Value",
			rubricTail=valueLine==null?"title":pairKey==null?"value"
					:"value for '" +pairKey+"'",
			input=getDialogInput(title,"Enter new " +rubricTail,proposal).trim();
		if(input.equals("")||proposal.equals(input))return false;
		if(valueLine==null)node.setTitle(input);
		else if(pairKey==null){
			int valueAt=pathAt;
			for(Object item:node.contents())
				if(item instanceof TypedNode)valueAt++;
			node.putAt(valueAt,input);
		}
		else{
			node.put(pairKey,input);
			try{
				checkKeyValue(node,pairKey);
			}
			catch(Exception e){
				node.put(pairKey,proposal);
			}
		}
		return true;
	}
	protected void checkKeyValue(ValueNode node,String key){
		if(key.equals("text"))node.getString(key);
		else if(key.equals("keyedInts"))node.getInts(key);
	}
	protected abstract String getDialogInput(String title,String rubric,
			String proposal);
}/*/*
This file forms part of Version 0.8.79 of Facets, a framework demonstrating 
the Superficial approach to GUI-driven interactive applications.  
http://superficial.sourceforge.net 				
Copyright (C) 2011  David M Wright 
This library is free software; you can redistribute it and/or modify it under 
the terms of the GNU Lesser General Public License as published by the 
Free Software Foundation; either release 3 of the License, or (at your 
option) any later release.
This library is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR 
A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more 
details.
You should have received a copy of the GNU Lesser General Public License along 
with this library; if not, write to the Free Software Foundation, Inc., 
59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/
