package facets.core.app.avatar;
import facets.core.app.PathSelection;
import facets.core.superficial.app.SSelection;
import facets.core.superficial.app.SViewer;
import facets.core.superficial.app.SelectionView;
import facets.core.superficial.app.ViewableFrame;
import facets.util.Debug;
import facets.util.Tracer;
import facets.util.Util;
import facets.util.datatree.TypedNode;

import java.io.Serializable;
/**
Mix-in for avatar creation, painting and drag policies. 
<p>{@link AvatarPolicies} enables content policies for an {@link AvatarView}
to be separated cleanly from the display policy. 
 */
public abstract class AvatarPolicies extends Tracer implements Serializable{
	public boolean isContentSelectable(AvatarContent content){
		return true;
	}
	/**
	/**
	Return painters for non-avatar view elements. 
	@param viewer for references to view, selection
	@param p must be supplied by a facet builder
	@return a non-<code>null</code> {@link Painter}[];by default an empty array 
	 */
	public Painter getBackgroundPainter(SViewer viewer,PainterSource p){
		return Painter.EMPTY;
	}
	/**
	/**
	Defines how an avatar should paint its content. 
	@param content to be painted
	@param p must be supplied by a facet builder
	 */
	public AvatarPolicy avatarPolicy(SViewer viewer,AvatarContent content,PainterSource p){
		throw new RuntimeException("Not implemented in "+Debug.info(this));
	}
	/**
	Defines how the viewer should drag an avatar selection. 
    @param content content of the avatars selected 
   @param hit specifies where the drag was initiated in <code>selection</code>
	@param p must be supplied by a facet builder
	@param view will use the policy
	 */
	public DragPolicy dragPolicy(AvatarView view,AvatarContent[]content, 
			Object hit, PainterSource p){
		throw new RuntimeException("Not implemented in "+Debug.info(this));
	}
	/**
	Return avatar content to be displayed by viewer. 
	<p>Facilitates delegation by implementations of {@link ViewableFrame#newViewerSelection(SViewer)};
	@param viewer controlled by the {@link AvatarView} returning the receiver 
	as {@link AvatarView#avatars()}
	 @param viewable the current {@link ViewableFrame#selection()}
	 */
	public SSelection newAvatarSelection(SViewer viewer,SSelection viewable){
		throw new RuntimeException("Not implemented in "+Debug.info(this));
	}
}

/*/*
This file forms part of Version 0.8.79 of Facets, a framework demonstrating 
the Superficial approach to GUI-driven interactive applications.  
http://superficial.sourceforge.net 				
Copyright (C) 2011  David M Wright 
This library is free software; you can redistribute it and/or modify it under 
the terms of the GNU Lesser General Public License as published by the 
Free Software Foundation; either release 3 of the License, or (at your 
option) any later release.
This library is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR 
A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more 
details.
You should have received a copy of the GNU Lesser General Public License along 
with this library; if not, write to the Free Software Foundation, Inc., 
59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

