package facets.core.app.avatar;
import facets.core.superficial.app.SViewer;
import facets.util.Stateful;
/**
Defines how a custom avatar should paint its content. 
<p>An {@link AvatarPolicy} supplies the {@link Painter}s needed 
  by an avatar to paint its content in different selection and 
  pick states. 
 */
public abstract class AvatarPolicy{
	final public static Integer STATE_NORMAL=0,STATE_PICKED_SELECTED=1;
	/**
	Return painters for the unpicked avatar. 
	@param selected is the avatar selected?
	@param active is the view active?
	 */
	public abstract Painter[]newViewPainters(boolean selected,boolean active);
	/**
	Return motion painters to appear over the view painters. 
	@param selected is the avatar selected?
	@param hit defines where the hit occurred that triggered the change in painters
	 */
	public abstract Painter[]newPickPainters(Object hit,boolean selected);
	/**
	Defines the cursor to appear over the avatar in different 
    pick states. @param state an implementation-dependent object 
    @return an implementation-dependent object
	 */
	public Object stateCursor(Object state){
		return state==STATE_PICKED_SELECTED?SViewer.CURSOR_HAND:
				SViewer.CURSOR_DEFAULT;
	}
	/**
	Allows the policy to define a response where no avatar 
    has been picked. 
	 */
	public void avatarNotPicked() {}
}/*/*
This file forms part of Version 0.8.79 of Facets, a framework demonstrating 
the Superficial approach to GUI-driven interactive applications.  
http://superficial.sourceforge.net 				
Copyright (C) 2011  David M Wright 
This library is free software; you can redistribute it and/or modify it under 
the terms of the GNU Lesser General Public License as published by the 
Free Software Foundation; either release 3 of the License, or (at your 
option) any later release.
This library is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR 
A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more 
details.
You should have received a copy of the GNU Lesser General Public License along 
with this library; if not, write to the Free Software Foundation, Inc., 
59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/
