package facets.core.app.avatar;
import facets.util.Debug;
import facets.util.shade.Shade;
import java.net.URL;
/**
Abstracts the features required to paint {@link AvatarContent}.
 */
public abstract class PainterMaster{
	/**
	Abstracts outline features required to paint {@link AvatarContent}.
	 */
	public static abstract class Outlined extends PainterMaster{
		public static final Object PEN_STYLE_USE_DEFAULT="Use default pen contentStyle";
		public final Shade fill,pen;
		public final boolean pickable;
		public Outlined(Shade fill,Shade pen,boolean pickable){
			if(fill==null&&pen==null)throw new IllegalArgumentException(
					"Null shades in "+Debug.info(this));
			this.fill=fill;
			this.pen=pen;
			this.pickable=pickable;
		}
		public Shade getFill(){
			return fill;
		}
		public Shade getPen(){
			return pen;
		}
		public boolean isPickable(){
			return pickable;
		}
		public abstract Object getOutline();
		public Object penStyle(){
			return PEN_STYLE_USE_DEFAULT;
		}
		public Scaling scaling(){
			return Scaling.NONE;
		}
	}
	/**
	Abstracts text features required to paint {@link AvatarContent}.
	 */
	public static class Textual extends PainterMaster{
		public static boolean antiAlias=true;
		public final String text,face;
		public final int points,style=0;
		public final double x,y;
		public final Shade shade;
		public Textual(String text,String face,int points,double x,double y,Shade shade){
			this.text=text;this.face=face;this.points=points;this.x=x;this.y=y;
			this.shade=shade;			
		}
	}
	/**
	Defines how an {@link Outlined} should scale its pen.
	 */
	public enum Scaling{NONE,OUTLINE,PEN}
	/**
	Abstracts raster features required to paint {@link AvatarContent}.
	 */
	public static class Rastered extends PainterMaster{
		public final URL url;
		public Rastered(URL url){
			this.url=url;
		}
	}
}/*/*
This file forms part of Version 0.8.79 of Facets, a framework demonstrating 
the Superficial approach to GUI-driven interactive applications.  
http://superficial.sourceforge.net 				
Copyright (C) 2011  David M Wright 
This library is free software; you can redistribute it and/or modify it under 
the terms of the GNU Lesser General Public License as published by the 
Free Software Foundation; either release 3 of the License, or (at your 
option) any later release.
This library is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR 
A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more 
details.
You should have received a copy of the GNU Lesser General Public License along 
with this library; if not, write to the Free Software Foundation, Inc., 
59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/
