package facets.core.app.avatar;
import facets.util.geom.Vector;
/**
Policy for a custom viewer that plots to an unbounded plane.     
 */
public interface PlaneView extends AvatarView{
	/**
	Possible responses to resizing of the viewer managed by a {@link PlaneView}.
	 */
	enum ResizeResponse{RESCALE,RESHAPE}
	String ZOOM_OUT="Zoom &Out",ZOOM_IN="Zoom &In";
	final public static double INCH_PTS=72;
	/**
	The minimum unscaled height of the viewport onto the plane. 
	@return a positive <code>double</code>
	 */
	double showHeight();
	/**
	The minimum unscaled width of the viewport onto the plane. 
	@return a positive <code>double</code>
	 */
  double showWidth();
  /**
  Which way up is the unscaled Y axis?. 
  @return either +1 or -1. 
   */
  int ySign();
  /**
  The location of the viewport origin. 
  @return any {@link Vector}
   */
  Vector plotShift();
	/**
	Does the view adjust its scale or its shape if the viewer facet is resized? 
	 */
	ResizeResponse resizeResponse();
	/**
	Set the view area. 
	@param width will be returned as {@link #showWidth()}
	 @param height will be returned as {@link #showHeight()}
	 @param plotShift will be returned as {@link #plotShift()}
	 @param scale will be returned as 
	 */
	void setShowValues(double width,double height,Vector plotShift,double scale);
  /**
  Identifier that should change with each change in view properties. 
  @return typically an {@link Integer} containing an auto-incremented value
   */
  Object stamp();
	double scale();
}
/*/*
This file forms part of Version 0.8.79 of Facets, a framework demonstrating 
the Superficial approach to GUI-driven interactive applications.  
http://superficial.sourceforge.net 				
Copyright (C) 2011  David M Wright 
This library is free software; you can redistribute it and/or modify it under 
the terms of the GNU Lesser General Public License as published by the 
Free Software Foundation; either release 3 of the License, or (at your 
option) any later release.
This library is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR 
A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more 
details.
You should have received a copy of the GNU Lesser General Public License along 
with this library; if not, write to the Free Software Foundation, Inc., 
59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

