package facets.core.app.avatar;
import static facets.util.Debug.*;
import static java.lang.Double.*;
import facets.core.superficial.SFrameTarget;
import facets.core.superficial.STarget;
import facets.core.superficial.SToggling;
import facets.util.Debug;
import facets.util.Util;
import facets.util.geom.Line;
import facets.util.geom.Point;
import facets.util.geom.Vector;
import facets.util.shade.Shades;
/**
{@link AvatarViewWorks} that implements {@link PlaneView}.
<p>Also has a number of utility class methods.  
 */
public class PlaneViewWorks extends AvatarViewWorks implements PlaneView{
	final public static int LINES_SINGLE=1,LINES_GROUP=4,
		MODEL_CONSTRAIN=0,MODEL_SNAP=1,MODEL_EDIT=2,MODEL_ELEMENTS=3;
	public static final double NO_SCALE=1.0;
  private boolean doSnap=true,doConstrain=true,doEdit=true;  
	private double showHeight=NaN,showWidth=NaN,scale=NaN;
	private Vector plotShift=new Vector(0,0);
	private int stamp;
	public PlaneViewWorks(String title,double showWidth,double showHeight,
  		Vector plotShift,double scale, AvatarPolicies avatars){
		super(title,avatars);
		setShowValues(showWidth,showHeight,plotShift,scale);
	}
  public void setShowValues(double width,double height,Vector plotShift,
  		double scale){
		if(width<0||height<0)throw new IllegalArgumentException(
				"Invalid sizes: width="+Util.fx(width)+", height="+Util.fx(height)+
				" in "+info(this));
		showWidth=width;showHeight=height;
		if((this.plotShift=plotShift)==null)throw new IllegalArgumentException(
				"Null plotShift in "+info(this));
		if(scale!=scale)throw new IllegalArgumentException(
				"Scale NaN in "+info(this));
		else this.scale=scale;
		stamp++;
	}
	public STarget newFrame(){
    return new SFrameTarget(this){
      protected STarget[] lazyElements(){
				return PlaneViewWorks.this.newFrameElements();
			}
    };
  }
  public void setState(Object state){
		PlaneViewWorks src=(PlaneViewWorks)state;
		showHeight=src.showHeight;showWidth=src.showWidth;
	}
  public String toString() {
		return false?super.toString()
				:("showWidth="+showWidth+", showHeight="+showHeight+" plotShift="+plotShift);
	}
	/**
	Generate sets of abstract lines. 
	 <p>The number and arrangement of lines is unspecified, but each set will fit
	 within a box proportioned as defined by <code>width</code> and <code>height</code>; 
	 the sets will be generated right and down from <code>at</code>. 
	 * @param at the starting point for generating line sets
	 * @param width the width of the defining bounding box
	 * @param height the height of the defining bounding box
	 * @param count how many line sets to generate (the more, the smaller)
	 */
	final public Line[][]newLineSets(Point at,double width,double height,
			int count){
		Line[][]lineSets=new Line[count][];
		for(int i=0;i<lineSets.length;i++)
			lineSets[i]=newTestLineSet(1d/count,at,height,width,count-i);
		return lineSets;
	}
	protected STarget[]newFrameElements(){
    final STarget constrain=new SToggling("Constrain",doConstrain,
        new SToggling.Coupler(){
      public void stateSet(SToggling t){doConstrain=t.isSet();}
    }),
    snap=new SToggling("Snap",doSnap,new SToggling.Coupler(){
      public void stateSet(SToggling t){doSnap=t.isSet();}
    }),
    edit=new SToggling("Editable",doEdit,new SToggling.Coupler(){
      public void stateSet(SToggling t)
      	{doEdit=t.isSet();snap.setLive(doEdit);constrain.setLive(doEdit);}
    });
    return new STarget[]{constrain,snap,edit};
  }
  public final Line[]newTestLineSet(double scale,Point boxAt,double height,
  		double width,int shiftCount){
  	final double left=boxAt.x(),top=boxAt.y();
    Vector shift=scale==1?new Vector(0,0):
    		new Vector(width/shiftCount-scale,-height/shiftCount+scale);
    Point tl=new Point(left,top),tr=new Point(left+width*scale,top),
      br=new Point(left+width*scale,top-height*scale),
      bl=new Point(left,top-height*scale);
    return new Line[]{
      new Line(tl.shifted(shift),tr.shifted(shift)),
      new Line(tr.shifted(shift),br.shifted(shift)),
      new Line(br.shifted(shift),bl.shifted(shift)),
      new Line(bl.shifted(shift),tl.shifted(shift)),
    };
  }
	final public double showHeight(){
		return showHeight;
	}
	final public double showWidth(){
		return showWidth;
	}
	final public Vector plotShift(){
		return plotShift;
	}
	final public double scale(){
		return scale;
	}
	public int ySign(){
		return -1;
	}
	public ResizeResponse resizeResponse(){
		return ResizeResponse.RESCALE;
	}
	public Object backgroundStyle(){
		return Shades.white;
	}
	public boolean doConstrain(){
		return doConstrain;
	}
	public boolean doSnap(){
		return doSnap;
	}
	public boolean isLive(){
		return doEdit;
	}
	final public int markLeapPixels(){
		return markPixels()/2+pickHitPixels();
	}
	final public int markPixels(){
		return 6;
	}
	final public int pickHitPixels(){
		return 2;
	}
	public Object stamp(){
		return stamp;
	}
}
/*/*
This file forms part of Version 0.8.79 of Facets, a framework demonstrating 
the Superficial approach to GUI-driven interactive applications.  
http://superficial.sourceforge.net 				
Copyright (C) 2011  David M Wright 
This library is free software; you can redistribute it and/or modify it under 
the terms of the GNU Lesser General Public License as published by the 
Free Software Foundation; either release 3 of the License, or (at your 
option) any later release.
This library is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR 
A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more 
details.
You should have received a copy of the GNU Lesser General Public License along 
with this library; if not, write to the Free Software Foundation, Inc., 
59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

