package facets.core.app.avatar;
import facets.util.geom.Point;
/**
Policy for a viewer that zooms and pans over a bounded plane area.    
 */
public interface ZoomPanView extends PlaneView{
  /**
  The unscaled distance from the viewport origin to the left edge 
  of the bounded plane. 
  @return any <code>double</code>
   */
  double originToLeft();
  /**
  The unscaled distance from the viewport origin to the top edge 
  of the bounded plane. 
  @return any <code>double</code>
   */
  double originToTop();
  /**
  The unscaled height of the bounded plane. 
  @return a positive <code>double</code>
   */
  double planeHeight();
  /**
  The unscaled width of the bounded plane. 
  @return a positive <code>double</code>
   */
  double planeWidth();
  /**
  The last position set with {@link #setFocusAt(Point)}. 
   */
  Point focusAt();
	/**
  Set a position within the bounded plane that should appear as near as possible
  to the centre of the viewport. 
  @param at should define a position within the bounded plane
   */
  void setFocusAt(Point at);
}
/*/*
This file forms part of Version 0.8.79 of Facets, a framework demonstrating 
the Superficial approach to GUI-driven interactive applications.  
http://superficial.sourceforge.net 				
Copyright (C) 2011  David M Wright 
This library is free software; you can redistribute it and/or modify it under 
the terms of the GNU Lesser General Public License as published by the 
Free Software Foundation; either release 3 of the License, or (at your 
option) any later release.
This library is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR 
A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more 
details.
You should have received a copy of the GNU Lesser General Public License along 
with this library; if not, write to the Free Software Foundation, Inc., 
59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

