package facets.core.superficial;
import facets.core.superficial.app.SAreaTarget;
/**
{@link STarget} representing a viewer or containing area
with an attached {@link SFacet}. 
<p>In addition to its two superinterfaces, {@link FacetedTarget}
defines other shared methods for the members of a area target tree. 
 */
public interface FacetedTarget extends STarget,Facetable{
	/**
	Attach an immutable facet. 
	<p>As a {@link STarget} representing its {@link #attachedFacet()},
	{@link FacetedTarget} can only attach a single viewer or area facet;
	it should complain if any attempt is made to attach another.  
   */
	void attachFacet(SFacet facet);
  /**
	The {@link SFacet} attached with {@link #attachFacet(SFacet)}. 
	<p>Return the facet attached to this {@link FacetedTarget},
	complaining if none yet attached. 
	 */
	SFacet attachedFacet();
	/**
	The parent in the area target tree. 
	 */
	SAreaTarget areaParent();
	/**
	Should the targeter tree treat the facet as having the GUI focus? 
	 */
	boolean isActive();
	/**
	Ensure that the targeter tree treats the faceted as having the GUI focus.  
	@param notify if <code>true</code>, also trigger a retargeting
	@see Notifying
	 */
	void ensureActive(boolean notify);
}
/*/*
This file forms part of Version 0.8.79 of Facets, a framework demonstrating 
the Superficial approach to GUI-driven interactive applications.  
http://superficial.sourceforge.net 				
Copyright (C) 2011  David M Wright 
This library is free software; you can redistribute it and/or modify it under 
the terms of the GNU Lesser General Public License as published by the 
Free Software Foundation; either release 3 of the License, or (at your 
option) any later release.
This library is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR 
A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more 
details.
You should have received a copy of the GNU Lesser General Public License along 
with this library; if not, write to the Free Software Foundation, Inc., 
59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

