package facets.core.superficial;
import static facets.util.app.Events.*;
import facets.core.superficial.Notifying.Impact;
import facets.util.Debug;
import facets.util.Identified;
import facets.util.Tracer;
import facets.util.app.Events;
import facets.util.datatree.TypedNode;

import java.io.Serializable;
/**
Core implementation of key interfaces. 
<p>{@link NotifyingCore} is the shared superclass of both the {@link STarget} and
{@link STargeter} class hierarchies. 
<p>Declared <code>public</code> for documentation purposes only; client code should 
use the concrete subclass hierarchies. 
 */
abstract class NotifyingCore extends Tracer implements Notifying,Identified{
	private static final class ContentNode extends TypedNode{
		private final Object[]contents;
		ContentNode(String title,Object[]contents){
			super(Object.class,"Graph",title);
			this.contents=contents;
		}
		public Object[]contents(){
			return contents;
		}
	}
  private static int identities;
  private static final String TYPE_CONTENT="Content";
  final protected static TypedNode newDebugContentNode(String title,
  		Object source){
		return newDebugContentNode(title,new Object[]{source});
	}
  final protected static TypedNode newDebugContentNode(String title,
  		final Object[]source){
    final Object[]contents=new Object[source.length];
    for(int i=0;i<contents.length;i++)contents[i]=
      source[i]instanceof NotifyingCore?
      		((NotifyingCore)source[i]).newGraphNode()
      :source[i]instanceof Integer||source[i]instanceof Double|| 
	      source[i]instanceof Boolean||source[i]instanceof String
	      		||source[i]instanceof SFacet?
	      	(TypedNode)new ContentNode(validTitleString(source[i]),new Object[]{})
      :new ContentNode(Debug.info(source[i]),new Object[]{});
    TypedNode node=new ContentNode(title+(title.startsWith("[")?"":true?"":"()"),
    		contents);
    for(int i=0;i<contents.length;i++)
    	if(contents[i]instanceof TypedNode)((TypedNode)contents[i]).setParent(node);
    return node;
  }
	private static String validTitleString(Object source){
		String check=source.toString().replaceAll("[\\w]+"," ").trim();
		return check.equals("")?"validTitleString":Debug.info(source);
	}
	final protected static TypedNode newDebugMonitorNode(final Notifiable monitor){
    return new ContentNode("[monitor]",new Object[]{Debug.info(monitor)});
  }
	/**
	Create a <code>TypedNode[]</code> representation of this {@link Notifying} 
	for debug purposes.  
	 */
	protected TypedNode[]newDebugChildren(){
	  return monitor==null?new TypedNode[]{}:
	    new TypedNode[]{newDebugMonitorNode(monitor)};
	}
	transient Notifiable monitor;
	private final int identity=identities++;
	/**
	Returns a descriptive node for use in debug object graph. 
	 */
	final public TypedNode newGraphNode(){
    final TypedNode children[]=newDebugChildren();
    TypedNode graphNode=new ContentNode(Debug.info(this),children);
    for(int i=0;i<children.length;i++)children[i].setParent(graphNode);
    return graphNode;
  }
	public Notifying.Impact impact(){
		return Notifying.Impact.DEFAULT;
	}
	public Object identity(){
		return identity;
	}
  /**
	The current {@link Notifiable} in the notification tree, 
	as set with <code>setMonitor</code>.  
	<p>Complains if no parent has been set. 
	
			 */
  final public Notifiable monitor(){
  	if(monitor==null)throw new IllegalStateException("No monitor in "+Debug.info(this));
  	return monitor;
  }
	public void notify(Notice notice){
		if(trace)traceEvent("Notified in "+this+" with "+notice);
		if(monitor==null)return;
		if(!blockNotification())monitor.notify(notice.addSource(this));
		else if(trace)traceEvent("Notification blocked in "+this);
	}
  final public void notifyMonitor(Impact impact){
    if(monitor==null)return;
    monitor.notify(new Notice(this,impact));
  }
  /**
  Enables notification to be restricted to this member of the tree. 
  <p>Checked by {@link #notify(Notice)}; default returns <code>false</code>.
   */
  protected boolean blockNotification(){return false;}
	public final void setMonitor(Notifiable monitor){
  	if(false)traceDebug("NotifyingCore: ",this);
		this.monitor=monitor;
	}
  public String toString(){return Debug.info(this);}
}
/*/*
This file forms part of Version 0.8.79 of Facets, a framework demonstrating 
the Superficial approach to GUI-driven interactive applications.  
http://superficial.sourceforge.net 				
Copyright (C) 2011  David M Wright 
This library is free software; you can redistribute it and/or modify it under 
the terms of the GNU Lesser General Public License as published by the 
Free Software Foundation; either release 3 of the License, or (at your 
option) any later release.
This library is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR 
A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more 
details.
You should have received a copy of the GNU Lesser General Public License along 
with this library; if not, write to the Free Software Foundation, Inc., 
59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

