package facets.core.superficial;
import facets.core.superficial.Notifying.Impact;
import facets.util.Titled;
/**
Superfical target. 
<p>{@link STarget} represents an application element to one or more 
facets; it is also a {@link Notifying} to enable it to form part of the 
	notification tree. 
 */
public interface STarget extends Titled,Notifying{
	/**
	Creates {@link STarget}s representing its data and logic. 
	<p>{@link Targeted} is primarily used to enable communication between
	application code and specialised facet actions, in particular those
	of viewers. 
	 */
	public interface Targeted{
		/**
		Return targets representing content and logic. 
		@return a non-<code>null</code> {@link STarget}[]
		 */
		STarget[]targets();
	}
	final STarget NONE=new TargetCore("No target"){};
	/**
	Dynamically-defined children. 
<p>Return {@link STarget} child elements of this {@link STarget}.
 
		@return a non-<code>null</code> {@link STarget}[]
	 */
	STarget[]elements();
	/**
	Indicates whether the {@link STarget} should
	be exposed by a surface facet as open to control eg 'enabled'.   
	<p>Returns <code>true</code> only if both the following conditions are met:</p>
	<ul>
	<li>the {@link STarget} itself is 'live' as constructed or  
	set by <code>setLive</code></li> 
	<li>any {@link STarget} monitor also returns <code>isLive</code> as
	<code>true</code></li>   
	</ul>   
	 
		 */
	boolean isLive();
	/**
	Sets the internal state used by <code>isLive</code>. 
  
	 */
  void setLive(boolean live);
	boolean wantsFocus();
}
/*/*
This file forms part of Version 0.8.79 of Facets, a framework demonstrating 
the Superficial approach to GUI-driven interactive applications.  
http://superficial.sourceforge.net 				
Copyright (C) 2011  David M Wright 
This library is free software; you can redistribute it and/or modify it under 
the terms of the GNU Lesser General Public License as published by the 
Free Software Foundation; either release 3 of the License, or (at your 
option) any later release.
This library is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR 
A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more 
details.
You should have received a copy of the GNU Lesser General Public License along 
with this library; if not, write to the Free Software Foundation, Inc., 
59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

