package facets.core.superficial;
import facets.core.superficial.Notifying.Impact;
import facets.util.Titled;
/** 
Superficial targeter. 
*<p>A {@link STargeter} mediates between a target, its exposing facet(s) and 
the surface by extending three significant interfaces: 
 <ul> 
 <li>{@link SRetargetable} enables to to have a target.
 <li>{@link Notifying} enables it to form part of the notification tree.
 <li>{@link Facetable} enables it to attach facet which it 
 applies with its own target; note that some {@link STargeter}s have
 no facet, functioning only as organising elements of the targeter tree.
</ul> 
 
 */
public interface STargeter extends SRetargetable,Titled,Notifying,Facetable{
  /**
	Sets the <code>target</code> of the {@link STargeter} 
and those of any <code>elements</code> or other {@link STargeter} members. 
<p>Also sets the {@link STargeter} as notification monitor of 
its {@link STargeter} members and of its target.
@see facets.core.superficial.Notifying
   */
  void retarget(STarget target,Impact impact);
	/**
	Dynamically-defined children.   
<p>Return any children created dynamically during initial retargeting to
correspond with the child <code>elements</code> of <code>target</code>.    
	 */
	STargeter[]elements();
  /**Adds <code>facet</code> to a list of facets to be updated during <code>retargetFacets</code>. 
   */
  void attachFacet(SFacet facet);
  /**Updates all facets attached with {@link #attachFacet(SFacet)} and those of any child
{@link #elements()}. 
<p>Application code need never call this method directly; it is called 
on the surface root targeter after retargeting following notification,
 and then recursively through the targeter tree. 
   */
  void retargetFacets(Impact impact);
}
/*/*
This file forms part of Version 0.8.79 of Facets, a framework demonstrating 
the Superficial approach to GUI-driven interactive applications.  
http://superficial.sourceforge.net 				
Copyright (C) 2011  David M Wright 
This library is free software; you can redistribute it and/or modify it under 
the terms of the GNU Lesser General Public License as published by the 
Free Software Foundation; either release 3 of the License, or (at your 
option) any later release.
This library is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR 
A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more 
details.
You should have received a copy of the GNU Lesser General Public License along 
with this library; if not, write to the Free Software Foundation, Inc., 
59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

