package facets.core.superficial;
import facets.util.Debug;
import facets.util.Times;
/**
{@link STarget} representing an application process. 
<p>{@link STrigger} represents an application process  
  to be exposed to user view and control in the surface; additional 
  mechanism must be defined in a 
  {@link facets.core.superficial.STrigger.Coupler}. 
 */
final public class STrigger extends TargetCore{
	/**
	Connects a {@link STrigger} to the application. 
	<p>A {@link Coupler} supplies application-specific mechanism and policy
	for a {@link STrigger}.
	 */
	public static abstract class Coupler implements TargetCoupler{
		/**
		Called by <code>Trigger.fire</code>. 
		@param t the calling {@link STrigger}
		 */
		public abstract void fired(STrigger t);
	}
	/**
	The <code>Trigger.Coupler</code> passed to the constructor. 
	 */
	public final Coupler coupler;
	/**
	Unique constructor. 
	@param title passed to superclass
	@param coupler supplies application-specific mechanism
	 */
	public STrigger(String title,Coupler coupler){
		super(title);
		if(coupler==null)throw new IllegalArgumentException("Null coupler in "+Debug.info(this));
		this.coupler=coupler;
	}
	/**
	Initiates the application process represented. 
	<p>Calls {@link Coupler#fired(STrigger)}. 
	 */
  public void fire(){
		String msg="Firing trigger "+Debug.info(this);
  	if(isLive())coupler.fired(this);
  }
}/*/*
This file forms part of Version 0.8.79 of Facets, a framework demonstrating 
the Superficial approach to GUI-driven interactive applications.  
http://superficial.sourceforge.net 				
Copyright (C) 2011  David M Wright 
This library is free software; you can redistribute it and/or modify it under 
the terms of the GNU Lesser General Public License as published by the 
Free Software Foundation; either release 3 of the License, or (at your 
option) any later release.
This library is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR 
A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more 
details.
You should have received a copy of the GNU Lesser General Public License along 
with this library; if not, write to the Free Software Foundation, Inc., 
59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/
