package facets.core.superficial;
import static facets.util.app.Events.*;
import facets.core.superficial.Notifying.Impact;
import facets.core.superficial.app.AreaTargeter;
import facets.core.superficial.app.SContentRootTargeter;
import facets.util.Debug;
import facets.util.ItemList;
import facets.util.Util;
import facets.util.ItemList.TreeItems;
import facets.util.app.Events;
import facets.util.datatree.TypedNode;
/**
Implements {@link STargeter}. 
<p>{@link TargeterCore} is a public implementation of {@link STargeter} 
  to provide for extension in other packages; instances are generally 
  created by an implementation of {@link facets.core.superficial.TargetCore#newTargeter()}. 
*/
public class TargeterCore extends NotifyingCore implements STargeter{
  /**
  Marks the effective type of a dynamically constructed targeter tree. 
  <p>{@link TargetType} is implemented by {@link TargetCore} and thus by all
  {@link STarget} implementations which means that the effective type of a 
  {@link TargeterCore} can usually be marked by its target; in certain special cases
  a {@link STarget} type may be shared between two effective types, requiring 
  some other {@link TargetType} to be used instead. 
   */
  public interface TargetType{}
	public static int targeters;
	private transient STargeter[]elements;
	private transient ItemList<SFacet>facets=new ItemList(SFacet.class);
	private transient STarget target;  
  public final Class targetType;
  public void retargetFacets(Impact impact){
		SFacet[]facets=this.facets.items();
		for(int i=0;i<elements.length;i++)elements[i].retargetFacets(impact);
		if(facets==null)return;
	  for(int i=0;i<facets.length;i++){
			facets[i].retarget(target,impact);
			if(trace)traceEvent("Retargeted facet " +Debug.info(facets[i])+" in "+this);
		}
	}
	final public void attachFacet(SFacet facet){
		if(facet==null)throw new IllegalArgumentException("Null facet in "+Debug.info(this));
		if(!facets.contains(facet))facets.addItem(facet);
		if(trace)traceEvent("Attached facet "+Debug.info(facet)+" to "+Debug.info(this));
	}
	/**
	Construct a {@link TargeterCore} to match <code>target</code>. 
	<p>The target is usually a {@link STarget}, but may be any
	implementor of {@link TargetType}. 
	@param target provides the class set as {@link #targetType}.
     */
	public TargeterCore(TargetType target){
		targetType=target.getClass();
    targeters++;
    if(trace)traceEvent("Created " +//"targeter " +targeters+" "+
				this);
	}
  public void retarget(STarget target,Impact impact){
    if(target==null)throw new IllegalArgumentException(
    		"Null target in "+Debug.info(this));
    if(trace)traceEvent("Retargeting "+Debug.info(this)+" on "+Debug.info(target));
    this.target=target;
		STarget[]targets=target.elements();
		if(targets==null)throw new IllegalStateException("No targets in "+Debug.info(this));
		if(elements==null){
      elements=new STargeter[targets.length];
  		for(int i=0;i<elements.length;i++){
  			elements[i]=((TargetCore)targets[i]).newTargeter();
  			elements[i].setMonitor(this);
  		}
    }
		if(targets.length==elements.length)
			for(int i=0;i<elements.length;i++)elements[i].retarget(targets[i],impact);			
		if(((TargetCore)target).notifiesTargeter())target.setMonitor(this);
  }  
  final public STargeter[]elements(){
	  if(elements==null)throw new IllegalStateException("No elements in "+Debug.info(this));
	  return elements;
	}
	final public STarget target(){
	  if(target==null)throw new IllegalStateException("No target in "+Debug.info(this));
	  return target;
	}
	final public String title(){
		return target==null?"":target.title();
	}
	public String toString(){
		String targetInfo=target==null?"Unapplied":Debug.info(target);
		return Debug.info(this)+(true?"":" ["+targetInfo+"]");
	}
  protected TypedNode[]newDebugChildren(){
    ItemList<TypedNode>items=new ItemList(TypedNode.class);
    items.addItems(super.newDebugChildren());
    items.addItem(newDebugContentNode("target",target));
    if(elements.length!=0)
      items.addItem(newDebugContentNode("elements",elements));
    if(facets.items().length>0)
      items.addItem(newDebugContentNode("[facets]",facets.items()));
    return items.items();
  }
	public static STargeter[]treeItems(STargeter root){
		STargeter[]tree=new TreeItems<STargeter>(STargeter.class,root){
			@Override
			protected STargeter[]getChildren(STargeter parent){
				ItemList<STargeter>children=new ItemList(STargeter.class);
				children.addItems(parent.elements());
				if(parent instanceof AreaTargeter){
					children.add(((AreaTargeter)parent).indexedTargeter());
					if(parent instanceof SContentRootTargeter){
						SContentRootTargeter root=(SContentRootTargeter)parent;
						children.addItems(root.indexedTargeter(),
								root.view(),
								root.viewer(),
								root.content(),
								root.selection());
					}
				}
				return children.items();
			}
		}.items();
		return tree;
	}
	public static STargeter newRetargeted(TargetCore target,boolean live){
		target.setLive(live);
		STargeter t=target.newTargeter();
		t.retarget(target,Notifying.Impact.DEFAULT);
		return t;
	}
}
/*/*
This file forms part of Version 0.8.79 of Facets, a framework demonstrating 
the Superficial approach to GUI-driven interactive applications.  
http://superficial.sourceforge.net 				
Copyright (C) 2011  David M Wright 
This library is free software; you can redistribute it and/or modify it under 
the terms of the GNU Lesser General Public License as published by the 
Free Software Foundation; either release 3 of the License, or (at your 
option) any later release.
This library is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR 
A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more 
details.
You should have received a copy of the GNU Lesser General Public License along 
with this library; if not, write to the Free Software Foundation, Inc., 
59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

