package facets.core.superficial.app;
import java.io.Serializable;
/**
Selection within application content. 
<p>{@link SSelection} encapsulates the two key features of a 
selection (potentially multiple) within content:
<ul>
<li>the selectable content itself
<li>the currently selected portion(s) of the content
</ul>
<p>Implementors should provide a valid implementation of at least one of
{@link #selection()} and {@link #selections()}; 
instances are intended to be immutable, each defining a specific selection.  
 */
public interface SSelection extends Serializable{
	/**
	The content within which to select. 
	<p>This will generally be the content to display within a viewer; and thus
	itself a portion of some larger content.  
	 */
	Object content();
	/**
	Single object defining the currently selected portion(s) of {@link #content()}. 
	 */
	Object selection();
	/**
	Array defining the currently selected portion(s) of {@link #content()}. 
	 */
	Object[]selections();
}
/*/*
This file forms part of Version 0.8.79 of Facets, a framework demonstrating 
the Superficial approach to GUI-driven interactive applications.  
http://superficial.sourceforge.net 				
Copyright (C) 2011  David M Wright 
This library is free software; you can redistribute it and/or modify it under 
the terms of the GNU Lesser General Public License as published by the 
Free Software Foundation; either release 3 of the License, or (at your 
option) any later release.
This library is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR 
A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more 
details.
You should have received a copy of the GNU Lesser General Public License along 
with this library; if not, write to the Free Software Foundation, Inc., 
59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

