package facets.core.superficial.app;
import static java.awt.Cursor.*;
/** 
Supplies {@link SView} policy and {@link SSelection} content to 
a viewer facet. 
<p>{@link SViewer} defines the basic interface for a viewer, 
	providing a suitable viewer facet with its view policy and content selection 
	and enabling it to report mouse-defined actions. 
 */
public interface SViewer{
	 int CURSOR_DEFAULT=DEFAULT_CURSOR,CURSOR_TEXT=TEXT_CURSOR,
	  CURSOR_HAND=HAND_CURSOR,CURSOR_CROSSHAIR=CROSSHAIR_CURSOR,
	  CURSOR_WAIT=WAIT_CURSOR,CURSOR_MOVE=MOVE_CURSOR,
	  CURSOR_WEST=W_RESIZE_CURSOR,CURSOR_NORTH=N_RESIZE_CURSOR;
	/**
	Defines the viewer facet type and supplies its policy. 
	 */
	SView view();
	/**
	Defines a region of viewable content to be displayed by the viewer facet. 
	 */
	SSelection selection();
	/**
	To be called by the viewer facet whenever it makes a 
	selection change. 
	 @param selection defines the new selection. 
	 */
	void selectionChanged(SSelection selection);
	/**
  To be called by the viewer facet whenever it proposes an
  edit to the selection passed. 
	@param selection if <code>null</code> implies edit should be applied to the 
	current selection; otherwise the selection should be changed to match
	@param edit defines the change proposed to the selection
	@param interim if <code>true</code> the edit forms part of a sequence 
   */
  void selectionEdited(SSelection selection,Object edit, boolean interim);
  /**
	Does the viewer facet have focus?  
	 */
	boolean isActive();
}
/*/*
This file forms part of Version 0.8.79 of Facets, a framework demonstrating 
the Superficial approach to GUI-driven interactive applications.  
http://superficial.sourceforge.net 				
Copyright (C) 2011  David M Wright 
This library is free software; you can redistribute it and/or modify it under 
the terms of the GNU Lesser General Public License as published by the 
Free Software Foundation; either release 3 of the License, or (at your 
option) any later release.
This library is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR 
A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more 
details.
You should have received a copy of the GNU Lesser General Public License along 
with this library; if not, write to the Free Software Foundation, Inc., 
59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

