package facets.core.superficial.app;
import facets.util.StatefulCore;
/**
{@link SSelection}-aware {@link SView} implementation. 
<p>{@link SelectionView} creates a suitable {@link SSelection} for use by 
the viewer(s) it controls, based on the complete selection within
viewable content. 
 */
public abstract class SelectionView extends StatefulCore implements SView{
	/**
	Unique constructor. 
	@param title passed to superclass
	 */
	protected SelectionView(String title){
		super(title);
	}
	/**
	Implements interface method. 
	<p>Default returns <code>false</code>. 
	 */
	public boolean isLive(){
		return false;
	}
	/**
	Implements interface method. 
	<p>Default returns <code>false</code>. 
	 */
	public boolean allowMultipleSelection(){
		return false;
	}
	/**
	Return content to be displayed by viewer. 
	<p>Enables delegation from implementations of {@link ViewableFrame#newViewerSelection(SViewer)};
	default returns <code>viewable</code>
	@param viewer controlled by this view 
	 @param viewable the current {@link ViewableFrame#selection()}
	 */
	public SSelection newViewerSelection(SViewer viewer,SSelection viewable){
		return viewable;
	}
}
/*/*
This file forms part of Version 0.8.79 of Facets, a framework demonstrating 
the Superficial approach to GUI-driven interactive applications.  
http://superficial.sourceforge.net 				
Copyright (C) 2011  David M Wright 
This library is free software; you can redistribute it and/or modify it under 
the terms of the GNU Lesser General Public License as published by the 
Free Software Foundation; either release 3 of the License, or (at your 
option) any later release.
This library is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR 
A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more 
details.
You should have received a copy of the GNU Lesser General Public License along 
with this library; if not, write to the Free Software Foundation, Inc., 
59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

