package facets.facet;
import static facets.facet.FacetFactory.*;
import facets.core.app.ActionAppSurface;
import facets.core.app.MenuFacets;
import facets.core.app.MountFacet;
import facets.core.app.ViewerContenter;
import facets.core.superficial.FacetedTarget;
import facets.core.superficial.SFacet;
import facets.core.superficial.STarget;
import facets.core.superficial.STargeter;
import facets.core.superficial.TargetCore;
import facets.core.superficial.app.SAreaTarget;
import facets.core.superficial.app.ViewerTarget;
import facets.facet.toolkit.KWrap;
import facets.facet.toolkit.KitFacet;
import facets.util.Debug;
import facets.util.NumberPolicy;
import facets.util.StringFlags;
import facets.util.Tracer;
import facets.util.app.AppValues;
import facets.util.datatree.ValueNode;
import facets.util.shade.Shade;
import facets.util.shade.Shades;
/**
Sub-factory for viewer, area and surface facets.
<p>{@link AreaFacets} provides a range of methods to create and attach viewer 
facets and their containing area facets to members of the area target tree.
 */
final public class AreaFacets extends Tracer{
	public class PaneFacets extends MenuFacets{
		private final SFacet[]facets;
		public PaneFacets(String title,STargeter t){
			super(title,t);
			STargeter[]elements=targeter.elements();
			String hints=HINT_NONE;
			facets=new SFacet[]{
				withDialog()?core.triggerMenuItems(elements[PANE_DIALOG],hints)
						:core.togglingCheckboxMenu(elements[PANE_SHOW],hints),
						core.triggerMenu(elements[PANE_LAYOUT],hints),
				core.triggerMenuItems(elements[PANE_ACTIVE],hints),
			};
		}
		protected boolean withDialog(){
			return false;
		}
		@Override
		public SFacet[]getFacets(){
			return facets;
		}
	}
	final public static Shade COLOR_DESKTOP=Shades.gray;
  final public static NumberPolicy SASH_SPLIT_POLICY=new NumberPolicy(10,90){
	  public int format(){return 0;}
	};
	final public static String PAGES_SINGLE="Single page",
		PAGES_SHARED="Shared page",
		TITLE_DESKTOP_TILE="Tile Windows",
		TITLE_DESKTOP_SCALE="Scale to Desktop",
		KEY_DESKTOP_NO_SCALE="desktopNoScale",
		KEY_DESKTOP_MAXIMUM="desktopMaximum";
	final public static int TARGET_DESKTOP_TILE=0,TARGET_DESKTOP_SCALE=1;
	/**Pane layout constant*/
	final public static int PANE_LEFT=0,PANE_UPPER=PANE_LEFT,PANE_RIGHT=1,
		PANE_LOWER=PANE_RIGHT,PANE_SPLIT_VERTICAL=0,PANE_SPLIT_HORIZONTAL=1,
		PANE_LAYOUT=0,PANE_SHOW=1,PANE_ACTIVE=2,PANE_DIALOG=3,
		PANE_LAST=PANE_DIALOG,PANE_ACTIVE_MAXIMISE=0,PANE_ACTIVE_RESTORE=1;
	/**State storage key element*/
	final public static String PANE_KEY_FLIP="paneFlip",PANE_KEY_SPLITS="paneSplits",
		PANE_RESET="Reset",PANE_FLIP="Flip",
		PANE_KEY_TAB_AND_HIDE="paneTabAndHide",
		PANE_KEY_ASK="paneAsk",PANE_KEY_DEFAULT="paneDefaults",
		TABLE_KEY_COLUMNS="tableColumns",STATE_NODE_AREA="areas";
	private final FacetsCore core;
	private final ValueNode areaState;
	public AreaFacets(FacetsCore core,AppValues values){
		this.core=core;
		areaState=values==null?new ValueNode("State","Area dummy")
			:values.state(AppValues.PATH_CORE,STATE_NODE_AREA);
	}
	/**
	Convenience method to define a pane set with a single root.
	<p>Duplicates {@link #panes(SAreaTarget, SFacet[], String, String, int[][], double[], int[], String[])}
	with constructed parameters.   
	@param aspect one of {@link #PANE_SPLIT_HORIZONTAL} or {@link #PANE_SPLIT_HORIZONTAL}
	 */
	public SFacet panes(SAreaTarget area,SFacet[]contents,String title,String hints,
			int aspect){
		return new PaneSet(core.kit,contents,
				new PaneSetLayout(area,areaState,title,new int[][]{{aspect}},new double[]{},
						new int[]{},new String[]{}));
	}
	/**
	Convenience method to define a pane set with a single root.
	<p>Duplicates {@link #panes(SAreaTarget, SFacet[], String, String, int[][], double[], int[], String[])}
	with a single-member <code>int[][]</code>. 
	 * @param layout codes defining the layout of the single root   
   */
  public SFacet panes(SAreaTarget area,SFacet[]contents,String title,String hints,
  		int[]layout,double[]splitDefaults,int[]tabAndHideDefaults){
		return new PaneSet(core.kit,contents,
			new PaneSetLayout(area,areaState,title,new int[][]{layout},splitDefaults,
					tabAndHideDefaults,new String[]{}));
	}
	/**
	Builds pane set based on a sashed mount.
	<p>The mount is attached to <code>area</code>; the members of  
	<code>contents</code> must be be attached to its children. 
	<p>The <code>int[]</code> members of <code>codes</code> 
	each define a layout as decribed below. 
	Any <code>int[]</code> member after the first defines a layout
	within the next available child defined by the first. 
	<p>Layouts are defined as follows using <code>PANE_</code> and <code>SPLIT_</code> constants:   
<ul>
	<li>The first constant specifies how the current pane is to be split ie 
	{@link #PANE_SPLIT_HORIZONTAL}
	or {@link #PANE_SPLIT_VERTICAL}
	<li>The second specifies which of the panes thus created is to be split further ie
	{@link #PANE_LEFT}/{@link #PANE_UPPER} or {@link #PANE_RIGHT}/{@link #PANE_LOWER}
	</ul>	
	@param area holds the children
	@param contents are attached to the children
	@param title may be used as key to store sash state
	@param hints may be HINT_PANEL_HAS_TOOLS
	@param codes define a tree of sash splits using <code>SASH_</code> 
	constants as described above
	@param splitDefaults defines default weightings for each left/upper split pane
	@param tabAndHideDefaults panes to be hidden initially, preceded by root areas
	to set children in tabs
	@param groups 
	@return the mount built
   */
  public SFacet panes(SAreaTarget area,SFacet[]contents,String title,String hints,
  		int[][]codes,double[]splitDefaults,int[]tabAndHideDefaults,String[]groups){
		return new PaneSet(core.kit,contents,
			new PaneSetLayout(area,areaState,title,codes,splitDefaults,tabAndHideDefaults,groups));
	}
	public STarget panesGetTargets(SAreaTarget area){
		return area.indexableTargets().length==1?new TargetCore("Single Pane") 
			:((PaneSet)area.attachedFacet()).targets;
	}
	public void panesValidateLayout(SAreaTarget area){
		((PaneSet)area.attachedFacet()).layout.validate();
	}
	/**
	Convenience method that creates and lays out viewer areas. 
	@param area containing at least one viewer area meeting the contract of 
	{@link ViewerContenter}<code>.newContentViewers</code>
	 @param splitAspect passed where required 
	to {@link #panes(SAreaTarget, SFacet[], String, String, int)}	
	@param vam may be null in which case a default is constructed
	 */
	public void attachViewerAreaPanes(SAreaTarget area,int splitAspect,ViewerAreaMaster vam){
		final boolean single=area.indexableTargets().length==1;
		if(vam==null)vam=new ViewerAreaMaster(){
			protected String hintString(){
				return single?HINT_BARE:HINT_NONE;
			}
		};
		if(area.indexedTarget()instanceof SAreaTarget){
			SFacet[]childAreas=viewerAreaChildren(area,vam);
			if(childAreas.length>1)panes(area,childAreas,"Panes",HINT_NONE,splitAspect);
			else mount(area,HINT_NONE).setFacets(childAreas[0]);
		}
		else viewerArea(area,vam);
	}
	/**
	Creates a viewer facet for the viewer frame contained by an area. 
	 <p>The facet, with an avatar pane defined by <code>vam</code>,
	  is attached to the {@link facets.core.superficial.app.ViewerTarget} 
	 returned by {@link SAreaTarget#activeFacet()} in <code>area</code>. 
	 <p>The facet returned <i>contains</i> the viewer facet and is 
	 accessible as {@link SAreaTarget#attachedFacet()}in <code>area</code>; 
	 it manages the toolkit container of the avatar pane. 
	 */
	public SFacet viewerArea(SAreaTarget area,final ViewerAreaMaster vam){
		return new ViewerFacet((ViewerTarget)area.activeFacet(),core.kit,vam,
				areaState).newAreaFacet(area);
	}
	/**
	Convenience method that creates viewers for an array of area children. 
	 */
	public SFacet[]viewerAreaChildren(SAreaTarget area,ViewerAreaMaster vam){
	  ViewerAreaMaster[]childMasters=vam.childMasters(area);
	  STarget[]childAreas=area.indexableTargets();
	  SFacet[]viewers=new SFacet[childMasters.length];
	  for(int i=0;i<childMasters.length;i++)
			viewers[i]=viewerArea((SAreaTarget)childAreas[i],childMasters[i]);
		return viewers; 
	}
	public void attachFacetArea(SFacet facet,SAreaTarget area){
		SAreaTarget then=(SAreaTarget)((FacetCore)facet).target();
		if(then.activeContent().framed==area.activeContent().framed)return;
		area.attachFacet(then.attachedFacet());
		area.activeFacet().attachFacet(then.activeFacet().attachedFacet());
	}
	public SFacet viewerGrid(SAreaTarget area,ViewerAreaMaster vam){
		final KWrap[]contents=FacetsCore.newBaseWraps(viewerAreaChildren(area, vam));
	  final String useHints=vam.hintString()+HINT_BARE+HINT_GRID;
	  return new AreaMountCore(area,core.kit){
			protected KWrap lazyBase(){
			  for(int i=0;i<contents.length;i++)registerPart(contents[i]);
	      return kit.wrapMount(this,contents,0,0,new StringFlags(useHints));
			}
		  public void setFacets(SFacet...facets){
				throw new RuntimeException("Not implemented in "+Debug.info(this));
			}
	  };
	}
	public SFacet viewerTabs(SAreaTarget area,ViewerAreaMaster vam){
		final SFacet[]viewers=viewerAreaChildren(area, vam);
		return new AreaMountCore(area,core.kit){
			final protected KWrap lazyBase(){return kit.viewerTabs(viewers);}
		  public void setFacets(SFacet...facets){
				throw new RuntimeException("Not implemented in "+Debug.info(this));
			}
		};
	}
	/**
	Attaches a {@link MountFacet} to the area passed. 
	@return the mount attached
	 */
	public MountFacet mount(SAreaTarget area,final String hints){
		return new AreaMount(area,core.kit){
			KWrap lazyBase(){
				return kit.spreadMount(this,new StringFlags(hints));
			}
		};		
	}
	final public MountFacet tabs(SAreaTarget area,final String hints){
		STarget[]children=area.indexableTargets();
		final SFacet[]facets=new SFacet[children.length];
		for(int i=0;i<facets.length;i++)
			facets[i]=((SAreaTarget)children[i]).attachedFacet();
		if(false)area.trace("tabs: " +area,children);
		return new AreaMount(area,core.kit){
			final protected KWrap lazyBase(){
				return kit.areaTabs(facets,new StringFlags(hints));
			}
		};
	}
	/**
	Create a programmatically switchable container for facets in an area tree. 
	<p>The container will be switchable programmatically but not within the GUI eg 
	a card layout. 
	@param area will have attached a {@link MountFacet} managing a container
	for those in turn managed by the {@link SFacet}s attached to the {@link FacetedTarget}
	members of its {@link SAreaTarget#indexableTargets()}. 
	 */
	final public MountFacet switchMount(SAreaTarget area){
		STarget[]children=area.indexableTargets();
		final KWrap[]bases=new KWrap[children.length];
		for(int i=0;i<bases.length;i++)bases[i]=
			((KitFacet)((SAreaTarget)children[i]).attachedFacet()).base();
		if(false)area.trace("switchMount: "+area,children);
		return new AreaMount(area,core.kit){
			KWrap lazyBase(){
				return kit.switchMount(this,bases,StringFlags.EMPTY);
			}
		};
	}
	/**
	Constructs the root facet of a multi-content application surface. 
	 <p>If the widget kit has the capacity to do so, may construct this root as 
	 <ul>
	  <li>a tab folder with tabs containing content root core</li> 
	 <li>an internal desktop whose windows contain these core</li>
	</ul>
	 @param app the app
	 @param surfaceRoot the surface root
	 @param desktop requests that the root be created as an internal desktop,
	 with internal windows attached to the children of <code>surfaceRoot</code>  
	 */
	public MountFacet appMultiContentFacet(final ActionAppSurface app,
			SAreaTarget surfaceRoot,final boolean desktop){
		return new AreaMount(surfaceRoot,core.kit){
			protected KWrap lazyBase(){
				return kit.rootMount(app,this,desktop);
			}
		};
	}
}
/*/*
This file forms part of Version 0.8.79 of Facets, a framework demonstrating 
the Superficial approach to GUI-driven interactive applications.  
http://superficial.sourceforge.net 				
Copyright (C) 2011  David M Wright 
This library is free software; you can redistribute it and/or modify it under 
the terms of the GNU Lesser General Public License as published by the 
Free Software Foundation; either release 3 of the License, or (at your 
option) any later release.
This library is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR 
A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more 
details.
You should have received a copy of the GNU Lesser General Public License along 
with this library; if not, write to the Free Software Foundation, Inc., 
59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

