package facets.facet;
import static facets.core.app.TextView.*;
import static facets.util.StringFlags.*;
import static facets.util.app.AppValues.*;
import facets.core.app.AppActions;
import facets.core.app.AppConstants;
import facets.core.app.AppHost;
import facets.core.app.AppHost.LayoutFeatures;
import facets.core.app.AppSpecifier;
import facets.core.app.AppSurface.ContentStyle;
import facets.core.app.Dialogs.ExceptionTexts;
import facets.core.app.MenuFacets;
import facets.core.app.NodePath;
import facets.core.app.StatefulViewable.ClipperSource;
import facets.core.app.SurfaceServices;
import facets.core.superficial.Notifying;
import facets.core.superficial.SFacet;
import facets.core.superficial.SIndexing;
import facets.core.superficial.STarget;
import facets.core.superficial.STargeter;
import facets.core.superficial.STextual;
import facets.core.superficial.STrigger;
import facets.core.superficial.TargetCore;
import facets.core.superficial.TargeterCore;
import facets.core.superficial.app.AreaTargeter;
import facets.core.superficial.app.IndexingTargeter;
import facets.core.superficial.app.SAreaTarget;
import facets.core.superficial.app.SContentRootTargeter;
import facets.core.superficial.app.SHost;
import facets.core.superficial.app.ViewerTarget;
import facets.facet.app.FacetAppActions;
import facets.facet.app.FacetAppSurface;
import facets.facet.app.FacetAppValues;
import facets.facet.app.FileAppActions;
import facets.facet.toolkit.KWrap;
import facets.facet.toolkit.Kit;
import facets.facet.toolkit.KitFacet;
import facets.facet.toolkit.swing.KitSwing;
import facets.util.Debug;
import facets.util.ItemList;
import facets.util.OffsetPath;
import facets.util.app.AppServices;
import facets.util.app.AppValues;
import facets.util.app.Events;
import facets.util.app.ProvidingCache;
import facets.util.datatree.ValueNode;
import facets.util.shade.Shade;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import javax.swing.JMenuBar;
/**
Factory that creates and lays out facets. 
<p>{@link FacetFactory} provides a facet builder API for creating facets, 
  attaching them to a targeter tree and assembling them into a 
  complete application layout. 
<p>Most of its methods comply with the following general contract: 
<ul>
  <li>The method name denotes the type of facet that will be returned and 
  often includes the type of target that it exposes.
  <li>The facet returned will be attached to the targeter passed, which
  must have a target of the right type.
  <li><a href="#Hints">Hints</a> passed will be honoured on a 
    best efforts basis.
</ul>
<p>The core {@link FacetFactory} for a surface must be created using one of the 
	class methods {@link #newDesktopCore(FacetAppValues, ToolkitType)} 
	and {@link #newAppletCore(SurfaceStyle, Shade, FacetAppValues)}; 
	it may then be passed to the constructor of concrete subclasses which 
	will thus use the same toolkit and other resources, typically within 
	methods re-implementing {@link LayoutFeatures}. 
<h3><a name="Hints"></a>Hints</h3>
<p>Hints can be any concatenation of the <code>HINT_XXX</code> class constants; 
	for examples of their use see <code>XXXFacets</code> classes in the <code>demo</code> 
	package tree included with the Facets distribution.
 */
public abstract class FacetFactory extends FacetsCore implements LayoutFeatures,
		AppServices{
  /**
  Coupler for any combination of indexed list and text box. 
	<p>Implements the coupler interfaces of both 
	{@link facets.core.superficial.SIndexing} and {@link facets.core.superficial.STextual}. 
	 */
	public abstract static class ComboCoupler extends facets.core.superficial.SIndexing.Coupler{
   public boolean indexedTitleEditable(SIndexing i){return true;}
   public abstract void indexedTitleEdited(String edit);
   final public void _textSet(STextual t){indexedTitleEdited(t.text());}
 }
	final public static int GRAPH_NONE=0,GRAPH_APP=1,GRAPH_DIALOGS=2,
		MENU_APP=0,MENU_EDIT=1,MENU_PANE=2,MENU_WINDOW=3,MENU_HELP=4;
	public static boolean graphBuild=true,graphFindNow;
	public static final String GRAPH_FIND_NONE="[Enter text to match in title]";
	public static String findGraphValue=GRAPH_FIND_NONE;
	protected final static OffsetPath GRAPH_NO_PATH=new NodePath(new int[]{0,0});
	public static int graphShowWhere=0,graphFindAt=0;
 /**Shade to be used for surface panels and menus.
 	<p>May be <code>null</code> to signify use of toolkit defaults. */
	public static Shade panelShade;
	/**
	Possible app surface styles. 
	 */
	public enum SurfaceStyle{
		/**
		Desktop surface contentStyle with toolbar above content area, status bar below. 
		 */
		DESKTOP,
		/**
		Applet surface contentStyle with tool panel below or beside content area. 
		 */
		APPLET,
		/**
		Applet surface contentStyle with panel shade to match page etc. 
		<p>Default page shade is assumed to be white. 
		 */
		BROWSER
	}
	/**
	App surface contentStyle. 
	 */
	public static SurfaceStyle surfaceStyle;
	public static int facets;
	/** Time delay for drag input notifications, in milliseconds. */
	public static int dragNotifyPause=50;
	/**Should drag notification only take place when drag is paused? */
	public static boolean notifyInterim=false,dragNotifyWait=true;
	/**Title for standard application menu. */
	public static final String TITLE_CORE_MENU="Application",
		TITLE_FILE_MENU="File",TITLE_EDIT_MENU="Edit",
		TITLE_WINDOW_MENU="Window",TITLE_HELP_MENU="Help";
	public static final String KEY_TAB_CLOSE_LO="TabCloseButtonLo",
		KEY_TAB_CLOSE_HI="TabCloseButtonHi";
	/**
	Alternative name for null facet that can be used as divider for facet layouts. 
	<p>Interpreted as follows:
	<ul>
	 <li>in <code>rowPanel</code> methods, as a row break</li> 
	 <li>in (menu)<code>items</code> methods, as a menu divider</li> 
	</ul> 
	 */
	final public static KitFacet BREAK=new KitFacet(){
    public KWrap base(){return KWrap.BREAK;}
    public KWrap[]items(){return new KWrap[]{base()};}
    public void targetNotify(Object notice, boolean interim){}
    public void retarget(STarget target,Notifying.Impact impact){}
    public STarget target(){return STarget.NONE;}
    public String title(){return "BREAK";}
    public String toString(){return title();}
  },
  NO_FACET=BREAK;
  public final static Integer[]fontSizes=new Integer[5];
  private static final int fontSizeMidAt=fontSizes.length/2;
	public static int fontSizeAt=fontSizeMidAt;
	public static void fillFontSizes(int kitMidSize){
		for(int i=0;i<fontSizes.length;i++)fontSizes[i]=kitMidSize-fontSizeMidAt+i;
	}
	public static final SIndexing fontIndexing=new SIndexing("Font Size",fontSizes,
			fontSizeAt,new SIndexing.Coupler(){
		@Override
		public void indexSet(SIndexing i){
			AppValues.stateDebug.put(FONT_OFFSET,(fontSizeAt=i.index())-fontSizeMidAt);
		}
		public String[]iterationTitles(SIndexing i){
			return new String[]{FONT_SMALLER,FONT_LARGER};
		}
	});
	public static double fontFactor(){
		return(double)fontSizes[fontSizeAt]/fontSizes[fontSizeMidAt];
	}
	/**Flag constant to be passed to facet factory methods; may be concatenated
	 with others. */
	final public static String TREE_EXPAND="E&xpand",TREE_COLLAPSE="Coll&apse",
	HINT_NONE=FLAG_NONE,
	HINT_EXTRAS_PANE=newFlag("In graph pane"),  
	HINT_NO_FLASH=newFlag("No flash notifications"),  
	HINT_BARE=newFlag("Bare"),  
	HINT_TALL=newFlag("Tall"), 
	HINT_TEXT_FONT=newFlag("Text font"), 
	HINT_LABEL_AUTOCLEAR=newFlag("Auto-paint label on change"), 
	HINT_TITLE2=newFlag("Title 2"),
	HINT_TITLE3=newFlag("Title 3"),
	HINT_SQUARE=newFlag("Square"),
	HINT_SPREAD=newFlag("Spread"),
	HINT_GRID=newFlag("Grid"),
	HINT_DEBUG=newFlag("Debug"),
	HINT_USAGE_PANEL=newFlag("Panel usage"),
	HINT_USAGE_FORM=newFlag("Form usage"),
	HINT_PAINT_CACHE_DEEP=newFlag("Deep (and packed) painting cache"),
	HINT_NO_FOCUS=newFlag("No focus"),
	HINT_PANEL_CENTER=newFlag("Center-align panel"),
	HINT_PANEL_MIDDLE=newFlag("Middle-align panel"),
	HINT_PANEL_RIGHT=newFlag("Right-align panel or place to right of panel"),
	HINT_PANEL_BELOW=newFlag("Place below panel"),
	HINT_PANEL_ABOVE=newFlag("Place above panel"),
	HINT_PANEL_INSET=newFlag("Inset panel"),
	HINT_PANEL_BORDER=newFlag("Bordered panel"),
	HINT_NUMERIC_NUDGERS_FIRST=newFlag("Numeric nudgers first"),
	HINT_NUMERIC_UNGROUPED=newFlag("Numeric ungrouped"),
	HINT_NUMERIC_FIELDS=newFlag("Slider fields"),
	HINT_SLIDER_TICKS=newFlag("Slider ticks"),
	HINT_SLIDER_LABELS=newFlag("Slider labels"),
	HINT_SLIDER_LOCAL=newFlag("Slider local"), 
	HINT_SLIDER_FIELDS_TICKS_LABELS=
	 	HINT_NUMERIC_FIELDS+HINT_SLIDER_TICKS+HINT_SLIDER_LABELS,
	HINT_MENU_NO_MNEMONICS=newFlag("Menu with no mnemonics"),
	HINT_MENU_PRELOAD=newFlag("Preload menu to get key bindings"),
	HINT_INDEXING_SELECT=newFlag("Indexing with no selection"),
	HINT_TOOLTIPS=newFlag("Tooltips");
	private FacetFactory(Kit kit,FacetAppValues values,SurfaceStyle surfaceStyle, 
			Shade panelShade,boolean isCore){
		super(kit,surfaceStyle,panelShade);
		this.values=values;
		int offset=AppValues.stateDebug.getOrPutInt(FONT_OFFSET,0);
		fontSizeAt=fontSizes.length/2+offset;
		helpPages=new HelpPages(values);
		if(!isCore)return;
		Events.traceEvent(">Got surface builder with toolkit "+Debug.info(kit));
		kit.readDecorationValues(values.decorationValues(),values.nature(true));
		Events.traceEvent(">Read surface builder values");
	}
	/**
	Possible widget toolkits and look-and-feels. 
	 */
	public enum ToolkitType{
		SWING("Swing"),SWING_SYSTEM("SwingSystem"),SWT("SWT");
		private final String type;
		ToolkitType(String type){this.type=type;}
		public String toString(){return "kit"+type;}
	}
	/**
	{@link SHost} that can create and set an applet layout. 
	 */
	public interface AppletHost extends facets.core.superficial.app.SHost{
		/**
		Return a simple layout for use in applets. 
		@param content main client area
		@param toolPanel panel beside or below main area
		@param menus to appear in applet menu bar
		@param panelToSide determines the location of any tool panel: 
		if <code>true</code> to the right of the content, otherwise below. 
		 */
		FacetLayout newAppletLayout(SFacet content,SFacet toolPanel,SFacet[]menus,
				boolean panelToSide);
	}
	private final FacetAppValues values;
	private final HelpPages helpPages;
	private AreaFacets areas;
	private ProvidingCache providingCache;
	private static final class AppletCore extends FacetFactory {
		AppletCore(SurfaceStyle style,Shade panelShade,FacetAppValues values){
			super(new KitSwing(style==SurfaceStyle.APPLET,true),values,style,panelShade,true);
		}
	}
	private static final class DesktopCore extends FacetFactory{
		DesktopCore(Kit kit,FacetAppValues values){
			super(kit,values,SurfaceStyle.DESKTOP,panelShade,true);
		}
	}
	/**
	Creates a basic {@link FacetFactory} for use in a desktop application. 
	<p>Widgets will be constructed using a toolkit of the type specified by
	<code>values</code>; applications can use the builder returned as the			 
	core of a custom builder.    
	@param values define the widget toolkit/look-and-feel
	 */
	public static FacetFactory newDesktopCore(FacetAppValues values,ToolkitType type){
		Events.traceEvent(">Getting surface builder");
		Kit kit=type==ToolkitType.SWT?null // new KitSwt()
				:new KitSwing(type==ToolkitType.SWING_SYSTEM, false);
		if(kit==null)throw new IllegalStateException(
				"Invalid toolkit type in "+Debug.info(values));
		DesktopCore core=new DesktopCore(kit,values);
		return core;
	}
	/**
	Creates a basic {@link FacetFactory} for use in an applet.  
	<p>Widgets will be constructed using the Swing toolkit in a suitable
	look-and-feel.   
	@param surfaceStyle sets L&F
	@param panelShade sets panel shade for browser contentStyle
	@param values for use with {@link FacetAppSurface}
	 */
	public static FacetFactory newAppletCore(SurfaceStyle surfaceStyle, 
			Shade panelShade, FacetAppValues values){
		return new AppletCore(surfaceStyle,panelShade,values);
	}
	/**
	 Constructor for use by subclasses to be passed as {@link LayoutFeatures}. 
	 @param src the instance whose members will be shared
	 */
	protected FacetFactory(FacetFactory src){
		this(src.kit,src.values,FacetFactory.surfaceStyle,FacetFactory.panelShade,false);
		providingCache=src.providingCache;
	}
	/**
	Creates a Swing host for an applet-based application surface.  
	 <p>The {@link SHost} returned is guaranteed to wrap a 
	 {@link java.awt.Component}.
	 @param menuBar should be passed from the hosting applet.    
	 */
	final public AppletHost appletHostWrap(JMenuBar menuBar){
		if(!(this instanceof AppletCore))throw new RuntimeException(
				"Not implemented in "+Debug.info(this));
		else return((KitSwing)kit).appletHostWrap(this,menuBar);
	}
	final public AppHost windowHost(FacetAppSurface app){
		if(!(this instanceof DesktopCore))throw new RuntimeException(
				"Not implemented in "+Debug.info(this));
		else return kit.newWindowHost(app);
	}
	/**
	Returns a sub-factory for viewer, area and surface facets. 
	 */
	final public AreaFacets areas(){
		return areas==null?areas=new AreaFacets(this,values):areas;
	}
	/**
	Implements interface method. 
	@return <code>null</code> by default
	 */
	public SFacet[]menus(){
		return null;
	}
	/**
	Implements interface method. 
	@return <code>null</code> by default
	 */
	public SFacet toolbar(){
		return null;
	}
	/**
	Implements interface method. 
	@return <code>null</code> by default
	 */
	public SFacet sidebar(){
		return null;
	}
	/**
	Implements interface method. 
	@return <code>null</code> by default
	 */
	public SFacet status(){
		return null;
	}
	/**
	Implements interface method. 
	@return <code>null</code> by default
	 */
	public SFacet extras(){
		return null;
	}
	/**
	/**
	Implements interface method. 
	@return <code>null</code> by default
	 */
	public SurfaceServices services(){
		return null;
	}
	/**
	Creates standard menus to match an {@link AppSpecifier}. 
	@return menus possibly modified by {@link #adjustMenuRoots(MenuFacets[])}
	 */
	final protected SFacet[]newAdjustedMenus(FacetAppSurface app,SContentRootTargeter root){
		STargeter pane=findPaneTargeter(root);
		MenuFacets appMenu=new AppMenuBuilder(root,app.spec).newMenuFacets(),
			editMenu=!app.spec.canEditContent()?null:new EditFacets(root),
			paneMenu=pane==null||pane.elements().length==0?null
					:areas().new PaneFacets("Pane",pane),
			windowMenu=app.contentStyle==ContentStyle.SINGLE?null:windowMenuFacets(root,false),
			helpMenu=helpMenuFacets(root);
		List<SFacet>menus=new ArrayList();
		for(MenuFacets possible:adjustMenuRoots(new MenuFacets[]{
				appMenu,editMenu,paneMenu,windowMenu,helpMenu
		}))
			if(possible!=null)menus.add(menuRoot(possible));
		if(menus==null||menus.size()<1)throw new IllegalStateException(
				"Null or empty menus in "+Debug.info(this));
		else return menus.toArray(new SFacet[]{});
	}
	/**
	Retrieve a {@link STargeter} targeting the return of 
	{@link AreaFacets#panesGetTargets(SAreaTarget)}. 
	@return the appropriate {@link STargeter} or <code>null</code>
	 */
	protected STargeter findPaneTargeter(SContentRootTargeter root){
		return null;
	}
	/**
	Enables modification of standard menus. 
	@param menus created in {@link #newAdjustedMenus(FacetAppSurface, SContentRootTargeter)}
	@return by default <code>menus</code>
	 */
	protected MenuFacets[]adjustMenuRoots(MenuFacets[]menus){
		return menus;
	}
	/**
	Creates facets for an application edit menu. 
	<p>{@link #getFacets()} returns facets 
	retargeted on elements of {@link SContentRootTargeter#viewer()} 
	and indexed by {@link ViewerTarget#ELEMENTS_VIEWABLE}.   
	 */
	public class EditFacets extends MenuFacets{
		private final STargeter[]elements;
		public EditFacets(SContentRootTargeter t){
			super(TITLE_EDIT_MENU,t.viewer());
			elements=t.viewer().elements()[ViewerTarget.ELEMENTS_VIEWABLE].elements();
		}
		public SFacet[]getFacets(){
			int count=elements[0].elements().length==0?0:elements.length;
			final SFacet[]facets=new SFacet[count];
			for(int i=0;i<facets.length;i++)facets[i]=triggerMenuItems(elements[i],HINT_NONE);
			return facets;
		}
	}
	public final class AppTargeters{
		private final STargeter app[],appTargeter;
		public final STargeter news,file,fileSave,fileRevert;
		final AppSpecifier spec;
		public AppTargeters(SContentRootTargeter rootTargeter,AppSpecifier spec){
			this.spec=spec;
			appTargeter=(AreaTargeter)rootTargeter.monitor();
			app=appTargeter.elements();
			int elementCount=appTargeter.elements().length;
			if(elementCount==0)throw new IllegalStateException("No elements in "+Debug.info(this));
			news=appTargeter.elements()[AppActions.TARGETS_NEW];
			file=elementCount<=FacetAppActions.TARGETS_LAST+1?
					null:appTargeter.elements()[FileAppActions.TARGETS_FILE];
			boolean noSave=file==null||!spec.canSaveContent();
			fileSave=noSave?null:file.elements()[FileAppActions.TARGET_SAVE];
			fileRevert=noSave?null:file.elements()[FileAppActions.TARGET_REVERT];
		}
	}
	/**
	Creates {@link MenuFacets} for an application main menu. 
	 */
	public class AppMenuBuilder{
		private final SFacet newsItem,coreItems,fileItems,recentFiles;
		private final AppTargeters apps;
		public AppMenuBuilder(SContentRootTargeter rootTargeter,AppSpecifier spec){
			apps=new AppTargeters(rootTargeter,spec);
			STargeter news=apps.news;
			newsItem=!apps.spec.canCreateContent()?null:news.elements().length==1?
					triggerMenuItems(news,HINT_NONE):triggerMenu(news,HINT_TITLE2);
			coreItems=triggerMenuItems(apps.appTargeter.elements()[AppActions.TARGETS_CORE],
					HINT_NONE);
			STargeter fileElements=apps.file;
			fileItems=fileElements==null||fileElements.elements().length==0?null
				:triggerMenuItems(fileElements,HINT_NONE);
			recentFiles=fileItems==null?null
				:indexingRadioButtonMenuItems(apps.app[FileAppActions.TARGETS_RECENT],
	    		HINT_INDEXING_SELECT+HINT_TOOLTIPS);
		}
		/**
		Creates facets for incorporation in menu. 
		 */
		final public MenuFacets newMenuFacets(){
			return new MenuFacets(fileItems!=null?TITLE_FILE_MENU:TITLE_CORE_MENU,
					apps.appTargeter){
				public SFacet[]getFacets(){
					final SFacet[]facets=fileItems!=null?assembleFileFacets(
							newsItem,fileItems,recentFiles,coreItems)
						:assembleNonFileFacets(newsItem,coreItems);
					for(SFacet facet:facets)if(facet==null)throw new IllegalStateException(
							"Null facet in "+Debug.info(this));
					return facets;
				}
			};
		}
		protected SFacet[]assembleNonFileFacets(SFacet newsItem,SFacet coreItems){
			return newsItem==null?new SFacet[]{coreItems} 
				:new SFacet[]{newsItem,coreItems};
		}
		protected SFacet[]assembleFileFacets(SFacet newsItem,SFacet fileActions,
				SFacet recentFiles,SFacet coreItems){
			return join(
				newsItem!=null?new SFacet[]{newsItem}:new SFacet[]{},
			new SFacet[]{
				fileActions,
				BREAK,
				recentFiles,
				BREAK,
				coreItems,
			});
		}
	}
	/**
	Creates facets for an application window menu. 
	<p>The {@link MenuFacets} created by this method 
	returns a menu facet retargeted on the indexing of the surface root, 
	optionally appended to facets retargeted on elements defined  
	in {@link AppActions}.   
	@param rootTargeter is passed to the {@link MenuFacets} created 
	@param withLayout include layout sub-menu?
	 */
	final public MenuFacets windowMenuFacets(SContentRootTargeter rootTargeter, 
			boolean withLayout){
		final ItemList<SFacet>listFacets=new ItemList(SFacet.class);
	  STargeter appTargeter=(AreaTargeter)rootTargeter.monitor(),
		  appElements[]=appTargeter.elements(),
			window=appElements[AppActions.TARGETS_WINDOW];
	  for(STargeter e:window.elements()){
	  	listFacets.addItem(e.target()instanceof STrigger?
	  			triggerMenuItems(e,HINT_NONE)
	  			:togglingCheckboxMenuItems(e,HINT_NONE));
		}
	  if(withLayout)listFacets.addItem(
	  		togglingCheckboxMenu(appElements[FacetAppActions.TARGETS_BARS_LAYOUT],
					HINT_TITLE2));
		listFacets.addItem(BREAK);
		listFacets.addItem(indexingRadioButtonMenuItems(
				((IndexingTargeter)appTargeter).indexing(),HINT_NONE));			
		return new MenuFacets(TITLE_WINDOW_MENU,appTargeter){
			public SFacet[]getFacets(){
				return listFacets.items();
			}
		};
	}
	/**
	Creates facets for an application edit toolbar. 
	<p>Calls {@link #toolGroups(STargeter, String, SFacet...)} with facets
	retargeted on elements provided by {@link SContentRootTargeter#viewer()} 
	and indexed by {@link ViewerTarget#ELEMENTS_VIEWABLE}.   
	 */
	final public SFacet[]editTools(SContentRootTargeter t){
		STargeter viewer=t.viewer(),
			elements[]=viewer.elements()[ViewerTarget.ELEMENTS_VIEWABLE].elements();
		SFacet[]facets=new SFacet[elements.length];
		for(int i=0;i<facets.length;i++)facets[i]=triggerButtons(elements[i],HINT_BARE);
		return facets;
	}
	/**
	Creates facets for an application help menu. 
	<p>The {@link MenuFacets} created by this method 
	returns menu facets retargeted on elements defined  
	in {@link AppActions}.   
	@param rootTargeter is passed to the {@link MenuFacets} created. 
	 */
	final public MenuFacets helpMenuFacets(SContentRootTargeter rootTargeter){
		final STargeter appTargeter=(AreaTargeter)rootTargeter.monitor(), 
			elements[]=appTargeter.elements(
					)[AppActions.TARGETS_HELP].elements();
		final SFacet[]facets=new SFacet[elements.length];
		for(int i=0;i<facets.length;i++)
			facets[i]=elements[i].title().equals(AppConstants.TITLE_APP_HELP)?
				togglingCheckboxMenuItems(elements[i],HINT_NONE)
				:triggerMenuItems(elements[i],HINT_NONE);
		return new MenuFacets(TITLE_HELP_MENU,appTargeter){
			public SFacet[]getFacets(){
				return facets;
			}
		};
	}
	final public SFacet extras(final AreaTargeter targeter){
		final ValueNode stateApp=values.state(PATH_CORE_APP);
		final SFacet graph=new ExtrasGraph(kit,targeter,stateApp),
			help=new ExtrasHelp(helpPages,stateApp,this);
		if(!(targeter instanceof SContentRootTargeter))return graph;
		return newExtrasMount(targeter,stateApp,graph,help);
	}
	final public ClipperSource statefulClipperSource(boolean useSystemClipboard){
		return kit.statefulClipperSource(useSystemClipboard);
	}
	final public void warningCritical(String appTitle,Exception e,boolean inOpen){
		kit.warningCritical(new ExceptionTexts(appTitle,e),e,inOpen);		
	}
	public final void setAppProvidingCache(ProvidingCache cache){
		kit.setCache(providingCache=cache);
	}
	final public ProvidingCache providingCache(){
		if(providingCache==null)throw new IllegalStateException(
				"Null providingCache in "+Debug.info(this));
		return providingCache;
	}
	public static String newCountsText(){
		return "[Created widgets "+Kit.widgets+
		", facets "+FacetFactory.facets+
		", targeters "+TargeterCore.targeters+
		", targets "+TargetCore.targets+
		"]";
	}
	final class FacetList extends ArrayList<SFacet>{
		public SFacet[]facets(){
			return toArray(new SFacet[]{});
		}
		public boolean addAll(SFacet[]facets){
			return super.addAll(Arrays.asList(facets));
		}
	}}
/*/*
This file forms part of Version 0.8.79 of Facets, a framework demonstrating 
the Superficial approach to GUI-driven interactive applications.  
http://superficial.sourceforge.net 				
Copyright (C) 2011  David M Wright 
This library is free software; you can redistribute it and/or modify it under 
the terms of the GNU Lesser General Public License as published by the 
Free Software Foundation; either release 3 of the License, or (at your 
option) any later release.
This library is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR 
A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more 
details.
You should have received a copy of the GNU Lesser General Public License along 
with this library; if not, write to the Free Software Foundation, Inc., 
59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

