package facets.facet;
import static facets.facet.AreaFacets.*;
import static facets.facet.FacetFactory.*;
import static facets.util.Debug.*;
import facets.core.superficial.Facetable;
import facets.core.superficial.FacetedTarget;
import facets.core.superficial.SFacet;
import facets.core.superficial.SNumeric;
import facets.core.superficial.STarget;
import facets.core.superficial.SToggling;
import facets.core.superficial.Notifying.Impact;
import facets.core.superficial.app.SAreaTarget;
import facets.facet.toolkit.KMount;
import facets.facet.toolkit.KWrap;
import facets.facet.toolkit.Kit;
import facets.util.Debug;
import facets.util.ItemList;
import facets.util.Objects;
import facets.util.StringFlags;
import java.util.ArrayList;
import java.util.List;
final class PaneSet extends AreaMount{
	final STarget targets;
	final PaneSetLayout layout;
	final private ChildPane rootPane=new ChildPane();		
	final private FacetCore[]mounts;
	private SFacet maximisedActive=NO_FACET;
	private int splitAt,mountAt;
	PaneSet(Kit kit,SFacet[]contents,PaneSetLayout layout){
		super(layout.area,kit);
		if(contents==null||contents.length<2)throw new IllegalArgumentException(
				"Null, empty or single contents in "+Debug.info(this));	
		else for(SFacet c:contents)if(c instanceof PaneSet)
				throw new IllegalArgumentException(info(c)+" not nestable inside "+info(this));
		mounts=Objects.newTyped(AreaMountCore.class,contents);
		targets=(this.layout=layout).buildPaneTargets(this);
		applyLayout();
	}
	@Override
	protected void traceOutput(String msg){
		if(false)super.traceOutput(msg);
	}
	void applyLayout(){
		mountAt=0;
		ChildPane[]rootChildren=null;
		for(int rootAt=0;rootAt<layout.codesTree.length;rootAt++){
			ChildPane pane=rootChildren==null?rootPane:rootChildren[rootAt-1],
					children[]=null;
			pane.dispose();
			boolean addTabs=rootAt>0&&layout.tabs.togglings[rootAt-1].isSet();
			int codes[]=layout.codesTree[rootAt],codeMounts=1+(codes.length+1)/2;
			trace(".applyLayout: rootAt="+rootAt+" addTabs="+addTabs+
					" codes="+codes.length+" codeMounts="+codeMounts);
			for(int codeAt=0,codeMountAt=0;mountAt<mounts.length
						&&(addTabs?codeMountAt<codeMounts:codeAt<codes.length);
					codeAt+=2,codeMountAt++,mountAt+=rootAt==0?0:1){
				if(false)trace(".applyLayout: codeMountAt="+codeMountAt+" codeAt="+codeAt +" mountAt="+mountAt);
				if(addTabs)rootChildren[rootAt-1].addTab();
				else{
					if(children!=null)pane=children[codes[codeAt-1]];
					else if(rootAt>0)mountAt++;
					children=pane.divide(codes[codeAt]);
				}
			}
			if(rootChildren==null)rootChildren=children;
		}
		splitAt=layout.splits.length-1;
		mountAt=0;
		rootPane.layOutMounts();
		SToggling[]shows=layout.shows.togglings;
		SFacet active=null;
		for(int i=0;active==null&&i<mounts.length;i++){
			SFacet mount=mounts[i];
			if(((SAreaTarget)mount.target()).isActive()&&shows[i].isSet())
				active=mount;
		}
		for(int i=0;active==null&&i<mounts.length;i++)if(shows[i].isSet())
				((SAreaTarget)(active=mounts[i]).target()).ensureActive(false);
		if(active==null)throw new IllegalStateException(
				"Null active mount in "+Debug.info(this));
		((Facetable)target).retargetFacets(Impact.DEFAULT);
		layout.panesLaidOut(maximisedActive!=NO_FACET);
	}
	void setMaximisedActive(boolean on){
		maximisedActive=NO_FACET;
		if(on){
			for(FacetCore mount:mounts)
				if(((SAreaTarget)mount.target()).isActive())maximisedActive=mount;
			if(maximisedActive==NO_FACET)throw new IllegalStateException(
					"Null active in "+Debug.info(this));
		}
		applyLayout();
	}
	final private class ChildPane{
		final KMount base=false?kit.packedMount(PaneSet.this,StringFlags.EMPTY)
				:kit.spreadMount(PaneSet.this,StringFlags.EMPTY);
		private final List<Integer>tabMountAts=new ArrayList();
		private ChildPane child0,child1;
		private int splitCode;
		ChildPane[]divide(int splitCode){
			this.splitCode=splitCode;
			if(false)trace(".divide: mountAt=",mountAt);
			return new ChildPane[]{child0=new ChildPane(),child1=new ChildPane()};
		}
		void addTab(){
			tabMountAts.add(mountAt);
			if(false)trace(".addTab: mountAt=",mountAt);
		}
		boolean layOutMounts(){
		  boolean showMe=false;
		  int tabCount=tabMountAts.size();
			SToggling[]shows=layout.shows.togglings;
			if(tabCount>0){
		  	ItemList<FacetCore>areas=new ItemList(FacetCore.class);
		  	if(false)trace(".layOutMounts: tabMountAts=",tabMountAts);
		  	for(Integer tabMountAt:tabMountAts){
					FacetCore mount=mounts[tabMountAt];
			  	boolean showMount=shows[tabMountAt].isSet()
			  		&&(maximisedActive==NO_FACET||maximisedActive==mount);
			  	FacetedTarget area=((SAreaTarget)mount.target()).activeFacet();
			  	area.setLive(showMount);
			  	mountAt++;
			  	showMe|=showMount;
			  	if(!showMount)continue;
					areas.add(mount);
			  }
		  	splitAt-=(tabCount+1)/2;
		  	if(showMe)base.setItem(kit.areaTabs(areas.items(),StringFlags.EMPTY));
		  }
		  else if(child0==null){
				FacetCore mount=mounts[mountAt];
				if(false)trace(".layOutMounts: mountAt=",mountAt);
		  	showMe=shows[mountAt].isSet()
		  		&&(maximisedActive==NO_FACET||maximisedActive==mount);
		  	FacetedTarget area=((SAreaTarget)mount.target()).activeFacet();
				area.setLive(showMe);
				mountAt++;
				if(showMe)base.setItem(mount.base());
		  }
			else{
				boolean show0=child0.layOutMounts(),show1=child1.layOutMounts();
				showMe=show0|show1;
				boolean wide=splitCode!=PANE_SPLIT_VERTICAL^(layout.flip==PANE_SPLIT_VERTICAL);
				SNumeric split=layout.splits[splitAt--];
				if(showMe){
					KMount mount=kit.splitMount(PaneSet.this,wide,split);
					base.setItem(mount);
					List<KWrap>showWraps=new ArrayList();
					if(show0)showWraps.add(child0.base);
					if(show1)showWraps.add(child1.base);
					mount.setItems(showWraps.toArray(new KWrap[]{}));
				}
			}
		  return showMe;
		}
		void dispose(){
			if(child0!=null){
				child0.dispose();child1.dispose();
				child0=null;child1=null;
			}
			base.setItem(null);
		}
	}
	protected KWrap lazyBase(){
		return rootPane.base;
	}
}/*/*
This file forms part of Version 0.8.79 of Facets, a framework demonstrating 
the Superficial approach to GUI-driven interactive applications.  
http://superficial.sourceforge.net 				
Copyright (C) 2011  David M Wright 
This library is free software; you can redistribute it and/or modify it under 
the terms of the GNU Lesser General Public License as published by the 
Free Software Foundation; either release 3 of the License, or (at your 
option) any later release.
This library is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR 
A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more 
details.
You should have received a copy of the GNU Lesser General Public License along 
with this library; if not, write to the Free Software Foundation, Inc., 
59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/
