package facets.facet;
import static facets.facet.FacetFactory.*;
import facets.core.superficial.SFacet;
import facets.core.superficial.STargeter;
import facets.core.superficial.app.SAreaTarget;
import facets.core.superficial.app.ViewerTarget;
import facets.facet.FacetMaster.Viewer;
import facets.facet.toolkit.swing.ViewerMaster;
import facets.util.StringFlags;
/**
	Parameter object for viewer creation methods in {@link AreaFacets}. 
	<p>{@link ViewerAreaMaster} has two roles. It 
<ul>
	<li>encapsulates parameters required when
	creating viewer facets and attaching them to the area target tree. 
	<li>can supply a custom {@link ViewerMaster}
<li>can supply a closure that creates a local viewer toolbar.
	</ul>	 
	*/
	public class ViewerAreaMaster{
		public final StringFlags hints(){
			return new StringFlags(hintString());
		}
		/**
		May return custom {@link FacetMaster.Viewer}. 
		<p>Default returns <code>null</code>, signalling that type be determined by 
		the surface builder based on view. 
		<p>Non-<code>null</code> returns should be of the 
	 extension class for the builder.
		 */
		public Viewer viewerMaster(){
			return null;
		}
		/**
		Return concatenated hint strings. 
		<p>Default returns {@link FacetFactory#HINT_NONE}. 
		 */
		protected String hintString(){
			return HINT_NONE;
		}
		/**
		May return custom local toolbar.
		@param viewTargeter targeting the {@link ViewerTarget#viewFrame()}
		@return <code>null</code> by default
		 */
		protected SFacet newViewerTools(STargeter viewTargeter){
			return null;
		}
		final ViewerAreaMaster[]childMasters(SAreaTarget area){
			final ViewerAreaMaster parent=this;
		  SAreaTarget[]areaChildren=(SAreaTarget[])area.indexableTargets();
		  ViewerAreaMaster[]childMasters=new ViewerAreaMaster[areaChildren.length];
		  for(int i=0;i<areaChildren.length;i++) {
		  	ViewerAreaMaster master=newChildMaster(areaChildren[i]);
		  	childMasters[i]=master!=null?master:
					new ViewerAreaMaster(){
						public Viewer viewerMaster(){
							return parent.viewerMaster();
						}
						protected String hintString(){
							return parent.hintString();
						}
						protected SFacet newViewerTools(STargeter viewTargeter){
							return parent.newViewerTools(viewTargeter);
						}
			  	};
		  }
			return childMasters;
		}
		/**
		May return specific master for the area passed. 
		<p>Default returns <code>null</code>.
		 */
		protected ViewerAreaMaster newChildMaster(SAreaTarget area){
			return null;
		}
	}/*/*
This file forms part of Version 0.8.79 of Facets, a framework demonstrating 
the Superficial approach to GUI-driven interactive applications.  
http://superficial.sourceforge.net 				
Copyright (C) 2011  David M Wright 
This library is free software; you can redistribute it and/or modify it under 
the terms of the GNU Lesser General Public License as published by the 
Free Software Foundation; either release 3 of the License, or (at your 
option) any later release.
This library is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR 
A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more 
details.
You should have received a copy of the GNU Lesser General Public License along 
with this library; if not, write to the Free Software Foundation, Inc., 
59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/
