package facets.facet;
import facets.core.superficial.Notice;
import facets.core.superficial.Notifiable;
import facets.core.superficial.Notifying;
import facets.core.superficial.SFacet;
import facets.core.superficial.SFrameTarget;
import facets.core.superficial.STarget;
import facets.core.superficial.STargeter;
import facets.core.superficial.Notifying.Impact;
import facets.core.superficial.app.SSelection;
import facets.core.superficial.app.SView;
import facets.core.superficial.app.ViewerTarget;
import facets.facet.FacetMaster.Viewer;
import facets.util.Debug;
/**
Builds and manages a form inside a {@link Viewer}. 
 */
public abstract class ViewerPaneForm{
	private Viewer pane;
	private STargeter targeter;
	private Object stateCheck;
	private final Notifying notifier=new Notifying() {
		private Notifiable monitor;
		final public void notify(Notice notice){
			ViewerTarget viewer=pane.viewerTarget();
			viewer.ensureActive(false);
			Object content=((SFrameTarget)targeter.target()).framed;
			if(stateChecksEqual(stateCheck,newContentStateCheck(content)))
				viewer.selectionChanged(newNotifiedSelection(content,notice));
			else viewer.selectionEdited(null,newNotifiedEdit(notice), false);
		}

		public void setMonitor(Notifiable monitor){
			this.monitor=monitor;			
		}
		public void notifyMonitor(Notifying.Impact impact){
			if(monitor!=null)monitor.notify(new Notice(this,impact));			
		}
		public Notifiable monitor(){
			if(monitor==null)
				throw new IllegalStateException("Null monitor in "+Debug.info(this));
			return monitor;
		}
		public Notifying.Impact impact(){
			return Notifying.Impact.DEFAULT;
		}
	};
	final public void attachPaneMaster(Viewer pane){
		if(pane==null)
			throw new IllegalArgumentException("Null pane in "+Debug.info(this));
		else if(this.pane!=null)throw new IllegalArgumentException(
				"Change of immutable pane in "+Debug.info(this));
		else this.pane=pane;
		notifier.setMonitor(pane.viewerTarget());
	}
	final public SFacet newRetargetedFormFacet(){
		SFrameTarget frameTarget=newSelectionFrame();
		targeter=frameTarget.newTargeter();
		targeter.setMonitor(notifier);
		targeter.retarget(frameTarget,Notifying.Impact.DEFAULT);
		SFacet facet=newFormFacet(targeter);
		return facet;
	}
	final public void refreshFacets(){
		SFrameTarget frameTarget=newSelectionFrame();
		targeter.retarget(frameTarget,Notifying.Impact.DEFAULT);
		targeter.retargetFacets(Notifying.Impact.DEFAULT);
		stateCheck=newContentStateCheck(frameTarget.framed);
	}
	protected abstract SFrameTarget newSelectionFrame();
	protected boolean elementNeedsSelectionFocus(SSelection selection,
			STarget element){
		return false;
	}
	protected Viewer pane(){
		if(pane==null) throw new IllegalStateException("Null pane in "+Debug.info(this));
		return pane;
	}
	protected abstract STarget[] lazySelectionElements(SFrameTarget frameTarget,
			SView view,SSelection selection);
	protected abstract SFacet newFormFacet(STargeter targeter);
	protected abstract Object newContentStateCheck(Object content);
	protected abstract boolean stateChecksEqual(Object then,Object now);
	protected abstract SSelection newNotifiedSelection(Object content,
			Notice notice);
	protected abstract Object newNotifiedEdit(Notice notice);
}/*/*
This file forms part of Version 0.8.79 of Facets, a framework demonstrating 
the Superficial approach to GUI-driven interactive applications.  
http://superficial.sourceforge.net 				
Copyright (C) 2011  David M Wright 
This library is free software; you can redistribute it and/or modify it under 
the terms of the GNU Lesser General Public License as published by the 
Free Software Foundation; either release 3 of the License, or (at your 
option) any later release.
This library is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR 
A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more 
details.
You should have received a copy of the GNU Lesser General Public License along 
with this library; if not, write to the Free Software Foundation, Inc., 
59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/
