package facets.facet.app;
import static facets.core.app.Dialogs.Response.*;
import static facets.util.app.AppValues.*;
import facets.core.app.ActionAppSurface;
import facets.core.app.AppActions;
import facets.core.app.Dialogs;
import facets.core.app.Dialogs.ExceptionTexts;
import facets.core.superficial.STarget;
import facets.core.superficial.SToggling;
import facets.core.superficial.TargetCore;
import facets.util.Debug;
import facets.util.Objects;
import facets.util.app.WatchableOperation;
import facets.util.app.AppValues;
import facets.util.app.WatchableOperation.CancelStyle;
import facets.util.app.AppWatcher.WatcherCoupler;
import facets.util.datatree.ValueNode;
import java.util.List;
/**
{@link AppActions} for a {@link FacetAppSurface}. 
<p>{@link FacetAppActions} extends its superclass by implementing abstract actions 
in terms of {@link FacetAppSurface}. 
 */
public class FacetAppActions extends AppActions{
	public static final String[]TEXTS_DEFAULT={
		"Creating Content",
		//Cancellable Operation 
		"Press Esc or Cancel to cancel this operation.",
		//Press Esc or Cancel to cancel this operation.
		"Cancel Requested",
		//Operation Cancelled
		"If you cancel this operation, $appTitle may  may freeze or close suddenly. " +
		"<br>Wait for it to complete?",
	};
	public static final int 
		TARGETS_BARS_LAYOUT=AppActions.TARGETS_LAST+1,
		TARGETS_LAST=TARGETS_BARS_LAYOUT;
	public static final String KEY_HIDE_TOOLBAR="hideToolbar",
		KEY_HIDE_SIDEBAR="hideSidebar",KEY_HIDE_STATUS="hideStatus",
		ARG_PREFERENCES="launchPreferences";
	/**
	Overrides superclass method. 
	<p>Adds the {@link SToggling}s returned by
	 {@link #useBarLayoutTargets(STarget, STarget, STarget)}
	 */
	protected STarget[]newSurfaceRootElements(){
		STarget[]superTargets=super.newSurfaceRootElements();
		final ValueNode stateApp=app.values.state(PATH_CORE_APP);
		STarget tools=new SToggling("Show Toolbar|Toolbar",
				!stateApp.getBoolean(KEY_HIDE_TOOLBAR),
				new SToggling.Coupler(){
			public void stateSet(SToggling t){
				stateApp.put(KEY_HIDE_TOOLBAR,!t.isSet());
				updateLayout();
			}
		}),
		side=new SToggling("Show S&idebar|S&idebar",
				!stateApp.getBoolean(KEY_HIDE_SIDEBAR),
				new SToggling.Coupler(){
			public void stateSet(SToggling t){
				stateApp.put(KEY_HIDE_SIDEBAR,!t.isSet());
				updateLayout();
			}
		}),
		status=new SToggling("Show St&atus Bar|St&atus Bar",
				!stateApp.getBoolean(KEY_HIDE_STATUS),
				new SToggling.Coupler(){
			public void stateSet(SToggling t){
				stateApp.put(KEY_HIDE_STATUS,!t.isSet());
				updateLayout();
			}
		});
		return TargetCore.join(superTargets,new STarget[]{
				new TargetCore("Layout",useBarLayoutTargets(tools,side,status))});
	}
	final protected void attemptClose(){
		((WindowAppHost)app.host()).requestAppClose();
	}
	/**
	Define which targets should be exposed by {@link SToggling}s in the surface. 
	<p>Indexed by {@link #TARGETS_BARS_LAYOUT}
	 */
	protected STarget[]useBarLayoutTargets(STarget tools,STarget side,
			STarget status){
		return new STarget[]{tools,side,status};
	}
	/**
	Implements abstract method.
	<p>Delegates finally to {@link FacetAppValues#newPreferences(ActionAppSurface) )}  
	 */
	@Override
	final protected StatePreferences lazyPreferences(){
		if(app==null)throw new IllegalStateException("App not attached in "+Debug.info(this));
		else return((FacetAppValues)app.values).newPreferences(app);
	}
}
/*/*
This file forms part of Version 0.8.79 of Facets, a framework demonstrating 
the Superficial approach to GUI-driven interactive applications.  
http://superficial.sourceforge.net 				
Copyright (C) 2011  David M Wright 
This library is free software; you can redistribute it and/or modify it under 
the terms of the GNU Lesser General Public License as published by the 
Free Software Foundation; either release 3 of the License, or (at your 
option) any later release.
This library is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR 
A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more 
details.
You should have received a copy of the GNU Lesser General Public License along 
with this library; if not, write to the Free Software Foundation, Inc., 
59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

