package facets.facet.app;
import static facets.core.app.AppActions.*;
import static facets.core.app.AppConstants.*;
import static facets.core.app.AppSurface.ContentStyle.*;
import static facets.core.app.Dialogs.Response.*;
import static facets.facet.FacetFactory.*;
import static facets.facet.app.FacetAppActions.*;
import static facets.facet.app.FacetAppValues.*;
import static facets.util.Debug.*;
import static facets.util.app.AppValues.*;
import static facets.util.app.AppWatcher.*;
import static java.lang.System.*;
import static javax.swing.SwingUtilities.*;
import facets.core.app.ActionAppSurface;
import facets.core.app.AppHost;
import facets.core.app.AppSurface;
import facets.core.app.Dialogs;
import facets.core.app.MenuFacets;
import facets.core.app.MountFacet;
import facets.core.app.SurfaceServices;
import facets.core.app.ViewerContenter;
import facets.core.superficial.FacetedTarget;
import facets.core.superficial.Notice;
import facets.core.superficial.Notifying;
import facets.core.superficial.Notifying.Impact;
import facets.core.superficial.SFacet;
import facets.core.superficial.SIndexing;
import facets.core.superficial.STarget;
import facets.core.superficial.STargeter;
import facets.core.superficial.STextual;
import facets.core.superficial.STextual.Coupler;
import facets.core.superficial.app.SAreaTarget;
import facets.core.superficial.app.SContentRootTargeter;
import facets.core.superficial.app.SContenter;
import facets.core.superficial.app.SHost;
import facets.core.superficial.app.SHost.FacetLayout;
import facets.facet.FacetFactory;
import facets.facet.FacetFactory.AppMenuBuilder;
import facets.facet.FacetFactory.ToolkitType;
import facets.util.Debug;
import facets.util.FileSpecifier;
import facets.util.Times;
import facets.util.app.AppFileValues;
import facets.util.app.WatchableOperation;
import facets.util.app.AppServices;
import facets.util.app.AppWatcher;
import facets.util.app.Events;
import facets.util.app.ProvidingCache.ItemProvider;
import facets.util.datatree.ValueNode;
import java.io.File;
import java.util.ArrayList;
import java.util.List;
/** 
{@link ActionAppSurface} with a built-in {@link FacetFactory}. 
<p>{@link FacetAppSurface} extends its superclass primarily by 
	passing the {@link FacetAppSpecifier} which is set as {@link AppSurface#spec};
	this in turn supplies a {@link FacetFactory} which can either be used as-is or 
	provide a core for content-specific subclasses.
	<p>Further general-purpose functionality
	is suppled by {@link FacetAppValues} and {@link FacetAppActions} members of 
	 {@link FacetAppSpecifier}.  
 */
public abstract class FacetAppSurface extends ActionAppSurface implements SurfaceServices{
	/**
	For use by subclasses and {@link facets.core.app.AppSurface.Contenter}s 
	 */
	public final FacetFactory ff;
	private final List<SIndexing>debugIndexings=new ArrayList();
	private final STargeter debugText;
	private SHost host;
	private Dialogs dialogs;
	final protected void traceOutput(String msg){
		msg=FacetAppSurface.class.getSimpleName()+msg;
		if(true)System.out.println(msg);
		else if(false)Times.printElapsed(msg);
	}
	/**
	Unique constructor. 
	 @param ff set as {@link #ff} after being passed to 
	 constructor as {@link AppServices}
	 */
	public FacetAppSurface(FacetAppSpecifier spec,FacetFactory ff){
		super(spec,ff);
		this.ff=ff;
		STextual textual=new STextual(title()+".debugText","",new Coupler());
		debugText=textual.newTargeter();
		debugText.retarget(textual,Impact.DEFAULT);
	}
/**
	Overrides superclass implementation. 
<ol>
	<li>Creates or gets the {@link AppHost} and obtains from it a 
	suitable {@link Dialogs}. 
	<li>Issues any {@link AppWatcher} warnings. 
	<li>Possibly puts up a splash panel and calls the super-implementation.
	<li>If applet-hosted, calls {@link #appOpened()} and returns
	<li>Otherwise opens app window, attempts to add content to an empty desktop,
	pulls down any splash panel, calls {@link #appOpened()}. 
	</ol>	
	 */
	final public void openApp(){
		Events.traceEvent(">Getting host for "+info(this));
	  final AppHost host=(AppHost)host();
	  dialogs=host.newDialogs(this);
	  Events.traceEvent(">Got host "+info(host));
		final FacetAppValues values=(FacetAppValues)this.values;
	  final ValueNode nature=values.nature(true);
		if(watcher!=null&&contentStyle!=DESKTOP)values.checkTimeouts(debugWatch());
		final boolean inWindow=host instanceof WindowAppHost;
		if(inWindow&&values.state(PATH_CORE_APP).getBoolean(KEY_SPLASH))
			((WindowAppHost)host).splashUp("Opening "+title()+" - Facets " +version()+"...");
		final boolean preferences=nature.getBoolean(ARG_PREFERENCES);
		if(preferences)nature.put(KEY_SHOW_PREFERENCES,true);
		super.openApp();
	  if(nature.getBoolean(NATURE_ENSURE_FIRST_ROOT_ACTIVE)){
			((FacetedTarget)((SAreaTarget)surfaceRootTargeter().area(AREA_APP).target()
  			).indexableTargets()[0]).ensureActive(true);
			notify(null);
		}
		if(!inWindow){
			appOpened();
	  	return;
	  }
		host.setTitle(newTitleBarText());
		((WindowAppHost)host).openWindow();
		hostWindowOpened();
		final boolean inSwt=((FacetAppSpecifier)spec).toolkitType()==ToolkitType.SWT;
		WatchableOperation finishOpen=new WatchableOperation("FacetedAppSurface.finishOpen"){
		public void doSimpleOperation(){
			if(findActiveContent()==emptyContent
					&&!nature.getBoolean(NATURE_OPEN_EMPTY)){
				if(actions instanceof FileAppActions){
					beforeContent();
					File file=null;
					if(((FileAppActions)actions).values().recentFiles().length==0)
						actions.launchAbout();
					else file=actions.getOpeningContentSourceFile();
					if(file!=null&&file.isFile())addContent(file);
				}
				else{
					beforeContent();
					for(Object s:getFixedOpeningContentSources())addContent(s);
				}
			}
			else beforeContent();
			Runnable launchDialog=new Runnable(){public void run(){
				if(preferences){
					if(true)actions.launchPreferences();
					else actions.launchAbout();
				}
			}};
			if(inSwt)launchDialog.run();
			else invokeLater(launchDialog);
			appOpened();
		}
		private void beforeContent(){
			((WindowAppHost)host).splashDown();
			if(watcher!=null)values.checkTimeouts(debugWatch());
		}};
		if(false||inSwt)finishOpen.doSimpleOperation();
		else runWatchedLater(finishOpen);
	}
	protected boolean debugWatch(){
		return false;
	}
	final public void debugWatch(final String callName,final boolean providing,
			final boolean exception){
		if(!Debug.natureDebug)return;
		final ItemProvider op=new ItemProvider(ff.providingCache(),this,
				"debugWatch."+(providing?"providing.":"")+callName){
			@Override
			public CancelStyle cancelStyle(){
				return providing?CancelStyle.Timeout:CancelStyle.None;
			}
			@Override
			protected Object newItem(){
				boolean runOn=!exception;
				long startAt=currentTimeMillis(),
					breakAt=startAt+(runOn?timeoutBlock*200:timeoutBlock/2),thenAt=startAt,nowAt;
				while(true)if((nowAt=currentTimeMillis())-thenAt>(runOn?2000:500)){
					if(false)System.out.println(this+" " +(nowAt-startAt)/1000+
							" providing="+providing+" throwException="+exception);
					thenAt=nowAt;
					if(nowAt<breakAt)continue;
					else if(exception)throw new RuntimeException(callName+" "+this);
					else break;
				}
				return null;
			}
			@Override
			protected long advanceByteCount(){
				return 0;
			}
		};
		if(providing)op.getForValues(this);
		else runWatched(op);
	}
	final public void runWatched(WatchableOperation r){
		if(watcher!=null)watcher.runWatched(r);
		else r.doSimpleOperation();
	}
	final public void runWatchedLater(final WatchableOperation r){
		invokeLater(new Runnable(){public void run(){runWatched(r);}});
	}
	protected CachingStyle cachingStyle(){
		return CachingStyle.checkItemCount;
	}
	protected long calculateCacheMb(int maxSafeMb){
		return 2*(long)Math.sqrt(maxSafeMb);
	}
	final protected MountFacet newMultiContentFacet(SAreaTarget surfaceRoot){
		boolean desktopRoot=contentStyle==ContentStyle.DESKTOP;
		return ff.areas().appMultiContentFacet(this,surfaceRoot,desktopRoot);
	}
	final public void notify(final Notice notice){
		if(notice==null)throw new IllegalArgumentException("Null notice in "+Debug.info(this));
		else if(notice.impact==Impact.FLASH){
			notifiedFlash(notice);
			return;
		}
		int switchAt=values.state(PATH_CORE_APP).getBoolean(KEY_DEBUG_STATUS)?1:0;
		for(SIndexing each:debugIndexings)each.setIndex(switchAt);
		((STextual)debugText.target()).setText(newDebugText());
		debugText.retargetFacets(Impact.DEFAULT);
		super.notify(notice);
		host().setTitle(newTitleBarText());
	}
	/**
	Called from {@link #notify(Notice)} when the {@link Notice} passed
	has {@link Impact#FLASH}. 
	<p>No other response is made to such notifications. 
	<p>Default...   
	 */
	protected void notifiedFlash(Notice notice){
		if(true||findActiveContent()==emptyContent)return;
		STargeter selection=activeContentRootTargeter().selection();
		((STextual)selection.elements()[0].target()).setText(
				notice.flashText());
		selection.retargetFacets(notice.impact);
	}
	final public SFacet newDebugSwitchLabel(SFacet client){
		MountFacet mount=ff.switchMount(title()+".debugSwitchLabel");
		mount.setFacets(client,ff.textualLabel(debugText,HINT_NONE));
		debugIndexings.add(FacetFactory.switchMountIndexing(mount,new SIndexing.Coupler()));
		return mount;
	}
	public final String areaTitle(int depth){
		return surfaceRootTargeter().area(depth).title();
	}
	/**
	Check to see if a non-applet app can be closed. 
	@param host should call if a request to close is made by the app or the toolkit window
	 */
	public final boolean closeAcceptable(WindowAppHost host){
		ValueNode nature=values.nature(true);
		boolean canClose=!(actions instanceof FileAppActions)
			||((FileAppActions)actions).appCloseAcceptable();
		canClose&=!nature.getBoolean(NATURE_CONFIRM_CLOSE)
				||dialogs().confirmOKCancel(TITLE_CLOSE_REQUESTED,RUBRIC_CONFIRM_CLOSE)==Ok;
		if(!canClose)return false;
		host.storeCheckedBounds();
		String stateSave=NATURE_CONFIRM_STATE_SAVE;
		if(!nature.getBoolean(stateSave)||dialogs().confirmYesNo(stateSave,stateSave)!=Yes)
			values.tryWrite("");
		appClosing();
		return true;
	}
	/**
	Supplies a simple implementation of {@link SurfaceServices}. 
	<p>This method is intended for use in implementations of
	{@link facets.core.app.AppSurface.Contenter#newContentLayout(SHost, SFacet, 
	SContentRootTargeter)},
	to meet the contract of layout creation methods such as 
	{@link AppHost#newDesktopLayout(SFacet, facets.core.app.AppHost.LayoutFeatures)}.
	<p>It provides a fine example of the closure-like use of anonymous classes, 
	in that a hidden reference to the enclosing {@link FacetAppSurface} is 
	 passed to the {@link SurfaceServices} returned. 
	When this is returned by {@link AppHost#activeServices()}
	in {@link #handleInvalidInput(STarget, Object)}, code can be executed
	using the reference even 
	though the {@link SurfaceServices} was created from within a contenter 
	and returned by the host. 
	@param contextFacets will ultimately be returned by 
	{@link #getContextMenuFacets()}
	 */
	final public SurfaceServices newSimpleServices(final MenuFacets contextFacets){
		return new SurfaceServices(){
			public MenuFacets getContextMenuFacets(){
				return contextFacets;
			}
			final public void handleInvalidInput(STarget target,Object input){
				String text="Invalid input " +input+" for "+target.title();
				STargeter status=null;
				if(status==null||values.state(PATH_CORE_APP
						).getBoolean(FacetAppActions.KEY_HIDE_STATUS))
					dialogs().errorMessage(title(),text);
				else{
					((STextual)status.target()).setText(text);
					status.retargetFacets(Notifying.Impact.DEFAULT);
				}
			}
		};
	}
	/**
	Implements interface method. 
	<p>To ensure the request is met by the active {@link SContenter},
	delegates to the {@link SurfaceServices} returned by 
	{@link AppHost#activeServices()} in {@link #host()}.  
	 */
	final public MenuFacets getContextMenuFacets(){
		SurfaceServices services=((AppHost)host()).activeServices();
		return services==null?null:services.getContextMenuFacets();
	}
	/**
	Implements interface method. 
	<p>To ensure the request is met by the active {@link SContenter},
	delegates to the {@link SurfaceServices} returned by 
	{@link AppHost#activeServices()} in {@link #host()}.  
	 */
	final public void handleInvalidInput(STarget target,Object input){
		((AppHost)host()).activeServices().handleInvalidInput(target,input);
	}
	/**
	The {@link Dialogs} created during {@link #openApp()}. 
	 */
	public final Dialogs dialogs(){
		if(dialogs==null)throw new IllegalStateException("Null dialogs in "+info(this));
		return dialogs;
	}
	/**
	Re-implementation to create/return a suitable {@link SHost}. 
	@return unless {@link #isHeadless()} returns <code>true</code>,
	the return of {@link FacetFactory#windowHost(FacetAppSurface)}
	 */
	public SHost host(){
		return host==null?(host=isHeadless()?newHeadlessHost():ff.windowHost(this))
			:host;
	}
	final public void setLayoutTargetsLive(boolean live){
	  surfaceRootTargeter().elements()[TARGETS_BARS_LAYOUT].target().setLive(live);
	}
	protected FacetLayout newEmptyDesktopLayout(SHost host,
			final SContentRootTargeter t,SFacet surfaceRootFacet){
		return((AppHost)host).newDesktopLayout(surfaceRootFacet,new FacetFactory(ff){
			public SFacet[]menus(){
				return new SFacet[]{
					menuRoot(new AppMenuBuilder(t,spec).newMenuFacets()),
					menuRoot(helpMenuFacets(t))
				};
			}
			public SFacet status(){
				return toolGroups(t,HINT_NONE, newDebugSwitchLabel(spacerWide(0)));
			}
			public SFacet extras(){
				return extras(t);
			}
		});
	}
	/**
	Implements abstract method. 
	<p>Default is invalid stub. 
	 */
	protected Object getInternalContentSource(){
		throw new RuntimeException("Not implemented in "+Debug.info(this));
	}
	/**
	What files can the application open?
	<p>Called from {@link AppFileValues#getOpenSpecifiers()} in {@link FileAppActions} 
	and typically from {@link ViewerContenter#sinkFileSpecifiers()};
	default returns {@link FileSpecifier#ALL}. 
	 */
	public FileSpecifier[]getFileSpecifiers(){
		return new FileSpecifier[]{FileSpecifier.ALL};
	}
	/**
	Implements abstract method. 
	<p>Default adjusts title bar to shown save warning. 
	 */
	protected String newTitleBarText(){
		String area=areaTitle(AppSurface.AREA_CONTENT);
		if(area.equals(TITLE_EMPTY))area="No Content";
		return area+(spec.canEditContent()&&findActiveContent().hasChanged()?"*":"")
			+" - "+title();
	}
	final protected String newDebugText(){
		return memory!=null?("Memory"+memory.newStatusText(false))
				:newCountsText();
	}
	/**
	Called from {@link #openApp()}. 
	<p>Default is empty stub. 
	 */
	protected void hostWindowOpened(){}
	/**
	Called from {@link #openApp()}. 
	<p>Default is empty stub. 
	 */
	protected void appOpened(){}
	/**
	Called before a <code>true</code> return from {@link #closeAcceptable(WindowAppHost)}. 
	<p>Default is empty stub. 
	 */
	protected void appClosing(){}
}
/*/*
This file forms part of Version 0.8.79 of Facets, a framework demonstrating 
the Superficial approach to GUI-driven interactive applications.  
http://superficial.sourceforge.net 				
Copyright (C) 2011  David M Wright 
This library is free software; you can redistribute it and/or modify it under 
the terms of the GNU Lesser General Public License as published by the 
Free Software Foundation; either release 3 of the License, or (at your 
option) any later release.
This library is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR 
A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more 
details.
You should have received a copy of the GNU Lesser General Public License along 
with this library; if not, write to the Free Software Foundation, Inc., 
59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

