package facets.facet.app;
import static facets.core.app.ActionAppSurface.*;
import static facets.core.app.AppConstants.*;
import static facets.core.app.avatar.PainterMaster.Textual.*;
import static facets.facet.FacetFactory.*;
import static facets.facet.app.FacetConstants.*;
import static facets.util.Debug.*;
import static facets.util.Times.*;
import static facets.util.app.AppWatcher.*;
import static facets.util.app.Events.*;
import static facets.util.datatree.Nodes.*;
import facets.core.app.ActionAppSurface;
import facets.core.app.ActionAppSurface.ActionAppValues;
import facets.core.app.AppActions;
import facets.core.app.AppActions.StatePreferences;
import facets.core.app.DialogSurface;
import facets.core.app.DialogSurface.Contenter;
import facets.core.app.Dialogs;
import facets.core.app.HtmlView;
import facets.core.app.TreeView;
import facets.util.Debug;
import facets.util.Objects;
import facets.util.app.AppValues;
import facets.util.app.Events;
import facets.util.datatree.ValueNode;
/**
{@link AppValues} for a {@link FacetAppSurface}.
 */
public class FacetAppValues extends ActionAppValues{
	/** Key for debug value. */
	final public static String 
		KEY_SPLASH="splashScreen",
		KEY_GRAPH_BUILD="graphPaneBuild",
		KEY_GRAPH_SPLIT="graphPaneSplit",
		KEY_GRAPH_OFFSETS="graphPaneOffsets",
		KEY_GRAPH_WHERE="graphPaneWhere",
		KEY_GRAPH_FIND="graphPaneFind",
		KEY_DEBUG_STATUS="debugStatus",
		KEY_DRAG_WAIT="dragNotifyPause",
		KEY_DRAG_PAUSE="dragNotifyWait",
		KEY_DRAG_NOTIFY="dragNotifyInterim",
		KEY_TEXT_ALIASING="textAntiAlias";
	public FacetAppValues(Class titleClass){
		super(titleClass);
	}
	final public static int DRAG_WAIT_MIN=10;
	/**
	Return values keyed to application titles and keys. 
	@return by default {@link FacetConstants#DECORATION_VALUES}
	 */
	public Object[][]decorationValues(){
		return DECORATION_VALUES;
	}
	protected void addNatureDefaults(ValueNode root){
		super.addNatureDefaults(root);
		mergeContents((ValueNode)child(root,PATH_CORE),new Object[]{
			NATURE_WRITABLE+"="+false,
			NATURE_RECORD_RUNS+"="+false,
			NATURE_DEBUG+"="+true,
			NATURE_SIZE_MIN+"=400,200",
			NATURE_SIZE_DEFAULT+"=600,600",
			NATURE_CONFIRM_CLOSE+"="+false,
			NATURE_OPEN_EMPTY+"="+false,
			NATURE_ICON_PATH+"=_image/icon",
			NATURE_STORE_PATH+"=__image/icon",
			NATURE_ICON_AWARE+"="+true,
			NATURE_RUN_WATCHED+"="+!Debug.natureDebug,
			NATURE_ENSURE_FIRST_ROOT_ACTIVE+"="+false,
		});
	}
	/**
	Overrides superclass method.
	<p>Fills {@link AppValues#stateDebug} with defaults for {@link Events}. 
	 */
	protected void addStateDefaults(ValueNode root){
		super.addStateDefaults(root);
		stateDebug.setContents(new Object[]{
				KEY_TRACE+"="+trace,
				KEY_EVENTS+"="+events,
				KEY_BUILD+"="+buildTimes,
				KEY_TIMES+"="+false,
				KEY_MEM+"="+false,
				KEY_ESTIMATE+"="+5000,
				KEY_TIMES_RESET+"="+1000,
				DEFAULT_FILTERS
			});
		mergeContents((ValueNode)descendantTyped(root,PATH_APP),new Object[]{
				KEY_SPLASH+"="+false,
				KEY_DEBUG_STATUS+"="+false
			}
		);
	}
	/**
	Overrides superclass method. 
	 */
	final public void adjustClassValues(){
		super.adjustClassValues();
		ValueNode debug=state(PATH_CORE_DEBUG);
		trace=debug.getBoolean(KEY_TRACE);
		events=debug.getBoolean(KEY_EVENTS);
		buildTimes=debug.getBoolean(KEY_BUILD);
		times=trace&&debug.getBoolean(KEY_TIMES);
		memCheck=trace&&debug.getBoolean(KEY_MEM);
		resetWait=debug.getOrPutInt(KEY_TIMES_RESET,(int)resetWait);
		timeoutBlock=debug.getOrPutInt(KEY_TIMEOUT,timeoutBlock);	
		systemTimeout=debug.getOrPutInt(KEY_TIMEOUT_SYSTEM,systemTimeout);	
		graphBuild=debug.getOrPutBoolean(KEY_GRAPH_BUILD,graphBuild);
		graphShowWhere=debug.getOrPutInt(KEY_GRAPH_WHERE,graphShowWhere);	
		notifyInterim=debug.getOrPutBoolean(KEY_DRAG_NOTIFY,notifyInterim);
		dragNotifyWait=debug.getOrPutBoolean(KEY_DRAG_WAIT,dragNotifyWait);
		dragNotifyPause=debug.getOrPutInt(KEY_DRAG_PAUSE,dragNotifyPause);	
		antiAlias=debug.getOrPutBoolean(KEY_TEXT_ALIASING,antiAlias);
		TreeView.debug=debug.getOrPutBoolean(TreeView.KEY_DEBUG,TreeView.debug);
		HtmlView.showSource=debug.getOrPutBoolean(HtmlView.KEY_SOURCE,HtmlView.showSource);
		traceEvent(">Read nature and state values");
	}
	/**
	Delegate for {@link FacetAppActions#lazyPreferences()}.
	@return by default a basic {@link FacetPreferences}; client code will usually return
	a trivial subclass. 
	 */
	public StatePreferences newPreferences(ActionAppSurface app){
		return new FacetPreferences((FacetAppSurface)app){
			@Override
			public Contenter[]newContenters(){
				Contenter[]contenters=super.newContenters();
				return true?contenters:new DialogSurface.Contenter[]{
						contenters[PREFERENCES_TRACE],
						/*PREFERENCES_VALUES  PREFERENCES_VIEW
						PREFERENCES_GRAPH PREFERENCES_FEATURES 
						PREFERENCES_DRAG  */
				};
			}
		};
	}
	public void checkTimeouts(boolean debugWatch){
		ValueNode stateApp=state(PATH_CORE_APP);
		final Dialogs dialogs=app.dialogs();
		if(stateApp.getBoolean(KEY_TIMED_OUT)){
			dialogs.infoMessage(KEY_TIMED_OUT,KEY_TIMED_OUT);
			stateApp.put(KEY_TIMED_OUT,false);
			if(false)stateApp.put(KEY_TIMEOUTS,true);
		}
		if(!debugWatch&&stateApp.getBoolean(KEY_TIMEOUTS)
				&&dialogs.warningYesNo(KEY_TIMEOUTS,KEY_TIMEOUTS)==Dialogs.Response.No)
			stateApp.put(KEY_TIMEOUTS,false);
	}
	public static Object[][]joinValues(Object[][]head,Object[][]tail){
		return Objects.join(Object[].class,head,tail);
	}
	/* 
	Reads value pairs from the console and writes them to XML. 
	<p>Output files are <i>MyAppValues.[nature|state].xml</i>; 
	implementations of {@link #addNatureDefaults(ValueNode)} and
	{@link #addStateDefaults(ValueNode)} demonstrate techniques for 
	constructing and modifying {@link ValueNode}s. 
	@param args read by {@link #readValues(String[])}
	 */
	static void main(String[]args)throws Exception{
		AppValues values=new FacetAppValues(FacetAppValues.class){
			protected void traceOutput(String msg){
				traceOutputWithClass(": "+msg);
			}
		};
		values.readValues(args);
		values.tryWrite("");
	}
}/*/*
This file forms part of Version 0.8.79 of Facets, a framework demonstrating 
the Superficial approach to GUI-driven interactive applications.  
http://superficial.sourceforge.net 				
Copyright (C) 2011  David M Wright 
This library is free software; you can redistribute it and/or modify it under 
the terms of the GNU Lesser General Public License as published by the 
Free Software Foundation; either release 3 of the License, or (at your 
option) any later release.
This library is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR 
A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more 
details.
You should have received a copy of the GNU Lesser General Public License along 
with this library; if not, write to the Free Software Foundation, Inc., 
59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/
