package facets.facet.app;
import static facets.facet.FacetFactory.*;
import static facets.facet.app.FacetAppActions.*;
import static facets.facet.app.FacetAppValues.*;
import static facets.util.app.AppValues.*;
import static facets.util.app.Events.*;
import facets.core.app.ActionAppSurface;
import facets.core.app.AppActions.StatePreferences;
import facets.core.app.AppSurface.ContentStyle;
import facets.core.app.DialogSurface;
import facets.core.app.DialogSurface.Contenter;
import facets.core.app.Dialogs.Surfacer;
import facets.core.app.HtmlView;
import facets.core.app.TreeView;
import facets.core.superficial.Notice;
import facets.core.superficial.Notifying;
import facets.core.superficial.SFacet;
import facets.core.superficial.SIndexing;
import facets.core.superficial.SNumeric;
import facets.core.superficial.STarget;
import facets.core.superficial.STargeter;
import facets.core.superficial.STextual;
import facets.core.superficial.STextual.Coupler;
import facets.core.superficial.SToggling;
import facets.core.superficial.STrigger;
import facets.core.superficial.app.SContentRootTargeter;
import facets.facet.FacetFactory;
import facets.facet.ValueDialogContenter;
import facets.util.NumberPolicy;
import facets.util.app.AppValues;
import facets.util.datatree.Nodes;
import facets.util.datatree.ValueNode;
import java.awt.Dimension;
/**
{@link StatePreferences} for a {@link FacetAppSurface}. 
<p>The default implementations {@link FacetPreferences} of
 */
public abstract class FacetPreferences implements StatePreferences{
	public static final String TITLE_START=false?"":FacetPreferences.class.getSimpleName(), 
		TITLE_TRACE=TITLE_START+"Trace",TITLE_VIEW=TITLE_START+"View",
		TITLE_FEATURES=TITLE_START+"Features",TITLE_GRAPH=TITLE_START+"Graph",
		TITLE_DRAG=TITLE_START+"Targeting",TITLE_VALUES=TITLE_START+"Values",
		TITLE_HEADER=TITLE_START+"Debug",TITLE_TIMES="Times Or Events";
	/**
	Indexes into {@link Contenter}s created by default. 
	 */
	final public static int 
	PREFERENCES_VALUES=0,
	PREFERENCES_TRACE=1,
	PREFERENCES_GRAPH=2,
	PREFERENCES_VIEW=3,
	PREFERENCES_FEATURES=4,
	PREFERENCES_DRAG=5;
	public final FacetAppSurface app;
	public FacetPreferences(FacetAppSurface app){
		this.app=app;
	}
	/**
	Implements abstract method.
	Creates {@link Contenter}s exposing framework session values.
	@return by default {@link Contenter}s indexed as described above;
	clients will usually override trivially to return appropriate {@link Contenter}s
	 */
	public Contenter[]newContenters(){
		ValueNode debug=AppValues.stateDebug,copyDebug=(ValueNode)debug.copyState(),
			stateApp=app.values.state(PATH_CORE_APP),copyApp=(ValueNode)stateApp.copyState();
		return new DialogSurface.Contenter[]{
				new AppValuesContenter(TITLE_VALUES,app),
				new Trace(app,debug,copyDebug),
				new Graph(app,debug,copyDebug),
				new View(app,debug,copyDebug),
				new Features(TITLE_FEATURES,app,stateApp,copyApp),
				new Drag(app,debug,copyDebug),
		};
	}
	/**
	Implements abstract method.
	@return by default a {@link Surfacer} created with 
	{@link FacetDialogSurface#newDefaultTabbedSurfacer()};
	clients may reimplement to create an appropriate {@link DialogSurface} to match
	{@link #newContenters()}  
	 */
	public Surfacer newSurfacer(){
		return FacetDialogSurface.newDefaultTabbedSurfacer();
	}
	static class Trace extends ValueDialogContenter{
		private static final int TARGET_TRACE=0,TARGET_TIMES=1,
			TARGET_FILTERS=2,TARGET_RESET=3;
		private final STarget eventFilters;
		private final SIndexing eventsOrBuildOrTimesOrMem;
		private final SToggling trace;
		private final SNumeric timesReset;
		private final NumberPolicy timesResetPolicy=new NumberPolicy(
				50,10000){
			public int format(){
				return FORMAT_DECIMALS_0;
			}
			public int columns(){
				return 4;
			}
		};
		public Dimension contentAreaSize(){
			return new Dimension(235,360);
		}
		Trace(FacetAppSurface app,ValueNode master,ValueNode working){
			super(TITLE_TRACE,app,master,working);
			trace=content.newToggling(working,KEY_TRACE);
			eventsOrBuildOrTimesOrMem=content.newIndexing(working,TITLE_TIMES,
				new String[]{KEY_EVENTS,KEY_BUILD,KEY_TIMES,KEY_MEM},
				working.getBoolean(KEY_BUILD)?KEY_BUILD
				:working.getBoolean(KEY_TIMES)?KEY_TIMES
				:working.getBoolean(KEY_MEM)?KEY_MEM
				:KEY_EVENTS);
			eventFilters=content.newMultipleIndexing((ValueNode)Nodes.child(working,
					KEY_FILTERS),KEY_FILTERS,FILTERS,true);
			timesReset=content.newNumeric(working,KEY_TIMES_RESET,timesResetPolicy);
		}
		protected STarget[]lazyContentRootElements(){
			return new STarget[]{trace,eventsOrBuildOrTimesOrMem,eventFilters,timesReset};
		}
		protected PanelFacets newPanelFacets(FacetFactory core){
			return new PanelFacets(core){
				public SFacet newContentPanel(SContentRootTargeter t){
					STargeter[]elements=t.elements();
					return rowPanel(t,0,5,HINT_PANEL_INSET,new SFacet[]{
						togglingCheckboxes(elements[TARGET_TRACE],HINT_BARE),
						BREAK,spacerWide(10),
						indexingRadioButtonSingle(elements[TARGET_TIMES],0,
								HINT_BARE),
						BREAK,spacerWide(20),
						indexingPaneChecked(elements[TARGET_FILTERS],200,6,
								HINT_TEXT_FONT+HINT_TALL),
						BREAK,spacerWide(10),
						indexingRadioButtonSingle(elements[TARGET_TIMES],1,HINT_BARE),
						BREAK,spacerWide(10),
						indexingRadioButtonSingle(elements[TARGET_TIMES],2,HINT_BARE),
						BREAK,spacerWide(20),
						numericFields(elements[TARGET_RESET],HINT_NUMERIC_UNGROUPED),
						BREAK,spacerWide(10),
						indexingRadioButtonSingle(elements[TARGET_TIMES],3,HINT_BARE),
						BREAK,fill()
					});
				}
			};
		}
		protected void targetValuesUpdated(STarget target,ValueNode values,String keys){
			if(target==eventsOrBuildOrTimesOrMem){
				int index=((SIndexing)target).index();
				values.put(KEY_EVENTS,index==0);
				values.put(KEY_BUILD,index==1);
				values.put(KEY_TIMES,index==2);
				values.put(KEY_MEM,index==3);
			}
		}
		protected void updateLivesOnRetargeted(ValueNode working){
			boolean trace=working.getBoolean(KEY_TRACE),
				events=eventsOrBuildOrTimesOrMem.index()==0;
			eventsOrBuildOrTimesOrMem.setLive(trace);
			eventFilters.setLive(trace&events);
			timesReset.setLive(trace&&!events);
		}
	}
	static class View extends ValueDialogContenter{
		private static final int TARGET_TREE=0,TARGET_HTML=1,TARGET_ALIASING=2,TARGET_FONT=3;
		private final SToggling treeDebug,htmlSource,aliasing;
		public Dimension contentAreaSize(){
			return new Dimension(260,105);
		}
		View(final FacetAppSurface app,ValueNode master,final ValueNode working){
			super(TITLE_VIEW,app,master,working);
			treeDebug=content.newToggling(working,TreeView.KEY_DEBUG);
			htmlSource=content.newToggling(working,HtmlView.KEY_SOURCE);
			aliasing=content.newToggling(working,KEY_TEXT_ALIASING);
			aliasing.setLive(false);
		}
		protected void targetValuesUpdated(STarget target,ValueNode values,String keys){}
		protected STarget[]lazyContentRootElements(){
			return new STarget[]{treeDebug,htmlSource,aliasing,fontIndexing};
		}
		protected void updateLivesOnRetargeted(ValueNode working){}
		protected PanelFacets newPanelFacets(FacetFactory core){
			return new PanelFacets(core){
				public SFacet newContentPanel(SContentRootTargeter t){
					STargeter[]elements=t.elements();
					return rowPanel(t,0,5,HINT_PANEL_INSET,new SFacet[]{
						togglingCheckboxes(elements[TARGET_TREE],HINT_BARE),
						BREAK,
						togglingCheckboxes(elements[TARGET_HTML],HINT_BARE),
						BREAK,
						indexingDropdownList(elements[TARGET_FONT],HINT_NONE),
						BREAK,
						togglingCheckboxes(elements[TARGET_ALIASING],HINT_BARE),
						BREAK,fill()
					});
				}
			};
		}
	}
	static class Graph extends ValueDialogContenter{
		private static final Object[]WHERES={"None","A&pplication","Dialogs"};
		private static final int TARGET_BUILD=0,TARGET_WHERE=1,
			TARGET_FIND_FIELD=2,TARGET_FIND_GO=3;
		private final SToggling build;
		private final SIndexing where;
		private final STextual findField;
		private final STrigger findGo;
		private DialogSurface dialog;
		public Dimension contentAreaSize(){
			return new Dimension(310,150);
		}
		Graph(final FacetAppSurface app,ValueNode master,final ValueNode working){
			super(TITLE_GRAPH,app,master,working);
			build=content.newToggling(working,KEY_GRAPH_BUILD);
			build.setLive(false);
			where=content.newIndexing(working,KEY_GRAPH_WHERE,
					WHERES,WHERES[graphShowWhere]);
			String stateFind=working.getString(KEY_GRAPH_FIND);
			findField=new STextual(KEY_GRAPH_FIND,
					findGraphValue=stateFind.equals("")?findGraphValue:stateFind,
					new Coupler(){
				public boolean updateInterim(STextual t){
					graphFindNow=false;
					return true;
				}
				public void textSet(STextual t){
					storeFind(working);
				}
			});
			findGo=new STrigger("F&ind Next",new STrigger.Coupler(){
				public void fired(STrigger t){
					storeFind(working);
					graphFindNow=true;
					app.notify(new Notice(t,Notifying.Impact.DEFAULT));
					graphFindNow=false;
				}
			});
		}
		protected STarget[]lazyContentRootElements(){
			return new STarget[]{build,where,findField,findGo};
		}
		private void storeFind(final ValueNode copy){
			String fieldFind=findField.text().trim();
			copy.put(KEY_GRAPH_FIND,
					findGraphValue=fieldFind.equals("")?GRAPH_FIND_NONE:fieldFind);
		}
		protected void updateLivesOnRetargeted(ValueNode working){		
			boolean graphs=true?build.isSet():working.getBoolean(KEY_GRAPH_BUILD);
			if(!graphs)where.setIndex(0);
			where.setLive(graphs);
			findField.setLive(graphShowWhere!=GRAPH_NONE&&where.isLive());
			findGo.setLive(findField.isLive()
					&&!findField.text().trim().equals(""));
		}
		protected PanelFacets newPanelFacets(FacetFactory core){
			return new PanelFacets(core){
				public SFacet newContentPanel(SContentRootTargeter t){
					STargeter[]elements=t.elements();
					return rowPanel(t,0,5,HINT_PANEL_INSET,new SFacet[]{
						togglingCheckboxes(elements[TARGET_BUILD],HINT_BARE),
						BREAK,spacerWide(10),
						indexingRadioButtons(elements[TARGET_WHERE],HINT_TALL),
						BREAK,spacerWide(30),
						rowPanel(t,5,0,HINT_PANEL_MIDDLE,
							textualField(elements[TARGET_FIND_FIELD],15,HINT_NONE),
							triggerButtons(elements[TARGET_FIND_GO],HINT_BARE)),
						BREAK,fill()
					});
				}
			};
		}
		protected void targetValuesUpdated(STarget target,ValueNode values,String keys){
			if(target==where)values.put(KEY_GRAPH_WHERE,((SIndexing)target).index());
		}
		public void applyChanges(){
			super.applyChanges();
			dialog.host().updateLayout(app);
		}
		public void setDialog(DialogSurface dialog){
			this.dialog=dialog;
		}
	}
	static class Features extends ValueDialogContenter{
		private static final int TARGET_STATUS=0,TARGET_SPLASH=1,TARGET_TIMEOUTS=2;
		private final SToggling status,splash,timeouts;
		private final ValueNode values,copy;
		public Dimension contentAreaSize(){
			return new Dimension(175,85);
		}
		Features(String title,FacetAppSurface app,ValueNode master,
				ValueNode working){
			super(title,app,master,working);
			this.values=master;
			this.copy=working;
			status=content.newToggling(working,KEY_DEBUG_STATUS);
			splash=content.newToggling(working,KEY_SPLASH);
			timeouts=content.newToggling(working,ActionAppSurface.KEY_TIMEOUTS+"|"+TITLE_FEATURES);
			status.setLive(!master.getBoolean(KEY_HIDE_STATUS));
			splash.setLive(app.contentStyle!=ContentStyle.DESKTOP);
		}
		Contenter asHeader(){
			return new Features(TITLE_HEADER,app,values,copy);
		}
		protected STarget[]lazyContentRootElements(){
			return new STarget[]{status,splash,timeouts};
		}
		protected PanelFacets newPanelFacets(FacetFactory core){
			return new PanelFacets(core){
				public SFacet newContentPanel(SContentRootTargeter t){
					STargeter[]elements=t.elements();
					return rowPanel(t,0,10,HINT_PANEL_INSET,
						togglingCheckboxes(elements[TARGET_STATUS],HINT_BARE),BREAK,
						togglingCheckboxes(elements[TARGET_SPLASH],HINT_BARE),BREAK,
						togglingCheckboxes(elements[TARGET_TIMEOUTS],HINT_BARE),BREAK,
						fill());
				}
			};
		}
		protected void targetValuesUpdated(STarget target,ValueNode values,String keys){}
		protected void updateLivesOnRetargeted(ValueNode working){}
	}
	static class Drag extends ValueDialogContenter{
		protected static final int TARGET_INTERIM=0,TARGET_PAUSE=1,TARGET_WAIT=2;
		private final NumberPolicy dragWaitPolicy=new NumberPolicy(
				DRAG_WAIT_MIN,10000){
			public int format(){
				return FORMAT_DECIMALS_0;
			}
			public int columns(){
				return 4;
			}
		};
		private final SNumeric dragWait;
		private final SToggling dragPause,notifyInterim;
		@Override
		public Dimension contentAreaSize(){
		return new Dimension(170,80);
		}
		Drag(FacetAppSurface app,ValueNode master,ValueNode working){
			super(TITLE_DRAG,app,master,working);
			notifyInterim=content.newToggling(working,KEY_DRAG_NOTIFY);
			dragPause=content.newToggling(working,KEY_DRAG_WAIT);
			dragWait=content.newNumeric(working,KEY_DRAG_PAUSE,dragWaitPolicy);
			updateLivesOnRetargeted(null);
		}
		protected STarget[]lazyContentRootElements(){
			return new STarget[]{notifyInterim,dragPause,dragWait};
		}
		protected PanelFacets newPanelFacets(FacetFactory core){
			return new PanelFacets(core){
				public SFacet newContentPanel(SContentRootTargeter t){
					STargeter[]elements=t.elements();
					return rowPanel(t,0,5,HINT_PANEL_INSET,
						togglingCheckboxes(elements[TARGET_INTERIM],HINT_BARE),
						BREAK,
						spacerWide(5),
						togglingCheckboxes(elements[TARGET_PAUSE],HINT_BARE),
						BREAK,
						spacerWide(15),
						numericFields(elements[TARGET_WAIT],HINT_NUMERIC_UNGROUPED),
						BREAK,
						fill());
				}
			};
		}
		protected void targetValuesUpdated(STarget target,ValueNode values,String keys){}
		protected void updateLivesOnRetargeted(ValueNode working){
			boolean interim = notifyInterim.isSet();
			dragPause.setLive(interim);
			dragWait.setLive(interim&&dragPause.isSet());
		}		
	}
}/*/*
This file forms part of Version 0.8.79 of Facets, a framework demonstrating 
the Superficial approach to GUI-driven interactive applications.  
http://superficial.sourceforge.net 				
Copyright (C) 2011  David M Wright 
This library is free software; you can redistribute it and/or modify it under 
the terms of the GNU Lesser General Public License as published by the 
Free Software Foundation; either release 3 of the License, or (at your 
option) any later release.
This library is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR 
A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more 
details.
You should have received a copy of the GNU Lesser General Public License along 
with this library; if not, write to the Free Software Foundation, Inc., 
59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/
