package facets.facet.app;
import static facets.core.app.AppConstants.*;
import static facets.core.app.AppSurface.ContentStyle.*;
import static facets.core.app.Dialogs.Response.*;
import facets.core.app.ActionAppSurface;
import facets.core.app.AppSurface.Contenter;
import facets.core.app.Dialogs;
import facets.core.app.Dialogs.Response;
import facets.core.app.ViewerContenter;
import facets.core.superficial.SIndexing;
import facets.core.superficial.SIndexing.Coupler;
import facets.core.superficial.STarget;
import facets.core.superficial.STrigger;
import facets.core.superficial.TargetCore;
import facets.core.superficial.app.SAreaTarget;
import facets.core.superficial.app.SHost;
import facets.util.Debug;
import facets.util.FileSpecifier;
import facets.util.ItemList;
import facets.util.app.AppFileValues;
import facets.util.app.WatchableOperation;
import java.io.File;
import java.io.IOException;
import java.util.List;
/**
{@link FacetAppActions} for an application that reads and writes files. 
<p>{@link FileAppActions} extends its superclass with a practical API for
using files as content sources and sinks.  
 */
public class FileAppActions extends FacetAppActions{
	private static final int FIRST=FacetAppActions.TARGETS_LAST+1;
	/** Index into file {@link STarget}s added in {@link #newSurfaceRootElements()}.  */
	public static final int 
		TARGETS_FILE=FIRST,
		TARGET_SAVE=1,TARGET_REVERT=3,
		TARGETS_RECENT=FIRST+1,
		TARGETS_LAST=TARGETS_RECENT;
	private final Coupler recentCoupler=new SIndexing.Coupler(){
		@Override
		public Object[]getIndexables(){
			File[]recentFiles=values.recentFiles();
			return recentFiles.length>0?recentFiles:SIndexing.NO_INDEXABLES;
		}
		@Override
		public void indexSet(SIndexing i){
			fileOpen((File)i.indexed());
		}
		@Override
		public String[]newIndexableTitles(SIndexing x){
			File[]recentFiles=values.recentFiles();
			int recentCount=recentFiles.length;
			x.setLive(x.isLive()&&recentCount>0);
			if(recentCount==0)return new String[]{"No recent files"};
			String[]titles=new String[recentCount];
			for(int i=0;i<titles.length;i++)
				titles[i]=(i+1)+" "+recentFiles[i].getName()+"|"+recentFiles[i].getPath();
			return titles;
		}
	};
	private final STarget open=new STrigger(TITLE_FILE_OPEN,new STrigger.Coupler(){
	  public void fired(STrigger t){
	  	fileOpen(null);
	  }
	}),				
	save=new STrigger(TITLE_FILE_SAVE,new STrigger.Coupler(){
	  public void fired(STrigger t){
			fileSave((ViewerContenter)app.findActiveContent());
		}
	}),				
	saveAs=new STrigger(TITLE_FILE_SAVE_AS,new STrigger.Coupler(){
	  public void fired(STrigger t){fileSaveAs();}
	}),				
	revert=new STrigger(TITLE_FILE_REVERT,new STrigger.Coupler(){
	  public void fired(STrigger t){revertContent();}
	}),
	fileMultiple[]={
		new STrigger(TITLE_FILE_CLOSE,new STrigger.Coupler(){
		  public void fired(STrigger t){fileClose();}
		}),				
		new STrigger(TITLE_FILE_CLOSEALL,new STrigger.Coupler(){
		  public void fired(STrigger t){fileCloseAll();}
		}),
	},
	recent=new SIndexing(STATE_NODE_RECENT,recentCoupler);
	private AppFileValues values;
	/**
	Adds target groups to super implementation. 
	<p>Targeter elements can be accessed using the following additional indices:
	<ul>
	<li>{@link #TARGETS_FILE} - {@link #fileOpen(File)} etc; 
	<li>{@link #TARGETS_RECENT} - recent files list
	</ul>
	<p>The contents of {@link #TARGETS_FILE} are determined by 
	{@link ActionAppSurface#spec}
	 */
	@Override
	protected STarget[]newSurfaceRootElements(){
		values.updateRecentFiles(null);
		if(!app.values.hasSystemAccess())throw new IllegalStateException(
				"No system access for "+Debug.info(app));
		ItemList fileTop=new ItemList(STarget.class);
		fileTop.addItem(open);
		if(app.spec.canSaveContent()){
			fileTop.addItem(save);
			fileTop.addItem(saveAs);
		}
		if(app.spec.canEditContent())fileTop.addItem(revert);
		if(app.contentStyle!=SINGLE)fileTop.addItems(fileMultiple);
		return TargetCore.join(super.newSurfaceRootElements(),new STarget[]{
			new TargetCore("File Menu",(STarget[])fileTop.items()),recent});
	}
	/**
	Re-implementation adjusting file menu live states. 
	 */
	@Override
	final protected void surfaceRetargeted(){
		Contenter content=app.findActiveContent();
		boolean empty=content==app.emptyContent;
		STarget[]elements=app.surfaceRootTargeter().target().elements(
				)[TARGETS_FILE].elements();
		for(int i=1;i<elements.length;i++)elements[i].setLive(!empty);
		boolean contentChanged=content instanceof ViewerContenter
			&&((ViewerContenter)content).hasChanged();
		save.setLive(contentChanged);
		revert.setLive(contentChanged);
	}
	/**
	Re-implementation of superclass method. 
	<p>Depending on nature values, usually launches file chooser dialog. 
	 */
	@Override
	public File getOpeningContentSourceFile(){
		return values.getOpeningFile(app.contentStyle!=DESKTOP);
	}
	private void fileOpen(final File open){
		WatchableOperation op=new WatchableOperation("FileAppActions.fileOpen"){
			public void doSimpleOperation(){
				File file=open;
				if(file==null||!file.isFile())
					file=app.dialogs().openFile(values.getOpenSpecifiers());
				if(file==null)return;
				values.updateRecentFiles(file);
				recent.setLive(true);
				SAreaTarget root=app.firstContentRoot(file);
				if(root!=null){
					root.ensureActive(true);
					return;
				}
				if(app.contentStyle==SINGLE){
					if(contentIsRemovable(TITLE_CLOSE_CONTENT,app.findActiveContent())!=Yes)
						return;
					app.replaceSingleContent(file);
				}
				else app.addContent(file);
			}
		};
		if(true)app.runWatched(op);
		else op.doSimpleOperation();
	}
	private void fileSaveAs(){
		ViewerContenter active=(ViewerContenter)app.findActiveContent();
		Object sink=active.sink();
		Dialogs dialogs=app.dialogs();
		List<ViewerContenter>contents=app.getViewerContents();
		FileSpecifier[]specifiers=active.sinkFileSpecifiers();
		File saveFile=sink instanceof File?(File)sink
				:specifiers[specifiers.length-1].specifiedFile(sink);
		while((saveFile=dialogs.saveFile(saveFile,specifiers))!=null){
			boolean badSink=false;
			for(ViewerContenter c:contents){
				Object checkSink=c.sink();
				if(checkSink==sink){
					if(badSink=!active.setSink(saveFile))
						dialogs.infoMessage("Can't Save","Cannot save as " +saveFile);
				}
				else if(checkSink instanceof File&&
						(badSink=((File)checkSink).equals(saveFile))){
					dialogs.infoMessage("Can't Save","This file is already open.");
					break;
				}
			}
			if(!badSink)break;
		}
		if(saveFile==null)return;
		if(saveFile.exists()&&dialogs.confirmYesNo("Save File",
				"Overwrite " +saveFile+"?")!=Yes)return;
		active.setSink(saveFile);
		fileSave(active);
	}
	private void fileSave(ViewerContenter content){
		Object sink=content.sink();
		if(!(sink instanceof File&&content.setSink(sink))){
			fileSaveAs();
			return;
		}
		File sinkFile=(File)sink;
		SHost host=app.host();
		try{
			content.saveToSink(sinkFile);
			values.updateRecentFiles(sinkFile);
			if(false)app.dialogs().infoMessage("File saved","Saved as "+sinkFile.getAbsolutePath());
		}catch(IOException e){
			if(true)app.dialogs().errorMessage("File not saved",sinkFile.getAbsolutePath()+
					" could not be saved:\n"+e);
			else throw new RuntimeException(e);
		}
	}
	@Override
	final protected void attachToApp(final ActionAppSurface app){
		super.attachToApp(app);
		values=new AppFileValues(app.values){
			protected File getOpeningRecent(String[]recentPaths){
				return app.dialogs().confirmGetFile(TITLE_FILE_OPEN_PREVIOUS,recentPaths[0]);
			}
			protected File getOpeningOther(FileSpecifier[]specifiers){
				return app.dialogs().openFile(specifiers);
			}
			protected void gotNoOpening(){
				final Dialogs dialogs=app.dialogs();
				if(app.contentStyle==DESKTOP)return;
				else if(app.spec.canCreateContent())
					dialogs.infoMessage(TITLE_NO_FILE,TITLE_NO_FILE);
				else{
					if(true)dialogs.infoMessage(TITLE_NO_NEW,TITLE_NO_NEW);
					System.exit(0);
				}
			}
			public FileSpecifier[]getOpenSpecifiers(){
				return ((FacetAppSurface)app).getFileSpecifiers();
			}
		};
	}
	private void fileClose(){
		boolean lastContent=app.getViewerContents().size()==1;
		if(app.contentStyle==DESKTOP||!lastContent)
			app.removeActiveContent();
		else if(app.dialogs().confirmOKCancel("Last Content",
				"This will close $appTitle")==Ok)
			attemptClose();
	}
	private void fileCloseAll(){
		if(app.contentStyle==DESKTOP)app.removeAllContent();
		else if(app.dialogs().confirmOKCancel("Closing All",
				"This will close $appTitle")==Ok)
			attemptClose();
	}
	final protected Response contentIsRemovable(String dialogTitle,Contenter content){
		ViewerContenter vc=(ViewerContenter)content;
		if(!app.spec.canSaveContent()||!vc.hasChanged())return Yes;
		switch(app.dialogs().warningYesNoCancel("Save file?",
		"Save changes to "+vc.title()+"?")){
			case Yes:fileSave(vc);return Yes;
			case No:return Yes;
			case Cancel:default:return Cancel;
		}
	}
	final boolean appCloseAcceptable(){
		String dialogTitle="Closing "+app.title();
		for(ViewerContenter vc:app.getViewerContents()){
			if(contentIsRemovable(dialogTitle,vc)==Cancel)return false;
		}
		Contenter activeContent=app.findActiveContent();
		if(!(activeContent instanceof ViewerContenter))return true;
		Object sink=((ViewerContenter)activeContent).sink();
		if(sink instanceof File)values.updateRecentFiles((File)sink);
		return true;
	}
	final public AppFileValues values(){
		if(values==null)throw new IllegalStateException(
				"Null values in "+Debug.info(this));
		return values;
	}
}
/*/*
This file forms part of Version 0.8.79 of Facets, a framework demonstrating 
the Superficial approach to GUI-driven interactive applications.  
http://superficial.sourceforge.net 				
Copyright (C) 2011  David M Wright 
This library is free software; you can redistribute it and/or modify it under 
the terms of the GNU Lesser General Public License as published by the 
Free Software Foundation; either release 3 of the License, or (at your 
option) any later release.
This library is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR 
A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more 
details.
You should have received a copy of the GNU Lesser General Public License along 
with this library; if not, write to the Free Software Foundation, Inc., 
59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

