package facets.facet.app.tree;
import static facets.core.app.ActionViewerTarget.Action.*;
import static facets.facet.app.tree.TreeTargets.*;
import facets.core.app.ActionViewerTarget;
import facets.core.app.AppHost;
import facets.core.app.AppSurface;
import facets.core.app.NodePath;
import facets.core.app.NodeViewable;
import facets.core.app.PathSelection;
import facets.core.app.ValueEdit;
import facets.core.app.ViewerContenter;
import facets.core.superficial.FacetedTarget;
import facets.core.superficial.Notice;
import facets.core.superficial.Notifying.Impact;
import facets.core.superficial.SFacet;
import facets.core.superficial.SFrameTarget;
import facets.core.superficial.STarget;
import facets.core.superficial.STrigger;
import facets.core.superficial.TargetCore;
import facets.core.superficial.app.SAreaTarget;
import facets.core.superficial.app.SContentRootTargeter;
import facets.core.superficial.app.SHost;
import facets.core.superficial.app.SHost.FacetLayout;
import facets.core.superficial.app.SSelection;
import facets.core.superficial.app.SView;
import facets.core.superficial.app.SViewer;
import facets.core.superficial.app.SelectionView;
import facets.core.superficial.app.ViewableAction;
import facets.core.superficial.app.ViewableFrame;
import facets.core.superficial.app.ViewerTarget;
import facets.facet.AreaFacets;
import facets.facet.app.FacetAppSpecifier;
import facets.facet.app.FacetAppSurface;
import facets.facet.app.FileAppActions;
import facets.util.Debug;
import facets.util.FileSpecifier;
import facets.util.Stateful;
import facets.util.Strings;
import facets.util.TextLines;
import facets.util.datatree.DataNode;
import facets.util.datatree.TypedNode;
import facets.util.datatree.XmlDocRoot;
import facets.util.datatree.XmlSpecifier;
import java.io.File;
import java.io.IOException;
/**
{@link ViewerContenter} that manages content for {@link TreeAppSpecifier}. 
<p>Effectively a package-private class; 
declared <code>public</code> for documentation purposes only. 
<p>The code exemplifies 'filling out' the <code>abstract</code> {@link ViewerContenter} 
to provide real-world functionality.  
 */
public final class TreeAppContenter extends ViewerContenter{
	/** Index into return of {@link #lazyContentRootElements(SAreaTarget)}. */
	public static final int TARGETS_PANE=0,TARGETS_CONTENT=1;
	private final static ViewableAction[]EDIT_ACTIONS={COPY,CUT,PASTE,PASTE_INTO,DELETE,EDIT};
	private static final String STATE_OFFSETS="selectionOffsets";
	private final FacetAppSurface app;
	private final TreeAppSpecifier treeSpec;
	private Object stateStamp=null;
	TreeAppContenter(Object source,FacetAppSurface app){
		super(source);
		this.app=app;
		treeSpec=(TreeAppSpecifier)app.spec;
	}
	@Override
	protected ViewableFrame newContentViewable(Object source){
		DataNode root=null;
		if(source instanceof File){
			File file=(File)source;
			for(FileSpecifier fileType:app.getFileSpecifiers())
				if(fileType.specifies(file))try{
					XmlSpecifier xml=(XmlSpecifier)fileType;
					XmlDocRoot rootParent=xml.newRootParent(xml.newDataNode(file));
					rootParent.readFromSource(file);
					root=rootParent.tree;
					stateStamp=root.stateStamp();
					break;
				}catch(IOException e){
					throw new RuntimeException(e);
				}
			if(root==null)throw new IllegalStateException("Bad file type in "+file);
		}
		else root=(DataNode)source;
		NodeViewable viewable=new NodeViewable(root,app.ff.statefulClipperSource(false)){
			@Override
			protected SSelection newViewerSelection(SViewer viewer){
				return ((SelectionView)viewer.view()).newViewerSelection(viewer,selection());
			}
			@Override
			public ViewableAction[]viewerActions(SView view){
				return view.isLive()?EDIT_ACTIONS:new ViewableAction[]{};
			}
			@Override
			protected void viewerSelectionChanged(SViewer viewer,
					SSelection selection){
				super.viewerSelectionChanged(viewer,selection);
				app.values.state().put(STATE_OFFSETS,((PathSelection)selection).paths[0].offsets);
			}
			@Override
			protected STarget[]lazyElements(){
				return new TreeTargets(app,this).appTargets();
			}
			@Override
			public boolean editSelection(){
				return new ValueEdit(((PathSelection)selection())){
					protected String getDialogInput(String title,String rubric,
							String proposal){
						return app.dialogs().getTextInput(title,rubric,proposal);
					}
				}.dialogEdit();
			}
		};
		int[]offsets=app.values.state().getInts(STATE_OFFSETS);
		PathSelection definition;
		try{
			NodePath path=new NodePath(offsets);
			path.members(root);
			definition=offsets.length==0?null:PathSelection.newSingle(root,path);
		}catch(Exception e){
			definition=null;
		}
		viewable.defineSelection(definition);
		return viewable;
	}
	/**
	Delegates to {@link TreeAppSpecifier} 
	 */
	@Override
	protected FacetedTarget[]newContentViewers(ViewableFrame viewable){
		return ActionViewerTarget.newViewerAreas(viewable,ViewerTarget.newViewFrames(
			treeSpec.newContentViews((NodeViewable)viewable)
		));
	}
	@Override
	protected void attachContentRootFacets(SAreaTarget rootArea){
		app.ff.areas().attachViewerAreaPanes(rootArea,AreaFacets.PANE_SPLIT_VERTICAL, null);
	}
	@Override
	public FacetLayout newContentLayout(SHost host,SFacet appRoot,
			SContentRootTargeter t){
		return((AppHost)host).newDesktopLayout(appRoot,new TreeAppFacets(app,t));
	}
	/**
	Re-implementation returning pane and extra tree menu targets. 
	@return {@link STarget}<code>[]</code> indexable by {@link #TARGETS_PANE}
	and {@link #TARGETS_CONTENT}; the latter created in 
		{@link TreeAppSpecifier#newContentRootTargets()}
	 */
	@Override
	protected STarget[]lazyContentRootElements(SAreaTarget root){
		return new STarget[]{
				app.ff.areas().panesGetTargets(root),
				new TargetCore("TreeMenuAdjustmentTargets",
						treeSpec.newContentRootTargets()),
		};
	}
	@Override
	public boolean hasChanged(){
		return app.actions instanceof FileAppActions
			&&((Stateful)contentFrame().framed).stateStamp()!=stateStamp;
	}
	@Override
	public FileSpecifier[]sinkFileSpecifiers(){
		Object sink=sink();
		String name=sink instanceof File?((File)sink).getName()
				:((TypedNode)sink).title()+"."+((TypedNode)sink).type();
		return FileSpecifier.filterByName(app.getFileSpecifiers(),name);
	}
	@Override
	public boolean setSink(Object sink){
		if(sink instanceof File&&((File)sink).getName().startsWith("_")||
			!app.spec.canOverwriteContent())return false;
		else return super.setSink(sink);
	}
	@Override
	public void saveToSink(Object sink)throws IOException{
		File file=(File)sink;
		String sinkName=file.getName();
		if(file.exists()&&!sinkName.startsWith("_"))
			new TextLines(file).copyFile("_"+sinkName);
		SFrameTarget frame=contentFrame();
		DataNode root=(DataNode)frame.framed;
		new XmlDocRoot(root,treeSpec.xmlPolicy()).writeToSink(file);
		stateStamp=root.updateStateStamp();
		root.setTitle(Strings.fileName(file));
		app.notify(new Notice(frame,Impact.DEFAULT));
	}
	@Override
	public void wasAdded(){
		if(false&&Debug.natureDebug)
			((STrigger)contentFrame().elements()[TARGET_SEARCH].elements()[0]).fire();
	}
}
/*/*
This file forms part of Version 0.8.79 of Facets, a framework demonstrating 
the Superficial approach to GUI-driven interactive applications.  
http://superficial.sourceforge.net 				
Copyright (C) 2011  David M Wright 
This library is free software; you can redistribute it and/or modify it under 
the terms of the GNU Lesser General Public License as published by the 
Free Software Foundation; either release 3 of the License, or (at your 
option) any later release.
This library is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR 
A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more 
details.
You should have received a copy of the GNU Lesser General Public License along 
with this library; if not, write to the Free Software Foundation, Inc., 
59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

