package facets.facet.toolkit;
import static facets.core.app.AppConstants.*;
import static facets.facet.FacetFactory.*;
import static facets.facet.app.FacetConstants.*;
import static facets.util.Util.*;
import facets.core.app.ActionAppSurface;
import facets.core.app.AppConstants;
import facets.core.app.Dialogs.ExceptionTexts;
import facets.core.app.StatefulViewable.ClipperSource;
import facets.core.superficial.Notice;
import facets.core.superficial.Notifiable;
import facets.core.superficial.SFacet;
import facets.core.superficial.SNumeric;
import facets.core.superficial.STextual;
import facets.core.superficial.Notifying.Impact;
import facets.core.superficial.STextual.Coupler;
import facets.core.superficial.app.SSelection;
import facets.core.superficial.app.SSurface;
import facets.core.superficial.app.SView;
import facets.facet.ViewerAreaMaster;
import facets.facet.app.FacetAppSurface;
import facets.facet.app.FacetConstants;
import facets.facet.app.WindowAppHost;
import facets.facet.toolkit.KWrap.ItemSource;
import facets.util.Debug;
import facets.util.NumberPolicy;
import facets.util.StringFlags;
import facets.util.Strings;
import facets.util.Tracer;
import facets.util.Util;
import facets.util.app.ProvidingCache;
import facets.util.datatree.DataNode;
import facets.util.datatree.ValueNode;
import java.io.File;
import java.io.Serializable;
import java.net.URL;
import java.util.HashMap;
import java.util.Map;
public abstract class Kit extends Tracer{
	public static final boolean inWindows=System.getProperty("os.name").indexOf(
			"Windows")>-1;
	public static final Object DECORATIONS_NO_KEY="No key specified";
	public final static class FlashTextNotice extends Notice{
		public FlashTextNotice(Object source,Notifiable monitor){
			super(new STextual(Debug.info(source),"",new Coupler()),Impact.FLASH);
			textual().setMonitor(monitor);
		}
		public void setTextAndNotify(String text){
			STextual t=textual();
			t.setText(text);
			t.monitor().notify(this);
		}
		public String flashText(){
			return textual().text();
		}
		private STextual textual(){
			return((STextual)sources.get(0));
		}
	}
	public abstract class TextPaneUpdate{
		private final SSelection selection;
		public TextPaneUpdate(SSelection selection){
			this.selection=selection;
		}
		public final void updatePane(){
			Object selection=this.selection.selection();
			String lines[]=selection instanceof DataNode?
					((DataNode)selection).values()
					:selection instanceof String?new String[]{(String)selection}
					:null;
			if(lines==null)throw new IllegalStateException("No text in "+Debug.info(this));
			String text=Strings.linesString(lines);
			updatePaneText(text);
		}
		protected abstract void updatePaneText(String now);
	}
	private final ClassLoader classLoader=getClass().getClassLoader();
	public static int widgets;
	final private Map<String,Object>icons=new HashMap(),disable=new HashMap();
	final private Map<String,String>titles=new HashMap();
	final Map<String,String>rubrics=new HashMap();
	protected final transient Map<String,Integer>keyCodes=new HashMap();
	private ProvidingCache providingCache=new ProvidingCache(50,null);
	final public void readDecorationValues(Object[][]values,ValueNode nature){
		if(values==null||nature==null)
			throw new IllegalArgumentException("Null values or nature in "+Debug.info(this));
		boolean iconAware=nature.getBoolean(NATURE_ICON_AWARE),canCopy;
		String iconPath=nature.getString(NATURE_ICON_PATH);
		File icons=new File(iconPath),store=new File(nature.getString(NATURE_STORE_PATH));
		try{
			canCopy=icons.canWrite()&&store.canRead();
		}catch(Exception e){
			canCopy=false;
		}
		if(false)trace(".readDecorationValues: canCopy=",canCopy);
		for(Object[]row:values){
			if(row.length==0)continue;
			String key=row[DECORATION_KEY].toString(),
				title=(String)row[DECORATION_TITLE],
				icon=row.length<=DECORATION_ICON?"":((String)row[DECORATION_ICON]).trim(),
				rubric=row.length<=DECORATION_RUBRIC?"":((String)row[DECORATION_RUBRIC]);
			Integer keyCode=row.length<=DECORATION_KEYCODE?0:(Integer)row[DECORATION_KEYCODE];
			if(key==null||key.equals(""))throw new IllegalArgumentException(
					"Null or empty key in "+Debug.info(this));
			if(!title.equals(""))titles.put(key,title);
			if(!rubric.equals(""))rubrics.put(key,rubric);
			if(keyCode!=0)keyCodes.put(key,keyCode);
			if(!iconAware||icon.equals(""))continue;
			for(Boolean disables:new Boolean[]{false,true}){
				if(disables)icon="disable/"+icon;
				String iconResource=iconPath+"/"+icon;
				if(false)trace(".readDecorationValues: key="+key+" iconResource=",iconResource);
				URL url=classLoader.getResource(iconResource);
				if(url==null&&canCopy){
					File storeFile=new File(store,icon);
					if(storeFile.exists())try{
						println("Kit.readDecorationValues : ",storeFile);
						copyFile(storeFile,new File(iconResource));
					}catch(Exception e){
						println("Kit.readDecorationValues : ",e);
					}
					url=classLoader.getResource(iconResource);
				}
				Map<String,Object>iconStore=disables?disable:this.icons;
				if(url!=null)iconStore.put(key,newDecorationIcon(url));
				else if(false&&!disables)
					println("Kit.readDecorationValues : no icon for key="+key+" icon="+icon);
			}
		}
	}
	public final Object getDecorationIcon(String key,boolean disable){
		if(key==null)throw new IllegalArgumentException("Null key in "+Debug.info(this));
	  key=key.trim();
		Object icon=disable?this.disable.get(key):icons.get(key);
		if(!disable||icon!=null||key.indexOf(".")<0)return icon;
		icon=newFileIcon(key);
		if(icon!=null)icons.put(key.toLowerCase(),icon);
		return icon;
	}
	abstract protected Object newDecorationIcon(URL url);
	abstract protected Object newFileIcon(String fileSpec);
	final public Decoration decoration(String title,StringFlags hints){
		return new Decoration(title,this,hints);
	}
	final public Decoration decoration(String title){
		return decoration(title,StringFlags.EMPTY);
	}
	public final String getDecorationTitle(String title){
		String got=titles.get(title);
		return got==null||got.equals("")?title:got;
	}
	protected abstract Object getDecorationKeyStroke(String title);
	public final String decodeTitleText(String text){
		String[]lines=text.split("\n");
		if(lines.length>1)throw new RuntimeException("Not implemented in "+Debug.info(this));
		String decoded="";
		for(int i=0;i<lines.length;i++)
			decoded+=(getDecorationTitle(lines[i])+" ");
		return decoded.trim();
	}
	public abstract void adjustMenuMnemonics(KWrap[]items);
	public final Object getAppIcon(SSurface app){
		Object icon=getDecorationIcon(app.title(), false);
		if(icon==null)icon=getDecorationIcon(AppConstants.NATURE_APP_ICON_LARGE, false);
		return icon!=null?icon:getDecorationIcon(AppConstants.NATURE_APP_ICON_SMALL, false);
	}
	final public KButton[]buttons(KitFacet facet,int type,int usage,
			String[]titles,StringFlags hints){
		if(hints.includeFlag(HINT_DEBUG))trace(".buttons: facet=",facet);
		KButton[]buttons=new KButton[titles.length];
		for(int i=0;i<buttons.length;i++)
			(buttons[i]=newButton(facet,type,usage)).redecorate(decoration(
					titles[i],hints));
		return buttons;
	}
	protected abstract KButton newButton(KitFacet facet,int type,int usage);
	public abstract ClipperSource statefulClipperSource(boolean useSystemClipboard);
	public abstract KList dropdownList(KitFacet facet,boolean asCombo, StringFlags hints);
	public abstract KWrap filler(KitFacet facet);
	public abstract KWrap label(KitFacet facet,String title,StringFlags hints);
	public abstract KWrap menu(KitFacet facet,String title,
			ItemSource itemSource,StringFlags hints);
	final public KWrap menu(KitFacet facet,String title,final KWrap[]items,
			StringFlags hints){
		ItemSource itemSource=new ItemSource(){
			public KWrap[]getItems(){
				return items;
			}
		};
		return menu(facet,title,itemSource,hints);
	}
	public abstract KMount switchMount(KitFacet facet,KWrap[]items, StringFlags hints);
	public abstract KMount rootMount(ActionAppSurface app,KitFacet facet, boolean desktop);
	public abstract KMount rowMount(KitFacet facet,int hgap,int vgap,
			StringFlags hints);
	public abstract WindowAppHost newWindowHost(FacetAppSurface app);
	public abstract KMount spreadMount(KitFacet facet, StringFlags hints);
	public abstract KMount packedMount(KitFacet facet, StringFlags hints);
	public abstract KTargetable sliderPanel(KitFacet facet,
			int width,KWrap label,KWrap box,StringFlags hints);
	public abstract KWrap spacer(KitFacet facet,int width,int height);
	public abstract KMount splitMount(KitFacet facet,boolean wide,SNumeric ratio);
	public abstract KField textField(KitFacet facet,int cols, 
			StringFlags hints);
	public abstract KField numberField(KitFacet facet,NumberPolicy policy,
			StringFlags hints);
	public abstract KWrap viewerTabs(SFacet[]viewers);
	public abstract KWrap wrapMount(KitFacet facet,KWrap[]contents,
			int hgap,int vgap,StringFlags hints);
  public abstract KWrap nudgersPanel(KitFacet facet,KWrap[]buttons,KWrap[]boxes,
  		KWrap[]labels, StringFlags hints);
  public abstract KField colorShader(KitFacet facet,StringFlags hints);
	public abstract KMount areaTabs(SFacet[]areas, StringFlags hints);
	public abstract double layoutFactor();
	public abstract KList listPane(KitFacet facet,int width,int rows);
	public abstract KList listPaneMultiple(KitFacet facet,int width,int rows);
	public abstract KList listPaneChecked(KitFacet facet,int width,int rows);
	public abstract KMount tabMount(KWrap[]items,String[]titles);
	public abstract KField textLabel(KitFacet facet,StringFlags hints);
	public abstract KViewer masteredViewer(KitFacet facet,ViewerAreaMaster vam,SView view,
			ValueNode stateNode);
	final public void setCache(ProvidingCache providingCache){
		this.providingCache=providingCache;
	}
	final public ProvidingCache providingCache(){
		return providingCache;
	}
	final static java.awt.Point pointAWT(int[]ints){
		final int X=0,Y=1,COORDS=2;
		return new java.awt.Point(ints[X],ints[Y]);
	}
	public abstract void warningCritical(ExceptionTexts tt,Exception e,boolean inOpen);
}/*/*
This file forms part of Version 0.8.79 of Facets, a framework demonstrating 
the Superficial approach to GUI-driven interactive applications.  
http://superficial.sourceforge.net 				
Copyright (C) 2011  David M Wright 
This library is free software; you can redistribute it and/or modify it under 
the terms of the GNU Lesser General Public License as published by the 
Free Software Foundation; either release 3 of the License, or (at your 
option) any later release.
This library is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR 
A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more 
details.
You should have received a copy of the GNU Lesser General Public License along 
with this library; if not, write to the Free Software Foundation, Inc., 
59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/
