package facets.facet.toolkit.avatar;
import static facets.util.Objects.*;
import facets.core.app.avatar.AvatarContent;
import facets.core.app.avatar.AvatarPolicy;
import facets.core.app.avatar.AvatarView;
import facets.core.app.avatar.Painter;
import facets.core.app.avatar.PainterSource;
import facets.core.app.avatar.Pickable;
import facets.core.superficial.app.SView;
import facets.core.superficial.app.SViewer;
import facets.util.Debug;
import facets.util.Objects;
import facets.util.Util;
import facets.util.geom.Point;
import facets.util.geom.Vector;
final class Avatar implements Pickable,Snappable{
	final AvatarContent content;
	private final AvatarCanvas canvas;
	final AvatarPolicy policy;
	private boolean selected;
	private Painter[]loPainters,hiPainters;
	private final int cursorPickedSelected;
	private SView thenView;
	private int cursorThen=Integer.MAX_VALUE;
	Avatar(AvatarCanvas canvas,AvatarContent content,PainterSource painterSource){
		this.canvas=canvas;
		this.content=content;
		SViewer viewer=canvas.viewer();
		this.policy=((AvatarView)viewer.view()).avatars().avatarPolicy(viewer,content,painterSource);
		cursorPickedSelected=(Integer)policy.stateCursor(
				AvatarPolicy.STATE_PICKED_SELECTED);
	}
	public Object checkCanvasHit(Point canvasAt,double hitGap){
		Object hit=null;
		PickPainter[]hitPainters={};
		if(content instanceof Pickable)
			hit=((Pickable)content).checkCanvasHit(canvasAt,hitGap);
		else{
			Pick hitPick=canvas.checkPickables(Objects.newTyped(Pickable.class,
					fetchPainters(selected,canvas.viewer().isActive())),
					canvasAt,hitGap);
			if(hitPick!=null){
				hit=hitPick.hit;
				hitPainters=hitPick.painters;
			}
		}
		int cursor=!selected||hit==null?SViewer.CURSOR_DEFAULT:
			cursorPickedSelected;
		if(cursorThen!=cursor)canvas.host.setCanvasCursor(cursorThen=cursor);
		if(hit==null){
			policy.avatarNotPicked();
			return null;
		}
		PickPainter[]painters=newTyped(PickPainter.class,policy.newPickPainters(hit,selected));
		return new Pick(this,hit,hitPainters==null?painters
				:join(PickPainter.class,hitPainters,painters));
	}
	public Vector checkSnap(AvatarCanvas canvas,Point check,double snapGap){
		Snappable[]snappables=fetchSnappables(selected);
		Vector snap=null;
		for(int i=0;i<snappables.length&&snap==null;i++)
			snap=snappables[i].checkSnap(canvas,check,snapGap);
		return snap;
	}
	private Snappable[]fetchSnappables(boolean selected){
		return Objects.newTyped(Snappable.class,
				fetchPainters(selected,canvas.viewer().isActive()));
	}
	public Painter[]getPainters(){
		return fetchPainters(selected,canvas.viewer().isActive());
	}
	private Painter[]fetchPainters(boolean selected,boolean active){
		Painter[]painters=selected?
			hiPainters==null?
				hiPainters=policy.newViewPainters(true,active):hiPainters
			:loPainters==null?
				loPainters=policy.newViewPainters(false,active):loPainters;
		if(painters==null)throw new IllegalStateException(
				"Null painters in "+Debug.info(this));
		return painters;
	}
	void destroyPainters(){loPainters=hiPainters=null;}
	boolean isSelected(){return selected;}
	void setSelected(boolean on){selected=on;}
	public String toString(){
		return Debug.info(this)+": "+Debug.info(content);
	}
}
/*/*
This file forms part of Version 0.8.79 of Facets, a framework demonstrating 
the Superficial approach to GUI-driven interactive applications.  
http://superficial.sourceforge.net 				
Copyright (C) 2011  David M Wright 
This library is free software; you can redistribute it and/or modify it under 
the terms of the GNU Lesser General Public License as published by the 
Free Software Foundation; either release 3 of the License, or (at your 
option) any later release.
This library is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR 
A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more 
details.
You should have received a copy of the GNU Lesser General Public License along 
with this library; if not, write to the Free Software Foundation, Inc., 
59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

