package facets.facet.toolkit.avatar;
import static java.awt.event.MouseEvent.*;
import facets.core.app.ArrayPath;
import facets.core.app.PathSelection;
import facets.core.app.avatar.AvatarContent;
import facets.core.app.avatar.AvatarView;
import facets.core.app.avatar.Painter;
import facets.core.app.avatar.PainterSource;
import facets.core.app.avatar.Pickable;
import facets.core.superficial.STextual;
import facets.core.superficial.Notifying.Impact;
import facets.core.superficial.STextual.Coupler;
import facets.core.superficial.app.SSelection;
import facets.core.superficial.app.SViewer;
import facets.core.superficial.app.ViewerTarget;
import facets.facet.FacetFactory;
import facets.facet.toolkit.KitFacet;
import facets.facet.toolkit.Kit.FlashTextNotice;
import facets.util.Debug;
import facets.util.ItemList;
import facets.util.Objects;
import facets.util.OffsetPath;
import facets.util.StringFlags;
import facets.util.Times;
import facets.util.Tracer;
import facets.util.geom.Point;
import java.util.ArrayList;
import java.util.List;
abstract class AvatarCanvas extends Tracer{
	interface CanvasHost{
	  void setAndPaintPainters(Painter background,Painter[]viewPainters,
	  		Painter[]motionPainters);
	  PainterSource newPainterSource();
		void launchDnD();
		void setCanvasCursor(int type);
	}
	static final int MOUSE_DRAG_NOTIFY=MOUSE_LAST+1;
  private final class MouseHandler{
  	private Painter[]painters;
    private Drag drag;	
    private Pick thenPick,lastClick,nullPick;
		boolean handleEvent(int id,Point at,int mods){
		 	if(!canPick())return false;
		  switch(id){
		  case MOUSE_MOVED:
		  	boolean thenPainters=painters!=null;
		    painters=checkAvatarsPick(id,at);
		    if(!thenPainters&&painters==null)return true;
		    else break;
		  case MOUSE_EXITED:restoreDefaults(true);break;
		  case MOUSE_CLICKED:
		    if(thenPick!=null)updateSelection((lastClick=thenPick).picked);
		    painters=checkAvatarsPick(MOUSE_MOVED,at);
		    break;
		  case MOUSE_PRESSED:
			  if(lastClick==null||thenPick==null)return false;
			  Object thenPicked=thenPick.picked;
		  	if(!(thenPicked==lastClick.picked||
		      (thenPicked instanceof Avatar&&((Avatar)thenPicked).isSelected())))
		  		return false;
		    drag=thenPicked instanceof Avatar?mousePressedNewDrag(thenPick,at):null;
		  case MOUSE_DRAGGED:
		    if(drag==null)return false;
		    painters=Objects.newTyped(Painter.class,
					drag.respondToMouse(AvatarCanvas.this,at,MOUSE_DRAGGED+mods,painterSource));
		    break;
		  case MOUSE_DRAG_NOTIFY:
		    if(drag!=null)mouseDragEdited(drag.avatars,
		    		drag.respondToMouse(AvatarCanvas.this,at,MOUSE_DRAG_NOTIFY,painterSource),
		    		true);
		  	break;
		  case MOUSE_RELEASED:
		    if(drag!=null)mouseDragEdited(drag.avatars,
		    		drag.respondToMouse(AvatarCanvas.this,at,id,painterSource),false);
		    restoreDefaults(false);
		    painters=checkAvatarsPick(MOUSE_MOVED,at);
		    break;
		  }
		  host.setAndPaintPainters(background,viewPainters,painters);
			return true;
		}
	  private void mouseDragEdited(Avatar[]dragged,Object[]edits,boolean interim){
	    if(edits==null)return;
	    if(edits.length!=dragged.length)
	    	throw new IllegalArgumentException("Non-matching edits length in "+Debug.info(this));
	    if(false)for(int i=0;i<dragged.length;i++)dragged[i].destroyPainters();
	    viewer().selectionEdited(null,edits,interim);
	  }
		private Drag mousePressedNewDrag(Pick pick,Point at){
			updateSelection(pick.picked);
	    ItemList<Avatar>selection=new ItemList(Avatar.class);
	    selection.addItem((Avatar)pick.picked);
	    for(int i=0;i<avatarSelectables.length;i++)
	      if(avatarSelectables[i].isSelected()&&avatarSelectables[i]!=pick.picked)
	      	selection.addItem(avatarSelectables[i]);
			Drag drag=new Drag((AvatarView)viewer().view(),selection.items(),
					at,pick.hit,painterSource);
			if(drag.hasPolicy)return drag;
			restoreDefaults(false);
			return null;
		}
	  private void updateSelection(Object picked){
		  boolean extending=viewer().view().allowMultipleSelection();
		  for(Avatar avatar:avatarSelectables)
		    if(extending&&picked instanceof Avatar)
		    	avatar.setSelected(avatar.isSelected()||avatar==picked);
		    else
		    	avatar.setSelected(avatar==picked);
		  SSelection selection;
		  if(!(picked instanceof Avatar))
		    selection=PathSelection.newMinimal(picked);
		  else{
		    ItemList<AvatarContent>selections=new ItemList(AvatarContent.class);
		  	for(int i=0;i<avatarSelectables.length;i++)
		      if(avatarSelectables[i].isSelected())
		      	selections.addItem(avatarSelectables[i].content);
		  	AvatarContent[]selected=selections.items();
		    OffsetPath[]paths=new OffsetPath[selected.length];
		    for(int i=0;i<paths.length;i++)
		    	paths[i]=new ArrayPath(selected,selected[i]);
		    selection=new PathSelection(selected,paths);
		  }
			viewer().selectionChanged(selection);    
		}
		private boolean canPick(){
			return viewer().isActive()&&viewer().view().isLive();
		}
		void reset(){
			painters=null;drag=null;thenPick=nullPick;
		}
		private PickPainter[]checkAvatarsPick(int eventType,Point at){
		  if(!canPick()||avatarSelectables==null)return null;
		  if(nullPick==null)thenPick=nullPick=Pick.newNullPick(viewer().view());
		  ItemList<Pickable>pickables=new ItemList(Pickable.class);
		  for(Avatar a:avatarSelectables)if(a.isSelected())pickables.addItem(a);
		  for(Avatar a:avatarSelectables)if(!a.isSelected())pickables.addItem(a);
		  Pick pick=checkPickables(pickables.items(),at,hitGap()),
		  	nowPick=pick!=null?pick:nullPick;
		  if(nowPick.picked==thenPick.picked)
		    return nowPick.respondToMouse(AvatarCanvas.this,at,eventType);
		  else{
		    thenPick.respondToMouse(AvatarCanvas.this,at,MOUSE_EXITED);
		    return(thenPick=nowPick).respondToMouse(AvatarCanvas.this,at,
		    		MOUSE_ENTERED);
		  }
		}
	}
	final Pick checkPickables(Pickable[]pickables,Point at,double hitGap){
		Pick pick=null;
		for(int i=0;i<pickables.length&&pick==null;i++)
		  pick=(Pick)pickables[i].checkCanvasHit(at,hitGap);
		return pick;
	}
	protected final KitFacet facet;
  protected CanvasHost host;
  protected Avatar[]avatarSelectables;
  private final MouseHandler mouseHandler=new MouseHandler();
  private final FlashTextNotice flash;
  private Painter viewPainters[],background;
  private PainterSource painterSource;
	private Object atThen;
	AvatarCanvas(KitFacet facet,StringFlags hints){
		this.facet=facet;
		if(!(viewer().view()instanceof AvatarView))throw new IllegalArgumentException(
				"Non-avatar viewer in "+Debug.info(this));
		flash=hints.includeFlag(FacetFactory.HINT_NO_FLASH)?null
				:new FlashTextNotice(this,facet.target());
	}
  final SViewer viewer(){
		return((ViewerTarget)facet.target());
	}
	final void restoreDefaults(boolean exiting){
		boolean launchDnD=((AvatarView)viewer().view()).doesDnD()&&exiting&&
			mouseHandler.drag!=null;
		mouseHandler.reset();
		host.setCanvasCursor(SViewer.CURSOR_DEFAULT);
		if(launchDnD)host.launchDnD();
	}
	protected void refreshViewPainters(){
		if(false)Times.printElapsed("AvatarCanvas.refreshViewPainters");
		restoreDefaults(false);
	  if(painterSource==null)painterSource=host.newPainterSource();
		SViewer viewer=viewer();
		SSelection selection=viewer.selection();
		AvatarContent[]contents=(AvatarContent[])selection.content();
		ItemList<Avatar>avatars=new ItemList(Avatar.class),
			selectables=new ItemList(Avatar.class);
		AvatarView view=(AvatarView)viewer.view();
		for(AvatarContent content:contents){
			Avatar a=new Avatar(this,content,painterSource);
			avatars.add(a);
			if(view.avatars().isContentSelectable(content))selectables.add(a);
		}
		this.avatarSelectables=selectables.items();
		background=view.avatars().getBackgroundPainter(viewer,painterSource);
	  List<Avatar>below=new ArrayList(),above=new ArrayList();		
		Object[]selected=selection.selections();
		for(Avatar a:avatars){
	  	boolean isSelected=false;
	  	for(Object s:selected)isSelected|=a.content==s;
	  	a.setSelected(isSelected);
	    if(a.isSelected())above.add(a);
			else below.add(a);
	  }
	  ItemList<Painter>viewList=new ItemList(Painter.class);
		for(Avatar b:below)viewList.addItems(b.getPainters());
		for(Avatar a:above)viewList.addItems(a.getPainters());
	  host.setAndPaintPainters(background,viewPainters=viewList.items(),null);
		if(false)Times.printElapsed("AvatarCanvas.~refreshViewPainters");
	}
	protected boolean consumeMouseEvent(int id,double atX,double atY,int mods){
	  Point at=new Point(atX,atY);
	  if(flash!=null&&!at.equals(atThen))flash.setTextAndNotify(at.toString());
	  atThen=at;
	  return mouseHandler.handleEvent(id,at,mods);
	}
	protected double markSize(){
		return((AvatarView)viewer().view()).markPixels();
	}
	protected double hitGap(){
		return((AvatarView)viewer().view()).pickHitPixels();
	}
	protected void paneSizeSet(int width,int height){}
}
/*/*
This file forms part of Version 0.8.79 of Facets, a framework demonstrating 
the Superficial approach to GUI-driven interactive applications.  
http://superficial.sourceforge.net 				
Copyright (C) 2011  David M Wright 
This library is free software; you can redistribute it and/or modify it under 
the terms of the GNU Lesser General Public License as published by the 
Free Software Foundation; either release 3 of the License, or (at your 
option) any later release.
This library is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR 
A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more 
details.
You should have received a copy of the GNU Lesser General Public License along 
with this library; if not, write to the Free Software Foundation, Inc., 
59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

