package facets.facet.toolkit.avatar;
import facets.core.app.avatar.Painter;
import facets.core.app.avatar.PlaneView;
import facets.facet.app.FacetAppSurface;
import facets.facet.toolkit.avatar.SwingPainterSource.ImageProvider;
import facets.facet.toolkit.swing.ViewerBase;
import facets.util.Debug;
import facets.util.Times;
import facets.util.Tracer;
import facets.util.app.ProvidingCache;
import facets.util.app.ProvidingCache.ItemProvider;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.RenderingHints;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.geom.AffineTransform;
import java.io.Serializable;
import java.util.Arrays;
import javax.swing.JPanel;
import javax.swing.Timer;
final class SwingCanvasPainters extends Tracer{
	private static final boolean timing=false;
	private final ProvidingCache debug=true?null:new ProvidingCache(
			false?ProvidingCache.PASS_THROUGH:10,null){
		@Override
		protected boolean trace(){
			return true;
		}
	};
	private final SwingAvatarMaster master;
	private final boolean smallShare;
	private Painter motionPainters[],viewPainters[],backPainter;
	private Dimension sizeThen;
	private Image immediate;
	private Timer timer;
	protected void traceOutput(String msg){
		if(true)super.traceOutput(msg);
	}
	SwingCanvasPainters(SwingAvatarMaster master){
		this.master=master;
		smallShare=true;
	}
	void setPainters(Painter background,Painter[]viewPainters,
			Painter[]motionPainters){
		this.backPainter=background;
		this.viewPainters=viewPainters;
		this.motionPainters=motionPainters;		
	}
	void doPainting(Graphics2D g2){
		if(viewPainters==null)return;
		boolean optimise=true;
		if(timing)Times.printElapsed("SwingCanvasPainters.doPainting optimise="+optimise);
		if(!optimise)prepareAndPaint(g2,true,true,true);
		else doOptimisedPainting(g2,debug!=null?debug:master.base().providingCache());
	  if(timing)Times.printElapsed("SwingCanvasPainters.~doPainting");
	}
	private void prepareAndPaint(Graphics2D g2,boolean back,boolean view,boolean motion){
		PlaneCanvas plane=(PlaneCanvas)master.canvas;
		double ySign=((PlaneView)plane.viewer().view()).ySign();
		AffineTransform plot=new AffineTransform();
		plot.scale(plane.scale,plane.scale*ySign);
		plot.translate(plane.origin.x(),plane.origin.y()*ySign);
		g2.transform(plot);
		g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
		    RenderingHints.VALUE_ANTIALIAS_ON);
	  if(back)backPainter.paintInGraphics(g2.create());
	  if(view){
			if(false)traceDebug(".prepareAndPaint: viewPainters=",viewPainters);
	  	for(Painter each:viewPainters)each.paintInGraphics(g2.create());
	  }
	  if(motion&&motionPainters!=null)
		  for(Painter each:motionPainters)each.paintInGraphics(g2.create());
	}
	private void doOptimisedPainting(Graphics2D g2,final ProvidingCache cache){
		final JPanel pane=master.findCanvasPane();
		Dimension size=pane.getSize();
		final int width=size.width,height=size.height;
		boolean hashIds=true;
		final Object backId=hashIds?backPainter.hashCode():backPainter,
				viewId=hashIds?Arrays.hashCode(viewPainters):viewPainters,
				backValues[]={backId,width,height},
				allValues[]={backId,viewId,width,height};
	  if(false&&immediate!=null&&sizeThen!=null&&!sizeThen.equals(size)
	  		&&(width>sizeThen.width||height>sizeThen.height)
	  		&&!newImages(cache,0,0,null).hasForValues(allValues)){
	  	if(timer!=null)timer.stop();
			g2.drawImage(false?immediate.getScaledInstance(width,height,Image.SCALE_FAST)
					:immediate,0,0,null);
			(timer=new Timer(50,new ActionListener(){
				@Override
				public void actionPerformed(ActionEvent e){
					pane.repaint();
				}
			}){
		  	public boolean isRepeats(){return false;}
		  }).start();
		}else{
			Image back=newImages(cache,width,height,null).getForValues(backValues);
				immediate=newImages(cache,width,height,back).getForValues(allValues);
			g2.drawImage(immediate,0,0,null);
		}
	  if(motionPainters!=null)
	  	prepareAndPaint((Graphics2D)g2.create(),false,false,true);
		sizeThen=size;
	}
	private ItemProvider<Image>newImages(final ProvidingCache cache,
			final int width,final int height,final Image back){
		return new ImageProvider(cache,this,
				SwingCanvasPainters.class.getSimpleName()+".newImages"){
			boolean backPainted=back!=null;
			@Override
			public CancelStyle cancelStyle(){
				return CancelStyle.None;
			};
			@Override
			protected Image newPaintedImage(){
				if(timing)Times.printElapsed("SwingCanvasPainters..newPaintedImage: back="+
						Debug.info(back));
				Image image=newPaintableImage(width,height,
					master.findCanvasPane().getBackground());
				Graphics2D gi=(Graphics2D)image.getGraphics();
				if(backPainted)gi.drawImage(back,0,0,null);
				prepareAndPaint(gi,!backPainted,backPainted,false);
				if(timing)Times.printElapsed("SwingCanvasPainters.~newPaintedImage: ");
				return image;
			}
			@Override
			protected int maximumSharePct(){
				return cache==debug?0:smallShare?10:50;
			}
			@Override
			protected long advanceByteCount(){
				return width*height*3;
			}
		};
	}
}/*/*
This file forms part of Version 0.8.79 of Facets, a framework demonstrating 
the Superficial approach to GUI-driven interactive applications.  
http://superficial.sourceforge.net 				
Copyright (C) 2011  David M Wright 
This library is free software; you can redistribute it and/or modify it under 
the terms of the GNU Lesser General Public License as published by the 
Free Software Foundation; either release 3 of the License, or (at your 
option) any later release.
This library is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR 
A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more 
details.
You should have received a copy of the GNU Lesser General Public License along 
with this library; if not, write to the Free Software Foundation, Inc., 
59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/
