package facets.facet.toolkit.avatar;
import static java.awt.image.Raster.*;
import static java.lang.Integer.*;
import facets.core.app.avatar.Painter;
import facets.core.app.avatar.PainterMaster;
import facets.core.app.avatar.PainterSource;
import facets.core.app.avatar.Pickable;
import facets.core.app.avatar.PlaneView;
import facets.core.app.avatar.PainterMaster.Outlined;
import facets.core.app.avatar.PainterMaster.Rastered;
import facets.core.app.avatar.PainterMaster.Textual;
import facets.facet.toolkit.KitFacet;
import facets.facet.toolkit.swing.ViewerMaster;
import facets.util.Debug;
import facets.util.PackedSerialized;
import facets.util.StringFlags;
import facets.util.Tracer;
import facets.util.Util;
import facets.util.app.ProvidingCache;
import facets.util.app.ProvidingCache.ItemProvider;
import facets.util.geom.Line;
import facets.util.geom.Point;
import facets.util.geom.Vector;
import facets.util.shade.Shade;
import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Component;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.RenderingHints;
import java.awt.Shape;
import java.awt.Stroke;
import java.awt.font.FontRenderContext;
import java.awt.geom.AffineTransform;
import java.awt.geom.Arc2D;
import java.awt.geom.Line2D;
import java.awt.geom.Rectangle2D;
import java.awt.geom.Rectangle2D.Double;
import java.awt.image.BufferedImage;
import java.awt.image.ColorModel;
import java.awt.image.DataBufferInt;
import java.awt.image.Raster;
import java.awt.image.SampleModel;
import java.awt.image.SinglePixelPackedSampleModel;
import java.net.URL;
import java.util.Arrays;
import javax.swing.JLabel;
final public class SwingPainterSource extends Tracer implements PainterSource{	
	private final static class ImagePacker extends Tracer{
		private static final boolean passThrough=false;
		private final ColorModel colorModel;
		private final int bufferSize,dataType,width,height,bitMasks[];
		private final BufferedImage asIs;
		private final PackedSerialized bufferData;
		ImagePacker(BufferedImage src){
			width=src.getWidth();
			height=src.getHeight();
			Raster in=src.getData();
			DataBufferInt buffer=(DataBufferInt)in.getDataBuffer();
			bufferSize=buffer.getSize();
			SampleModel sm=in.getSampleModel();
			dataType=sm.getDataType();
			bitMasks=((SinglePixelPackedSampleModel)sm).getBitMasks();
			colorModel=src.getColorModel();
			bufferData=passThrough?null:new PackedSerialized(buffer.getBankData());
			asIs=passThrough?src:null;
		}
		BufferedImage recreateImage(){
			return passThrough?asIs:new BufferedImage(colorModel,
				false?createWritableRaster(newSampleModel(1,1),
							new DataBufferInt(1),null)
				:createWritableRaster(newSampleModel(width,height),
					new DataBufferInt((int[][])bufferData.unpack(),bufferSize),null),
			false,null);
		}
		private SampleModel newSampleModel(int width,int height){
			return new SinglePixelPackedSampleModel(dataType,width,height,bitMasks);
		}
	}
	public static class ImageProvider extends ItemProvider<Image>{
		public ImageProvider(ProvidingCache c,Object source,String title){
			super(c,source,ImageProvider.class.getSimpleName()+"."+title);
		}
		@Override
		final protected Image newItem(){
			return newPaintedImage();
		}
		@Override
		final protected Object packItem(Image item){
			return new ImagePacker((BufferedImage)item);
		}
		@Override
		final protected Image unpackItem(Object packed){
			return ((ImagePacker)packed).recreateImage();
		}
		@Override
		protected long finalByteCount(Object storeReady){
			return !packsItems()?advanceByteCount()
				:((ImagePacker)storeReady).bufferSize;
		}
		final protected Image newPaintableImage(int width,int height,Color c){
			Image image=new BufferedImage(width,height,BufferedImage.TYPE_INT_RGB);
			Graphics base=image.getGraphics().create();
			base.setColor(c);
			base.fillRect(0,0,width,height);
			return image;
		}
		protected Image newPaintedImage(){
			throw new RuntimeException("Not implemented in "+Debug.info(this));
		}
	}
	private class SwingTransform extends AffineTransform implements Transform{}
	private final Component swing;
	private final PlaneCanvas canvas;
	private FontRenderContext liveFontContext;
	private int newHashes;
	SwingPainterSource(PlaneCanvas canvas,Component swing){
		this.canvas=canvas;
		this.swing=swing;
	}
	public Painter bundle(final Painter[]painters,final Object...hashValues){
		return new PickPainter(){
			public Object checkCanvasHit(Point canvasAt,double hitGap){
				for(Painter p:painters){
					Object hit=p instanceof Pickable?
							((Pickable)p).checkCanvasHit(canvasAt,hitGap):null;
					if(hit!=null)return hit;
				}
				return null;
			}
			int hashCode=MAX_VALUE;
			public void paintInGraphics(Object graphics){
				for(Painter painter:painters)
					painter.paintInGraphics(((Graphics)graphics).create());
			}
			public int hashCode(){
				return hashCode==MAX_VALUE?
						hashCode=false?newHashes++
							:Arrays.hashCode(hashValues.length>0?hashValues:painters)
					:hashCode;
			}
			public boolean equals(Object o){
				return hashCode()==o.hashCode();
			}
		};
	}
	private Graphics createGraphics(){
		return swing.getGraphics().create();
	}
	public Painter textTooltip(final String text,double x,double y){
		Util.println("SwingPainterSource:"+text);
		return new PickPainter(){
			public Object checkCanvasHit(Point canvasAt,double hitGap){
				throw new RuntimeException("Not implemented in "+Debug.info(this));
			}
			public void paintInGraphics(Object graphics){
			}
		};
	}
	public Painter textLineAt(String text,String face,int points,Shade shade,double x,
			double y){
		PainterMaster master=new PainterMaster.Textual(text,face,points,x,y,shade);
		return mastered(master);
	}
	public Painter rectangle(double x,double y,double width,double height,
			Shade pen){
		Point 
		lt=new Point(x,y),
		rt=new Point(x+width,y),
		lb=new Point(x,y+height),
		rb=new Point(x+width,y+height);
		Line[]square=new Line[]{
				new Line(lt,rt),
				new Line(lt,lb),
				new Line(rb,rt),
				new Line(rb,lb),
		};
		final Double r=new Rectangle2D.Double(x,y,width,height);
		PainterMaster master=new Outlined(null,pen,true) {
			public Object getOutline(){
				return r;
			}
			public Scaling scaling(){
				return Scaling.PEN;
			}
		};
		return mastered(master);
	
	}
	public Painter pointMark(final Point at,final Shade fill,final boolean pickable){
		return new PickPainter(){
			private Rectangle2D mark;
			public void paintInGraphics(Object graphics){
				mark=plotMark(canvas);
				Graphics2D g=(Graphics2D)((Graphics)graphics).create();
				g.setColor(new Color(fill.rgb()));
				g.fill(mark);
			}
			private Rectangle2D plotMark(PlaneCanvas canvas){
				double size=canvas.markSize();
				Point drawAt=at.newPoint().shifted(new Vector(-size/2,-size/2));
				return new Rectangle2D.Double(drawAt.x(),drawAt.y(),size,size);
			}
			public Object checkCanvasHit(Point canvasAt,double hitGap){
				if(!pickable)return null;
				if(mark==null)mark=plotMark(canvas);
				Point checkAt=canvas.plot(canvasAt);
				int pxHit=true?2:((PlaneView)canvas.viewer().view()).pickHitPixels();
				boolean hit=new Rectangle2D.Double(checkAt.intX()-pxHit,
						checkAt.intY()-pxHit,pxHit*2,pxHit*2).intersects(mark);
				return !hit?null:new Pick(this,at.newPoint(),null);
			}
		};
	}
	public Painter bar(double x,double y,double length,
			double width,Shade fill,boolean pickable) {
		final Double r=new Rectangle2D.Double(x,y,length,width);
		final Object[]values={x,y,length,width,fill,pickable};
		PainterMaster master=new Outlined(fill,null,true) {
			public Object getOutline(){
				return r;
			}
			public Scaling scaling(){
				return Scaling.OUTLINE;
			}
			public int hashCode(){
				return Arrays.hashCode(values);
			}
		};
		return mastered(master);
	}
	public Painter backgroundRaster(URL url){
		PainterMaster master=new PainterMaster.Rastered(url);
		return mastered(master);
	}
	public Painter textOutline(final String text,String face,int points,
			boolean bold,boolean italic,Shade fill,Shade pen){
		final int style=(bold?Font.BOLD:0)|(italic?Font.ITALIC:0);
		if(face.equals(""))face="Serif";
		final Font font=new Font(face,style,points);
		final Object[]values={text,face,points,style,fill,pen};
		PainterMaster master=new Outlined(fill,pen,true) {
			private Shape geometry;
			public Object getOutline(){
				if(geometry==null||liveFontContext==null)geometry=
					font.createGlyphVector(getFontContext(),text).getOutline(0,0);
				return geometry;
			}
			public Scaling scaling(){
				return Scaling.NONE;
			}
			public int hashCode(){
				return Arrays.hashCode(values);
			}
		};
		return mastered(master);
	}
	public Painter turnMark(final double size,final double width,Shade fill) {
		PainterMaster master=new Outlined(null,fill,true) {
			private Shape geometry;
			private final Object stroke=new BasicStroke((float)width,
					BasicStroke.CAP_ROUND,BasicStroke.JOIN_ROUND,
					10,new float[]{5,10},0);
			public Object getOutline(){
				return geometry==null?geometry=new Arc2D.Double
						(-size/2,-size/2,size,size,60,290,Arc2D.OPEN)
	    	:geometry;
			}
			public Scaling scaling(){
				return Scaling.PEN;
			}
			public Object penStyle(){
				return stroke;
			}
		};
		return mastered(master);
	}
	public Painter stretchMark(final double height,final double length,
			final double width,Shade fill){
		PainterMaster master=new Outlined(fill,null,true){
			private Shape geometry;
			public Object getOutline(){
				return geometry==null?geometry=
					new BasicStroke((float)width,BasicStroke.CAP_ROUND,
	  	    BasicStroke.JOIN_ROUND,(float) height,new float[]{4,6},6)
		    	.createStrokedShape(new Line2D.Double(
		    			length,0,height*2+length,0))
		  	    	:geometry;
			}
			public Scaling scaling(){
				return Scaling.PEN;
			}
		};
		return mastered(master);
	}
	public Painter backgroundLines(Line[]lines,Shade shade){
		final Painter[]painters=new PickPainter[lines.length];
		for(int i=0;i<painters.length;i++)
			painters[i]=line(lines[i],shade,NO_PATTERN,false);
		return bundle(painters);
	}
	public Transform transformScale(double xy){
		SwingTransform t=new SwingTransform();
		t.setToScale(xy,xy);
		return t;
	}
	public Transform transformAt(double x,double y){
		SwingTransform t=new SwingTransform();
		t.setToTranslation(x,y);
		return t;
	}
	public Transform transformTurn(double radians, double atX, double atY){
		SwingTransform t=new SwingTransform();
		t.setToRotation(radians,atX,atY);
		return t;
	}
	public void applyTransforms(Transform[]transforms,boolean concatenate,Painter...painters){
		Transform transform=transforms[0];
		for(int i=1;i<transforms.length;i++)
			((AffineTransform)transform).concatenate((AffineTransform)transforms[i]);
		for(int i=0;i<painters.length;i++){
			((SwingPainterOutlined)painters[i]).addTransform(transform,concatenate);
		}
	}
	public double textLength(String text,String face,int size,boolean bold,
			boolean italic){
		int style=(bold?Font.BOLD:0)|(italic?Font.ITALIC:0);
		Font font=new Font(face,style,size);
		return true?new JLabel().getFontMetrics(font).stringWidth(text)*.95:
			font.createGlyphVector(
				getFontContext(),text).getOutline(0,0).getBounds2D().getWidth();
	}
	public static ViewerMaster planeMaster(KitFacet facet,StringFlags hints){
		return new SwingPlaneMaster(new PlaneCanvas(facet,hints));
	}
	public Painter line(final Line line,Shade shade,int style,boolean pickable){
		final Shape line2d=new Line2D.Double(line.from.x(),line.from.y(),
				line.to.x(),line.to.y());
		final Stroke stroke=style==NO_PATTERN||style==HAIRLINE?new BasicStroke():
			new BasicStroke(1.0f,BasicStroke.CAP_ROUND,
				BasicStroke.JOIN_ROUND,10,new float[]{3,style},0);
		return mastered(new Outlined(null,shade,pickable){
			private int hashCode=Integer.MAX_VALUE;
			public Scaling scaling(){
				return Scaling.PEN;
			}
			public Object penStyle(){
				return stroke;
			}
			public Object getOutline(){
				return line2d;
			}
			public int hashCode(){
				return hashCode==Integer.MAX_VALUE?
						hashCode=Arrays.hashCode(new Object[]{stroke,line}):hashCode;
			}
		});
	}
	public Painter mastered(PainterMaster master){
		return master instanceof Textual?
				new SwingPainterTextual((Textual)master,canvas,swing)
			:master instanceof Rastered?
				new SwingPainterRastered((Rastered)master,canvas,swing)
			:new SwingPainterOutlined((Outlined)master,canvas);
	}
	private FontRenderContext getFontContext() {
		if(liveFontContext!=null)return liveFontContext;
		Graphics2D graphics=(Graphics2D)swing.getGraphics();
		if(graphics==null)return new FontRenderContext(null,true,true);
		graphics.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
		    RenderingHints.VALUE_ANTIALIAS_ON);
		return liveFontContext=graphics.getFontRenderContext();
	}
	public static SwingPainterSource newDummy(){
		return new SwingPainterSource(null,new JLabel());
	}
}/*/*
This file forms part of Version 0.8.79 of Facets, a framework demonstrating 
the Superficial approach to GUI-driven interactive applications.  
http://superficial.sourceforge.net 				
Copyright (C) 2011  David M Wright 
This library is free software; you can redistribute it and/or modify it under 
the terms of the GNU Lesser General Public License as published by the 
Free Software Foundation; either release 3 of the License, or (at your 
option) any later release.
This library is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR 
A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more 
details.
You should have received a copy of the GNU Lesser General Public License along 
with this library; if not, write to the Free Software Foundation, Inc., 
59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/
