package facets.facet.toolkit.swing;
import static facets.facet.FacetFactory.*;
import facets.core.superficial.Notifying.Impact;
import facets.facet.FacetFactory;
import facets.facet.FacetFactory.SurfaceStyle;
import facets.facet.toolkit.Decoration;
import facets.facet.toolkit.KButton;
import facets.facet.toolkit.Kit;
import facets.facet.toolkit.KitFacet;
import facets.facet.toolkit.Kit.FlashTextNotice;
import facets.util.Debug;
import facets.util.Times;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import javax.swing.AbstractButton;
import javax.swing.FocusManager;
import javax.swing.Icon;
import javax.swing.JButton;
import javax.swing.JCheckBoxMenuItem;
import javax.swing.JMenuItem;
import javax.swing.JRadioButton;
import javax.swing.JRadioButtonMenuItem;
import javax.swing.JToggleButton;
import javax.swing.KeyStroke;
import javax.swing.SwingConstants;
import javax.swing.SwingUtilities;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
final class Button extends Widget implements KButton{
	private final boolean forMenu;
	private boolean ignoreActionEvents;
	private final int usage,type;
	private ActionEvent lastEvent;
	private int mnemonic=-1;
	Button(final KitFacet facet,int type,int usage){
		super(facet,null);
		this.type=type;
		this.usage=usage;
		forMenu=usage==USAGE_MENU;
		AbstractButton swing;
		switch(type){
	  case TYPE_FIRE:
			swing=forMenu?new JMenuItem()
				:newSmartButton(usage==USAGE_TOOLBAR);
			break;
	  case TYPE_CHECK:
			swing=forMenu?new JCheckBoxMenuItem()
				:usage==USAGE_TOOLBAR?new JToggleButton(){
				public Insets getMargin(){
					return adjustedButtonMargin(true,super.getMargin());
				}
			}
			:new TristateCheckBox();
			break;
	  case TYPE_RADIO:
			swing=forMenu?new JRadioButtonMenuItem()
				:true?new JRadioButton():new TristateRadioButton();
			break;
	  default:throw new IllegalStateException("Bad button type "+type);
	  }
		KitSwing.adjustFont(swing);
		swing.addActionListener(this);
		setSwing(swing);
	}
	public void redecorate(Decoration d){
		this.decorations=d;
		debug=d.hints.includeFlag(HINT_DEBUG);
		final AbstractButton button=(AbstractButton)swing;
		button.setFocusable(!d.hints.includeFlag(HINT_NO_FOCUS));
		button.setActionCommand(message());
		String caption=d.caption;
		boolean appletStyle=FacetFactory.surfaceStyle!=SurfaceStyle.DESKTOP,
			noMnemonics=appletStyle||d.hints.includeFlag(HINT_MENU_NO_MNEMONICS);
		boolean hintTooltips=d.hints.includeFlag(HINT_TOOLTIPS);
		if(!noMnemonics&&(mnemonic=d.mnemonic)>0)button.setMnemonic(mnemonic);
		if(!forMenu){
			if(d.icon!=null&&!d.hints.includeFlag(HINT_USAGE_PANEL)){
				button.setIcon((Icon)d.icon);
				button.setDisabledIcon((Icon)d.disable);
				button.setMnemonic(mnemonic=0);
				if(!hintTooltips)button.setToolTipText(!d.rubric.equals("")?d.rubric:d.caption);
			}else{
				button.setText(caption);
				if(!hintTooltips&&!d.rubric.equals(""))button.setToolTipText(d.rubric);
			}
			return;
		}
		final JMenuItem item=(JMenuItem)button;
		item.setText(caption);
		if(hintTooltips&&d.titleSplits.length>1)item.setToolTipText(d.titleSplits[1]);
		if(d.keyStroke!=Kit.DECORATIONS_NO_KEY)item.setAccelerator((KeyStroke)d.keyStroke);
		final FlashTextNotice flash=new FlashTextNotice(item,facet.target());
		item.addChangeListener(new ChangeListener(){
			private boolean armedThen;
			public void stateChanged(ChangeEvent e){
				if(button.getSize().height==0)return;
				boolean armed=item.isArmed();
				if(armed!=armedThen)
					flash.setTextAndNotify((armedThen=armed)?decorations.rubric:"");
			}
		});
	}
	public void setEnabled(boolean enabled){
		super.setEnabled(enabled);
		AbstractButton button=(AbstractButton)swing;
		if(enabled&&mnemonic>=0)button.setMnemonic(mnemonic);
		else if(!enabled)button.setMnemonic(-1);
	}
	private AbstractButton newSmartButton(final boolean toolbar){
		return new JButton(){
			private final static boolean fireLater=false;
		  private ActionEvent later;
			private Dimension lockedSize;
		  protected void fireActionPerformed(ActionEvent e){
		  	if(debug)Times.printElapsed("Button.fireActionPerformed: "+facet().title());
				if(fireLater){later=e;return;}
				else super.fireActionPerformed(e);	  
		  }
		  public Insets getMargin(){
				return adjustedButtonMargin(toolbar,super.getMargin());
			}
		  public Dimension getMinimumSize() {
		  	return lockedSize==null||!isShowing()?
		  			lockedSize=super.getMinimumSize():lockedSize;
		  }
		  public Dimension getMaximumSize() {
		  	return getMinimumSize();
		  }
		  public Dimension getPreferredSize() {
		  	return getMinimumSize();
		  }
		  public void paint(Graphics g){
				super.paint(g);
				if(debug)Times.printElapsed("Button.paint: "+facet().title());
				if(!fireLater||later==null)return;
				final ActionEvent send=later;later=null;
				SwingUtilities.invokeLater(new Runnable(){public void run(){
	          fireLater(send);
	      }});
		  }
	    protected void fireLater(ActionEvent send){
				super.fireActionPerformed(send);
			}
		};
	}
	public void setSelected(boolean selected){
		ignoreActionEvents=true;
		((AbstractButton)swing).setSelected(selected);
		ignoreActionEvents=false;
	}
	public void setIndeterminate(boolean on){
		if(swing instanceof TristateCheckBox)
			((TristateCheckBox)swing).setIndeterminate();
		else if(swing instanceof TristateRadioButton)
			((TristateRadioButton)swing).setIndeterminate();
		else super.setIndeterminate(on);
	}
	public String message(){
		return decorations.keyText;
	}
	public void actionPerformed(ActionEvent e){
		if(ignoreActionEvents)return;
		if(swing instanceof TristateCheckBox||swing instanceof TristateRadioButton){
			super.actionPerformed(e);
			return;
		}
		if(e==lastEvent)return;
		lastEvent=e;
		super.actionPerformed(e);
	}
	public boolean isSelected(){
		return((AbstractButton)swing).isSelected();
	}
	private Insets adjustedButtonMargin(boolean toolbar,Insets sm){
		if(sm==null)return sm;
		return toolbar?new Insets(sm.top,sm.left/3,sm.bottom,sm.right/3)
				:false?sm:new Insets(3,10,3,10);
	}
}
/*/*
This file forms part of Version 0.8.79 of Facets, a framework demonstrating 
the Superficial approach to GUI-driven interactive applications.  
http://superficial.sourceforge.net 				
Copyright (C) 2011  David M Wright 
This library is free software; you can redistribute it and/or modify it under 
the terms of the GNU Lesser General Public License as published by the 
Free Software Foundation; either release 3 of the License, or (at your 
option) any later release.
This library is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR 
A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more 
details.
You should have received a copy of the GNU Lesser General Public License along 
with this library; if not, write to the Free Software Foundation, Inc., 
59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

