package facets.facet.toolkit.swing;
import static facets.core.app.DialogSurface.*;
import static facets.facet.app.FacetAppValues.*;
import static facets.facet.app.FacetDialogSurface.*;
import facets.core.superficial.SFacet;
import facets.core.superficial.app.SHost;
import facets.core.superficial.app.SSurface;
import facets.facet.FacetFactory;
import facets.facet.toolkit.DialogHostCore;
import facets.facet.toolkit.Kit;
import facets.facet.toolkit.KitFacet;
import facets.util.Debug;
import facets.util.app.AppValues;
import facets.util.app.BusyCursor;
import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Rectangle;
import java.awt.Window;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import javax.swing.JDialog;
import javax.swing.JPanel;
final class DialogHostSwing extends DialogHostCore{
	private final Container swingLayout=new JPanel(new BorderLayout()){
		private final KeyAdapter escaper=new KeyAdapter(){
			public void keyPressed(KeyEvent e){
				if(e.getKeyCode()==KeyEvent.VK_ESCAPE)closeDialog();
			}
		};
		public void doLayout(){
			super.doLayout();
			for(Component each:KitSwing.allComponents(this)){
				each.removeKeyListener(escaper);
				each.addKeyListener(escaper);
			}
		}
	};
	private String lastTitle;
	private Container contentSwing,extendSwing,extrasSwing;
	private boolean extendSideways;
	private int extendAmount;
	private BusyCursor cursors;
	DialogHostSwing(SHost host,Kit kit){
		super(host,kit);
		cursors=((KitSwing)kit).newBusyCursor(swingLayout,null);
	}
	protected Object newWrapped(){
		Window parent=(Window)host.wrapped();
		boolean parentVisible=parent.isVisible();
		JDialog swing=new JDialog((parentVisible?parent:null),"");
		swing.setModal(true);
		swing.add(swingLayout);
		return swing;
	}
	protected void setLayoutWidgets(KitFacet content,KitFacet buttons,KitFacet extras){
		if(swingLayout.getComponentCount()>0)
			throw new RuntimeException("Not implemented in "+Debug.info(this));;
		Container rightButtons=new JPanel(new FlowLayout(FlowLayout.RIGHT));
		rightButtons.add((Component)buttons.base().wrapped());
		swingLayout.add(rightButtons,BorderLayout.SOUTH);
		contentSwing=(Container)content.base().wrapped();
		if(extras!=null)extrasSwing=(Container)extras.base().wrapped();
	}
	public void updateLayout(SSurface surface){
		boolean graph=extrasSwing!=null
			&&AppValues.stateDebug.getInt(KEY_GRAPH_WHERE)==FacetFactory.GRAPH_DIALOGS;
		swingLayout.remove(extrasSwing);
		swingLayout.remove(contentSwing);
		if(graph)swingLayout.add(extrasSwing,BorderLayout.CENTER);
		swingLayout.add(contentSwing,graph?BorderLayout.EAST:BorderLayout.CENTER);
		swingLayout.validate();
	}
	@Override
	public void setWindowExtension(SFacet extension,boolean sideways){
		if(extension!=null&&extendSwing!=null)return;
		traceDebug(".setWindowExtension: extension=",extension);
		if(extendSwing!=null)swingLayout.remove(extendSwing);
		extendSwing=extension==null?null:(Container)((KitFacet)extension).base().wrapped();
		if(!(extendSideways=sideways))throw new RuntimeException(
				"Not implemented for extendSideways=" +extendSideways+" in "+Debug.info(this));
		if(extendSwing!=null)swingLayout.add(extendSwing,BorderLayout.EAST);
		swingLayout.validate();
		if(wrapped!=null&&((JDialog)wrapped).isVisible())adjustExtend();
	}
	private void adjustExtend(){
		final JDialog swing=(JDialog)wrapped;
		traceDebug(".adjustExtend: extendAmount="+extendAmount+" extendSwing=",extendSwing);
		Runnable adjustSize=new Runnable(){public void run(){
			Dimension then=swing.getSize();
			swing.setSize(then.width+(extendSideways?extendAmount:0),
					then.height+(extendSideways?0:extendAmount));
			trace(".adjustSize: then.width="+then.width+" extendAmount="+extendAmount
					+" swing="+swing.getWidth());
		}};
		if(extendSwing!=null&&extendAmount==0){
			extendAmount=extendSideways?extendSwing.getWidth():extendSwing.getHeight();
			if(extendAmount==0)throw new IllegalStateException(
					"Zero extendAmount in "+Debug.info(this));
			adjustSize.run();
		}
		else if(extendAmount>0){
			extendAmount*=-1;
			adjustSize.run();
			extendAmount=0;
		}
		swingLayout.validate();
		swing.validate();
	}
	final protected void traceOutput(String msg){
		if(false)super.traceOutput(msg);
	}
	final private static int TRIMMINGS_ACROSS=10,TRIMMINGS_DOWN=100;
	protected void launchInWindow(Rectangle bounds,boolean resizable){
		JDialog swing=(JDialog)wrapped;
		if(swing.getParent()==null){
			Dimension box=bounds.getSize();
			bounds=new Rectangle(calculateSmartDialogAt(box,host),box);
		}
		traceDebug(".launchInWindow: noParent=" +(swing.getParent()==null)+
				" bounds="+bounds.height+" extension=",extendSwing);
		bounds.width+=TRIMMINGS_ACROSS;
		bounds.height+=TRIMMINGS_DOWN;
		swing.setBounds(bounds);
		swing.setResizable(resizable);
		swing.setTitle(kit.decodeTitleText(lastTitle));
		adjustExtend();
		((KitSwing)kit).watcher.pushCursor(cursors);
		swing.setVisible(true);
		((KitSwing)kit).watcher.popCursor();
		adjustExtend();
	}
	protected Rectangle getClosingBounds(){
		JDialog swing=(JDialog)wrapped;
		Rectangle bounds=swing.getBounds();
		if(swing.getParent()==null){
			bounds.x=AT_NOT_SET;
			wrapped=null;
		}
		bounds.width-=TRIMMINGS_ACROSS;
		bounds.height-=TRIMMINGS_DOWN;
		trace(".getClosingBounds: bounds=",bounds.getSize());
		return bounds;
	}
	public void dismissWindow(){
		closeDialog();
	}
	private void closeDialog(){
		((JDialog)wrapped).setVisible(false);
	}
	public final void setTitle(String title){
		lastTitle=title;
		if(wrapped!=null)((JDialog)wrapped).setTitle(kit.decodeTitleText(lastTitle));
	}
}/*/*
This file forms part of Version 0.8.79 of Facets, a framework demonstrating 
the Superficial approach to GUI-driven interactive applications.  
http://superficial.sourceforge.net 				
Copyright (C) 2011  David M Wright 
This library is free software; you can redistribute it and/or modify it under 
the terms of the GNU Lesser General Public License as published by the 
Free Software Foundation; either release 3 of the License, or (at your 
option) any later release.
This library is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR 
A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more 
details.
You should have received a copy of the GNU Lesser General Public License along 
with this library; if not, write to the Free Software Foundation, Inc., 
59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/
