package facets.facet.toolkit.swing;
import static facets.core.app.AppConstants.*;
import static facets.core.app.Dialogs.Response.*;
import static facets.facet.FacetFactory.*;
import static facets.facet.toolkit.swing.SmartOptionPane.*;
import static javax.swing.JOptionPane.*;
import static javax.swing.ScrollPaneConstants.*;
import facets.core.app.ActionAppSurface;
import facets.core.app.AppSurface;
import facets.core.app.DialogHost;
import facets.core.app.Dialogs;
import facets.facet.toolkit.Kit;
import facets.util.Debug;
import facets.util.FileSpecifier;
import facets.util.Objects;
import facets.util.Strings;
import facets.util.TextLines;
import facets.util.Tracer;
import facets.util.Util;
import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Container;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Insets;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.io.File;
import java.net.URL;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import javax.swing.ActionMap;
import javax.swing.InputMap;
import javax.swing.JComponent;
import javax.swing.JEditorPane;
import javax.swing.JLabel;
import javax.swing.JMenuBar;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.KeyStroke;
import javax.swing.text.StyledEditorKit;
import javax.swing.text.html.HTMLEditorKit;
final class DialogsSwing extends Dialogs{
	private static final int TEXT_ROWS=0,TEXT_COLS=1,TEXT_CONTENT=2;
	final private static Tracer t=new Tracer(){
		protected void traceOutput(String msg) {
			System.out.println(DialogsSwing.class.getSimpleName()+msg);
		};
	};
	private final static SmartOptionPane choicePane=new SmartOptionPane(DEFAULT_AT,false),
		exceptionPane=new SmartOptionPane(DEFAULT_AT,true),
		inputPane=new SmartOptionPane(DEFAULT_AT,false),
		htmlPane=new SmartOptionPane(DEFAULT_AT,true){
			JEditorPane text;
			@Override
			public
			void updateMessage(Object...values){
				if(text==null){
					text=new JEditorPane();
					text.setEditorKit(false?new StyledEditorKit():new HTMLEditorKit());
					text.setEditable(false);
					JPanel messagePane=new JPanel(new BorderLayout(0,10));
					messagePane.add(new JScrollPane(text,VERTICAL_SCROLLBAR_AS_NEEDED,
							HORIZONTAL_SCROLLBAR_AS_NEEDED),BorderLayout.CENTER);
					setMessage(messagePane);
				}
				ClassLoader loader=getClass().getClassLoader();
				String path=(String)values[0];
				URL url=loader.getResource(path);
				if(url==null)url=loader.getResource("Facets.html");
				String noContent="[Missing content: "+path+"]",html;
				try{
					html=false?noContent:new TextLines(url).readLinesString();
				}catch(Exception e){
					html=noContent;
				}
				double fontFactor=(Double)values[1];
				try{
					String _fontSize="font-size:\\s*(\\d+)";
					Pattern p=Pattern.compile(_fontSize);
					int from=0;
					Matcher m;
					while((m=p.matcher(html)).find(from)){
						String fontSize=m.group();
						double size=Util.sf(Double.valueOf(fontSize.replaceAll(_fontSize,"$1"))*fontFactor);
						fontSize=fontSize.replaceAll(_fontSize,"font-size:" +size);
						html=html.substring(0,m.start())+fontSize+html.substring(m.end());
						from=m.start()+fontSize.length();
					}
				}
				catch(Exception e){}
				if(false){
					Dimension size=text.getSize();
					t.trace(".updateMessage: size=",size);
					String[]sizeValues=html.replaceAll(
							"(?s).+<table.*width=\"(\\d+).*height=\"(\\d+).*>.*","$1 $2"
					).split(" ");
					try{
						size=new Dimension((int)(Integer.valueOf(sizeValues[0])*fontFactor),
							(int)(size.height<50?Integer.valueOf(sizeValues[1])*fontFactor:size.height));
					}catch(Exception e){
						size=new Dimension(800,200);
					}		
					t.trace(".updateMessage: ~size=",size);
					text.setMinimumSize(size);
				}
				text.setText(html);
				text.setCaretPosition(0);
				validate();
			}
		};
	public void htmlPane(String title,String path){
		htmlPane.updateMessage(path,fontFactor()*kit.layoutFactor());
		htmlPane.displayInDialog(hostComponent(),kit.getDecorationTitle(title));
	}
	static Response warningExceptionSwing(ExceptionTexts tt,Exception e,Component parent,
			boolean inOpen){
		String htmlTag=false?"":"<html>";
		Object content;
		if(tt.detailTop.trim().equals(""))content=(htmlTag+tt.rubricTop+tt.rubricTail);
		else{
			JTextArea text=new JTextArea();
			text.setEditable(false);
			text.setFont(new Font("SansSerif",0,11).deriveFont(new JMenuBar().getFont().getSize2D()));
			text.setCursor(Cursor.getPredefinedCursor(Cursor.TEXT_CURSOR));
			text.setRows(10);
			text.setColumns(50);
			text.setText(tt.detailTop+
					"Stack trace:\n"+Strings.linesString(Debug.stackTraceLines(e))+
					"\nThreads:\n"+Objects.toString(Debug.getSortedThreads(),"\n"));
			text.setCaretPosition(0);
			JPanel messagePane=new JPanel(new BorderLayout(0,15)){
				public Insets getInsets(){
					return new Insets(5,0,5,40);
				}
			};
			messagePane.add(new JLabel(htmlTag+tt.rubricTop),BorderLayout.NORTH);
			messagePane.add(new JScrollPane(text,VERTICAL_SCROLLBAR_AS_NEEDED,
					HORIZONTAL_SCROLLBAR_AS_NEEDED),BorderLayout.CENTER);
			messagePane.add(new JLabel(htmlTag+tt.rubricTail),BorderLayout.SOUTH);
			KitSwing.adjustFont(messagePane);
			content=messagePane;
		}
		Response response=getChoiceSwing(tt.title,ERROR_MESSAGE,
				inOpen?DEFAULT_OPTION:YES_NO_OPTION,true,content,parent,exceptionPane);
		choicePane.setMessage(null);
		return response;
	}
	public Response warningException(ExceptionTexts t,Exception e,boolean inOpen){
		return warningExceptionSwing(t,e,hostComponent(),inOpen);
	}
	private static Response getChoiceSwing(String title,int messageType,int optionType,
				boolean zeroIsYes,Object content,Component parent,SmartOptionPane pane){		
			pane.setMessageType(messageType);
			pane.setMessage(content);
			pane.validate();
			pane.setOptionType(optionType);
			if(true)for(Container each:KitSwing.allComponents(choicePane)){
				JComponent c=(JComponent)each;
				InputMap inputMap=c.getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW);
				if(inputMap.allKeys()==null)continue;
				if(true){
					c.addKeyListener(new KeyAdapter(){
						@SuppressWarnings("deprecation")
						public void keyPressed(KeyEvent e){
							if(Character.isLetter(e.getKeyChar()))
								e.setModifiers(e.getModifiers()|KeyEvent.ALT_DOWN_MASK);
						}
					});
					continue;
			}
			t.trace(": component=",c.hashCode());
			ActionMap actionMap=c.getActionMap();
			for(KeyStroke reg:c.getRegisteredKeyStrokes())
				if(reg.getModifiers()==0&&reg.getKeyEventType()==KeyEvent.KEY_RELEASED
						&&reg.getKeyCode()!=KeyEvent.VK_SPACE){
					if(true)t.trace(": key=" ,reg
							+" action="+c.getActionForKeyStroke(reg));
					KeyStroke addKey=KeyStroke.getKeyStroke(reg.getKeyCode(),0,false);
					for(Object action:actionMap.allKeys())inputMap.put(addKey,action);
					t.trace(": ",actionMap.get(inputMap.get(addKey)));
			}
		}
		pane.displayInDialog(parent,title);
	  Object value=pane.getValue();
		int choice=value==null?-1:((Integer)value).intValue();
	  switch(choice){
	    case 0:return zeroIsYes?Yes:Ok;
	    case NO_OPTION:return No;
	    case CANCEL_OPTION:case -1:return Cancel;
	    default:throw new RuntimeException("Unhandled choice "+choice);
	  }
	}
	private final KitSwing kit;
  private FileChooser fileChooser;
	DialogsSwing(AppSurface app,KitSwing kit){
		super(app);
		Kit.widgets+=2;
		this.kit=kit;
	}
	protected DialogHost newHost(){
		return new DialogHostSwing(app.host(),kit);
	}
	public void textPane(String title,String rubric,int rows,int cols){
		throw new RuntimeException("Not implemented in "+Debug.info(this));
	}
	public String getTextInput(String title,String rubric,String proposal){
		Component parent=hostComponent();
		if(false)return showInputDialog(parent,
				rubric,proposal);
	  inputPane.setWantsInput(false);
	  inputPane.setWantsInput(true);
	  inputPane.setMessageType(QUESTION_MESSAGE);
	  inputPane.setOptionType(OK_CANCEL_OPTION);
	  inputPane.setMessage(rubric);
	  inputPane.setInputValue("");
	  inputPane.setInitialSelectionValue(proposal);
	  inputPane.displayInDialog(parent,kit.getDecorationTitle(title));
	  String input=(String)inputPane.getInputValue();
		return input.equals(UNINITIALIZED_VALUE)?"":input;
	}
	public File confirmGetFile(String title,String path){
		return confirmYesNo(title,kit.decoration(title).rubric+path+"?")==Yes?
			 new File(path):null;
	}
	private Response getChoice(String title,String rubric,int messageType,
			int optionType,boolean zeroIsYes){
		if(title==null||title.trim().equals("")||rubric==null||rubric.trim().equals(""))
			throw new IllegalArgumentException("Null or empty title or rubric in "+Debug.info(this));
		if(rubric.equals(title))rubric=kit.decoration(title).rubric;
		String appTitle=app.title();
		rubric=rubric.replaceAll(FIND_APP_TITLE,appTitle);
		title=kit.getDecorationTitle(title).replaceAll(FIND_APP_TITLE,appTitle);
		return getChoiceSwing(title,messageType,optionType,zeroIsYes,"<html>"+rubric,
				hostComponent(),choicePane);
	}
	public Response warningYesNo(String title,String rubric){
	  return getChoice(title,rubric,WARNING_MESSAGE,YES_NO_OPTION,true);
	}
	public Response warningYesNoCancel(String title,String rubric){
	  return getChoice(title,rubric,WARNING_MESSAGE,YES_NO_CANCEL_OPTION,true);
	}
	public Response confirmOKCancel(String title,String rubric){
	  return getChoice(title,rubric,QUESTION_MESSAGE,OK_CANCEL_OPTION,false);
	}
	public Response confirmYesNo(String title,String rubric){
	  return getChoice(title,rubric,QUESTION_MESSAGE,YES_NO_OPTION,true);
	}
	public void infoMessage(String title,String rubric){
		getChoice(title,rubric,INFORMATION_MESSAGE,DEFAULT_OPTION,false);
	}
	public void errorMessage(String title,String rubric){
		getChoice(title,rubric,ERROR_MESSAGE,DEFAULT_OPTION,false);
	}
	public File openFile(FileSpecifier[]filters){
		return fileChooser().getFile(null,filters);
	}
	public File saveFile(File proposed,FileSpecifier[]filters){
		return fileChooser().getFile(proposed,filters);
	}
	Component hostComponent(){
		return (Component)app.host().wrapped();
	}
	private FileChooser fileChooser(){
		return fileChooser==null?new FileChooser((ActionAppSurface)app):fileChooser;
	}
}
/*/*
This file forms part of Version 0.8.79 of Facets, a framework demonstrating 
the Superficial approach to GUI-driven interactive applications.  
http://superficial.sourceforge.net 				
Copyright (C) 2011  David M Wright 
This library is free software; you can redistribute it and/or modify it under 
the terms of the GNU Lesser General Public License as published by the 
Free Software Foundation; either release 3 of the License, or (at your 
option) any later release.
This library is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR 
A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more 
details.
You should have received a copy of the GNU Lesser General Public License along 
with this library; if not, write to the Free Software Foundation, Inc., 
59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

