package facets.facet.toolkit.swing;
import facets.facet.FacetFactory;
import facets.facet.toolkit.KList;
import facets.facet.toolkit.Kit;
import facets.facet.toolkit.KitFacet;
import facets.util.Debug;
import facets.util.Util;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Container;
import java.awt.FlowLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.BorderFactory;
import javax.swing.DefaultListModel;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.ListSelectionModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
final class ListPane extends Widget implements KList,ListSelectionListener{
	private boolean ignoreActionEvents,indeterminate,
		selectAllEnabled,deselectAllEnabled;
	private final JList pane;
	private final JCheckBox selectAll,deselectAll;
	ListPane(KitFacet facet,Kit toolkit,int width,int rows,boolean multiSelection){
		super(facet,null);
		final boolean spread=width<=0;
		final JScrollPane scroll=new JScrollPane(JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
				JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		scroll.setViewportView(pane=new JList(new DefaultListModel()){
			public boolean getScrollableTracksViewportWidth(){
				return !spread?false:getPreferredSize().width<scroll.getViewport().getSize().width;
			};
		});
		KitSwing.adjustFont(pane);
		JPanel swing=new JPanel(new BorderLayout());
		if(false)swing.setBorder(BorderFactory.createLineBorder(Color.blue));
		if(FacetFactory.panelShade!=null)
			swing.setBackground(new Color(FacetFactory.panelShade.rgb()));
		setSwing(swing);
		swing.add(scroll,BorderLayout.CENTER);
		if(!spread){
			pane.setFixedCellWidth(width);
			pane.setVisibleRowCount(rows);
		}
		pane.addListSelectionListener(this);
		pane.setSelectionMode(multiSelection?
				ListSelectionModel.MULTIPLE_INTERVAL_SELECTION
				:ListSelectionModel.SINGLE_SELECTION);
		if(!multiSelection){
			selectAll=deselectAll=null;
			return;
		}
		JPanel selects=new JPanel(new FlowLayout(FlowLayout.LEADING));
		selectAll=new JCheckBox(toolkit.getDecorationTitle("Select All"));
		deselectAll=new JCheckBox(toolkit.getDecorationTitle("Deselect All"));
		selects.add(selectAll);selects.add(deselectAll);
		swing.add(selects,BorderLayout.SOUTH);
		selectAll.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e){
				if(selectAll.isSelected())pane.setSelectionInterval(0,
						pane.getModel().getSize()-1);
			}
		});
		deselectAll.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e){
				if(deselectAll.isSelected())pane.getSelectionModel().clearSelection();
			}
		});
		Kit.widgets+=4;
	}
	public void valueChanged(ListSelectionEvent e) {
		if(pane.getValueIsAdjusting())return;
		if(selectAll!=null){
			if(false)trace(".valueChanged: ");
			int selectedCount=pane.getSelectedIndices().length;
			boolean allSelected=selectedCount==pane.getModel().getSize(),
				noneSelected=selectedCount==0,paneEnabled=pane.isEnabled();
			selectAll.setSelected(allSelected);
			selectAll.setEnabled(selectAllEnabled=!allSelected&&paneEnabled);
			deselectAll.setSelected(noneSelected);
			deselectAll.setEnabled(deselectAllEnabled=!noneSelected&&paneEnabled);
		}
		actionPerformed(null);
	}
	public void actionPerformed(ActionEvent e){
		if(ignoreActionEvents)return;
		super.actionPerformed(e);
	}
	public void setEnabled(boolean enabled){
		super.setEnabled(enabled);
		pane.setEnabled(enabled);
		if(selectAll==null)return;
		if(false) {
			trace(".setEnabled: ",enabled);
			trace("selectAllEnabled=",selectAllEnabled+
					", enabled&selectAllEnabled="+(enabled&selectAllEnabled));
		}
		selectAll.setEnabled(enabled&selectAllEnabled);
		deselectAll.setEnabled(enabled&deselectAllEnabled);
	}
	public int[]indices(){
		int[]indices=pane.getSelectedIndices();
		if(indeterminate)
			for(int i=0;i<indices.length;i++)indices[i]--;
		return indices;
	}
	public void setIndeterminate(boolean on){
		if(indeterminate==on)return;indeterminate=on;
		setEnabled(!on);
	}
	public void setTitles(String[]titles){
		DefaultListModel list=(DefaultListModel) pane.getModel();
		int itemCount=list.getSize();
		if(titles.length==itemCount){
			boolean sameItems=true;
			for(int i=0;i<titles.length;i++)
				sameItems&=titles[i]==list.getElementAt(i);
			if(sameItems)return;
		}
		ignoreActionEvents=true;
		if(pane.getSelectedIndex()>-1)list.removeAllElements();
		for(int i=0;i<titles.length;i++)list.addElement(titles[i].trim());		
		ignoreActionEvents=false;
	}
	public void setIndices(int[]indices){
		ignoreActionEvents=true;
	  pane.setSelectedIndices(indices);
		ignoreActionEvents=false;
	}
  public void setIndex(int index,boolean titleEditable){
    throw new RuntimeException("Not implemented in "+Debug.info(this));
  }
}
/*/*
This file forms part of Version 0.8.79 of Facets, a framework demonstrating 
the Superficial approach to GUI-driven interactive applications.  
http://superficial.sourceforge.net 				
Copyright (C) 2011  David M Wright 
This library is free software; you can redistribute it and/or modify it under 
the terms of the GNU Lesser General Public License as published by the 
Free Software Foundation; either release 3 of the License, or (at your 
option) any later release.
This library is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR 
A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more 
details.
You should have received a copy of the GNU Lesser General Public License along 
with this library; if not, write to the Free Software Foundation, Inc., 
59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

