package facets.facet.toolkit.swing;
import static facets.facet.FacetFactory.*;
import static java.awt.GridBagConstraints.*;
import facets.facet.toolkit.KWrap;
import facets.facet.toolkit.Kit;
import facets.facet.toolkit.KitFacet;
import facets.facet.toolkit.swing.Spacer.Filler;
import facets.util.Debug;
import facets.util.StringFlags;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Container;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import javax.swing.BorderFactory;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.border.Border;
final class RowMount extends MountCore{
	private final boolean debug,bordered,titled,gridSplit;
	private final int align,hgap,vgap;
	private final double layoutFactor;
	private KWrap[]lastItems;
	private int nowRow=-1;
	private boolean inLastRow;
	@Override
	protected void traceOutput(String msg){
		if(debug)super.traceOutput(msg);
	}
	public RowMount(KitFacet facet,int hgap,int vgap,final StringFlags hints){
	  super(facet,null);
	  layoutFactor=false?1:KitSwing.layoutFactorSwing();
		this.hgap=(int)(hgap*layoutFactor);
		this.vgap=(int)(vgap*layoutFactor);
	  debug=hints.includeFlag(HINT_DEBUG);
		gridSplit=hints.includeFlag(HINT_GRID);
		align=hints.includeFlag(HINT_PANEL_MIDDLE)?WEST
				:hints.includeFlag(HINT_PANEL_RIGHT)?NORTHEAST
				:hints.includeFlag(HINT_PANEL_CENTER)?CENTER
				:NORTHWEST;
		bordered=hints.includeFlag(HINT_PANEL_BORDER);
		titled=!hints.includeFlag(HINT_BARE);
		JPanel swing=new JPanel(){
			public Insets getInsets(){
				int in=(int)(10*layoutFactor);
				return!hints.includeFlag(HINT_PANEL_INSET)
					||(lastItems!=null&&lastItems.length==1)?super.getInsets()
						:new Insets(in-RowMount.this.vgap,in,in,in);
			}
		};
		if(debug)swing.setBackground(Color.red);
		setSwing(swing);
		if(bordered)((JComponent)swing).setBorder(BorderFactory.createTitledBorder(
					!titled?"":(" "+facet.title()+" ")));
	}
	private class RowLayout extends GridBagLayout{		
		final int rowCount;
		RowLayout(KWrap[]items){
			int i=0,rowCount=1;
			while(i<items.length)if(items[i++]==BREAK)rowCount++;
			rowWeights=new double[rowCount];
			for(i=0;i<rowWeights.length;i++)rowWeights[i]=1;
			rowWeights[rowCount-1]=1000;
			columnWeights=gridSplit?new double[]{0,1}:new double[]{1};
			this.rowCount=rowCount;
		}
	}
	private void setNewItems(KWrap[]items){
		swing.removeAll();
		final RowLayout rowGrid=new RowLayout(items);
		swing.setLayout(rowGrid);	  
		JPanel rowMount=null;
		GridBagConstraints rowConstraints=null;
		for(int i=0;i<items.length;i++){
			KWrap item=items[i];
			if(item==BREAK){
				if(i==0||i==items.length-1)throw new IllegalStateException(
						"BREAK must not be first or last item:" +Debug.info(item)+
						" in \n"+Debug.arrayInfo(items));
				swing.add(rowMount,rowConstraints);
			}
			if(i==0||item==BREAK){
				inLastRow=++nowRow==rowGrid.rowCount-1;
				rowMount=new JPanel(rowGrid.rowCount>1&&inLastRow?new BorderLayout()
					:new GridBagLayout());
				rowConstraints=new GridBagConstraints(){{
				  	ipady=vgap;
						anchor=align;
						if(inLastRow)fill=BOTH;
						gridx=gridSplit?1:0;
						gridy=nowRow;
					}};
				if(debug)rowMount.setBackground(Color.green);
				else if(panelShade!=null)rowMount.setBackground(new Color(panelShade.rgb()));
				Kit.widgets++;
				if(item==BREAK)continue;
		  }
			traceDebug(".setNewItems: item=",item);
		  JComponent wrapped=(JComponent)swingWrapped(item);
			if(debug)wrapped.setBorder(BorderFactory.createLineBorder(Color.yellow));
			if(rowGrid.rowCount>1&&inLastRow)rowMount.add(wrapped);
			else if(!gridSplit)rowMount.add(wrapped,new GridBagConstraints(){{
					ipadx=hgap;
			  }});
		  else{
		  	Component components[]=wrapped.getComponents(),
		  		first=components.length<2?wrapped:components[0];
				swing.add(first,new GridBagConstraints(){{
			  	anchor=EAST;
			  	gridx=0;
			  	gridy=nowRow;
			  }});
		  	for(Component component:components)
		  		if(component!=first)rowMount.add(wrapped,new GridBagConstraints(){{
				  	anchor=WEST;
				  }});
		  }
		}
		swing.add(rowMount,rowConstraints);
	}
	public void setItems(KWrap... items){
		if(lastItems==null||items.length!=lastItems.length)
			setNewItems(lastItems=items);
		else{
			int i=0;for(;i<items.length&&i<items.length;i++)
				if(items[i]!=lastItems[i])break;
			if(i<items.length)setNewItems(lastItems=items);
		}
		if(true)return;
		swing.validate();swing.repaint();
	}
}
/*/*
This file forms part of Version 0.8.79 of Facets, a framework demonstrating 
the Superficial approach to GUI-driven interactive applications.  
http://superficial.sourceforge.net 				
Copyright (C) 2011  David M Wright 
This library is free software; you can redistribute it and/or modify it under 
the terms of the GNU Lesser General Public License as published by the 
Free Software Foundation; either release 3 of the License, or (at your 
option) any later release.
This library is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR 
A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more 
details.
You should have received a copy of the GNU Lesser General Public License along 
with this library; if not, write to the Free Software Foundation, Inc., 
59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

