package facets.facet.toolkit.swing;
import facets.facet.FacetFactory;
import facets.facet.SwingPanelFacet;
import facets.facet.FacetFactory.SurfaceStyle;
import facets.facet.toolkit.Decoration;
import facets.facet.toolkit.KButton;
import facets.facet.toolkit.KWidget;
import facets.facet.toolkit.KWrap;
import facets.facet.toolkit.Kit;
import facets.facet.toolkit.KitFacet;
import facets.util.Debug;
import facets.util.Tracer;
import facets.util.Util;
import facets.util.shade.Shade;

import java.awt.Color;
import java.awt.Container;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusAdapter;
import java.awt.event.FocusEvent;
import java.util.HashMap;
import java.util.Map;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.SwingUtilities;
import javax.swing.Timer;
class Widget extends Tracer implements KWidget,ActionListener{
	final class FocusRequester{
		private static final int limit=5;
		private int requests;
		private final JComponent swing;
		private final Timer requester=new Timer(300,new ActionListener(){
			public void actionPerformed(ActionEvent e){
				if(++requests>limit)throw new RuntimeException(
						"Too many focus requests for "+Debug.info(swing));
				swing.requestFocusInWindow();
			}
		});
		private final FocusAdapter listener=new FocusAdapter() {
			public void focusGained(FocusEvent e){
				requester.stop();
			}
		};
		FocusRequester(JComponent swing){
			(this.swing=swing).addFocusListener(listener);
		}
		void startRequesting(){
			if(swing.isFocusOwner())return;
			swing.requestFocusInWindow();
			requester.start();			
		}
	}
	protected final KitFacet facet;
	protected Decoration decorations;
	protected Container swing;
	protected boolean debug;
	private boolean enabled;
	final Map<String,Container>swingMap=new HashMap();
	Widget(KitFacet facet,Container swing){
    if((this.facet=facet)==null)throw new IllegalArgumentException(
    		"Null facet in "+Debug.info(this));
		if(swing!=null)setSwing(swing);
	}
	protected void setSwing(final Container swing){
    Kit.widgets++;
		if(swing==null)throw new IllegalArgumentException("Null swing in "+Debug.info(this));
		else this.swing=swing;
		if(FacetFactory.surfaceStyle==SurfaceStyle.BROWSER&&!(swing instanceof JButton))
			swing.setBackground(new Color(FacetFactory.panelShade.rgb()));
		swingMap.put(SwingPanelFacet.KEY_PANEL,swing);
	}
	public void redecorate(Decoration decorations){}
	public void setBackground(Shade color){
		swing.setBackground(new Color(color.rgb()));
	}
	public boolean isEnabled() {
		if(enabled!=swing.isEnabled())throw new IllegalStateException(
				"Bad enabled in "+Debug.info(this));
		return enabled;
	}
	public void setEnabled(boolean enabled){
		swing.setEnabled(this.enabled=enabled);
	}
	public void setIndeterminate(boolean on){
		setEnabled(!on&&enabled);
	}
	public void actionPerformed(ActionEvent e){
		if(facet!=null)facet.targetNotify(this,false);
	}
	final public boolean hasFocus(){
		return swing.isFocusOwner();
	}
	public void requestFocus(){
		throw new RuntimeException("Not implemented in "+Debug.info(this));
	}
	public KitFacet facet(){return facet;}
  public Object wrapped(){
		if(swing==null)throw new IllegalStateException("No swing in "+Debug.info(this));
		return swing;
	}
	public Object newWrapped(Object parent){
    throw new RuntimeException("Not implemented in "+Debug.info(this));
  }
  public String toString(){
		return Debug.info(this)+(facet==null?"":" "+facet);
	}
	final protected void notifyFacet(boolean interim){
		facet.targetNotify(this,interim);
	}
	Container swingWrapped(KWrap wrap){
		if(wrap==null)throw new IllegalArgumentException("No wrap in "+Debug.info(this));
		return(Container)wrap.wrapped();
	}
	public Map<String,Container>components(){
		return swingMap;
	}
}
/*/*
This file forms part of Version 0.8.79 of Facets, a framework demonstrating 
the Superficial approach to GUI-driven interactive applications.  
http://superficial.sourceforge.net 				
Copyright (C) 2011  David M Wright 
This library is free software; you can redistribute it and/or modify it under 
the terms of the GNU Lesser General Public License as published by the 
Free Software Foundation; either release 3 of the License, or (at your 
option) any later release.
This library is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR 
A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more 
details.
You should have received a copy of the GNU Lesser General Public License along 
with this library; if not, write to the Free Software Foundation, Inc., 
59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

