package facets.idiom;
import facets.facet.toolkit.swing.SmartOptionPane;
import facets.util.Debug;
import facets.util.Tracer;
import facets.util.app.AppValues;
import facets.util.datatree.ValueNode;
import java.awt.Component;
import java.awt.Point;
import javax.swing.JLabel;
/**
Platform for desktop applets demonstrating Facets idioms. 
<p>As well as supporting other demonstrator applets, {@link IdiomApplet}
itself demonstrates use of {@link AppValues}
to store its position between sessions. 
<p>{@link IdiomApplet} is declared <code>abstract</code> because 
{@link #newDialogMessagePanel()} is an invalid stub; 
incidentally demonstrating the more general idiom of an abstract
class that can be run immediately and messages which of its methods need valid implementations.    
 */
public abstract class IdiomApplet extends Tracer{
	private static final String KEY_LAST_AT="lastAt";
	final public String idiomName=getClass().getSimpleName();
	protected final AppValues values;
	protected IdiomApplet(String[]args){
		values=new AppValues(IdiomApplet.class);
		values.readValues(args);
	}
	final public void buildAndLaunch(boolean buildOnly)throws Exception{
		Component panel=newDialogMessagePanel();
		if(buildOnly)return;
		ValueNode state=values.state();
		int[]ints=state.getInts(KEY_LAST_AT);
		Point lastAt=ints.length==0?SmartOptionPane.DEFAULT_AT
				:new Point(ints[0],ints[1]);
		SmartOptionPane pane=new SmartOptionPane(lastAt,true);
		pane.updateMessage(panel);
		lastAt=pane.displayInDialog(null,getClass().getSimpleName());
		state.put(KEY_LAST_AT,lastAt.x+","+lastAt.y);
		values.tryWrite("");
		System.exit(0);
	}
	protected Component newDialogMessagePanel()throws Exception{
		throw new RuntimeException("Not implemented in "+Debug.info(this));
	}
	static void main(String[]args)throws Exception{
		new IdiomApplet(args){
			protected Component newDialogMessagePanel()throws Exception{
				return new JLabel("newDialogMessagePanel");
			};
		}.buildAndLaunch(false);
	}
}
/*/*
This file forms part of Version 0.8.79 of Facets, a framework demonstrating 
the Superficial approach to GUI-driven interactive applications.  
http://superficial.sourceforge.net 				
Copyright (C) 2011  David M Wright 
This library is free software; you can redistribute it and/or modify it under 
the terms of the GNU Lesser General Public License as published by the 
Free Software Foundation; either release 3 of the License, or (at your 
option) any later release.
This library is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR 
A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more 
details.
You should have received a copy of the GNU Lesser General Public License along 
with this library; if not, write to the Free Software Foundation, Inc., 
59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

