package facets.util;
import static facets.util.PackedSerialized.*;
import static facets.util.Times.*;
import static facets.util.Util.SerializableBytes.*;
import java.io.Serializable;
import java.io.UnsupportedEncodingException;
import java.util.Arrays;
/**
Encoding of <code>byte[]</code>s as ASCII text.
<p>Currently implemented with simple hex-encoding; compactness can be improved by
chaining with {@link PackedSerialized}.  
 */
public final class ByteStrings{
	private static abstract class Codec{
		abstract String encode(byte[]bytes);
		abstract byte[]decode(String str);
	}
	static final Codec hexDigits=new Codec(){
		@Override
		String encode(byte[]bytes){
			StringBuilder sb=new StringBuilder();
			int byteAt=0;
			for(byte b:bytes){
				int uint=0xff&(b);
				sb.append(""+(uint<0x10?"0":"")
						+Integer.toHexString(uint).toUpperCase());
			}
			return sb.toString();
		}
		@Override
		byte[]decode(String str){
			byte[]bytes=new byte[str.length()/2];
			for(int i=0;i<bytes.length;i++)
				bytes[i]=(byte)(Integer.parseInt(str.substring(i*2,i*2+2),16));
			return bytes;
		}
	};
	static final Codec memString=true?null:new Codec(){
		private final String charset=true?System.getProperty("file.encoding"):
			"UTF-8";
		@Override
		String encode(byte[]bytes){
			try{
				return new String(bytes,charset);
			}catch(UnsupportedEncodingException e){
				throw new RuntimeException(e);
			}
		}
		@Override
		byte[]decode(String str){
			try{
				return str.getBytes(charset);
			}catch(UnsupportedEncodingException e){
				throw new RuntimeException(e);
			}
		}
	};
	private static final boolean trace=false;
	/**
	Encodes bytes as ASCII text decodable by {@link #decodeString(String)}.  
	@param in the bytes to encode
	@return the encoded string or <code>null</code> if an error occurred when checking
	the encoding.
	 */
	public static String encodeBytes(byte[]in){
		try{
			String out=hexDigits.encode(in);
			hexDigits.decode(out);
			return out;
		}
		catch(Exception e){
			if(trace)Util.println("ByteStrings.encodeBytes e=",e);
			return null;
		}
	}
	/**
	Decodes a string created with {@link #encodeBytes(byte[])}. 
	@param in the encoded string, which may contain formatting whitespace
	@return the decoded bytes or <code>null</code> if decoding fails for any reason
	 */
	public static byte[]decodeString(String in){
		try{
			return hexDigits.decode(in.replaceAll("\\s+",""));
		}
		catch(Exception e){
			if(trace)Util.println("ByteStrings.decodeString e=",e);;
			return null;
		}
	}
	static void main(String[]args)throws Exception{
		times=true;
		resetWait=4000;
		printElapsed("");
		Serializable ser="��1234567890";
		printElapsed("ser\t"+(ser instanceof String?ser:serialize(ser).length));
		boolean pack=true;
		byte[]in=!pack?serialize(ser):newUnpackChecked(ser).packed;
		printElapsed("Xin\t" +in.length+"\t"+hexDigits.encode(in));
		String text=memString==null?hexDigits.encode(in):memString.encode(in);
		printElapsed("text\t" +text.length()+"\t"+text);
		if(false)printElapsed("Xmem\t"+hexDigits.encode(text.getBytes()));
		byte[]out=memString==null?hexDigits.decode(text):memString.decode(text);
		printElapsed("out==in\t"+Arrays.equals(out,in));
		printElapsed("Xout\t" +out.length+"\t"+hexDigits.encode(out));
		ser=!pack?deserialize(out):newUnpacked(out);
		printElapsed("ser\t"+(ser==null||ser instanceof String?ser:serialize(ser).length));
	}
}/*/*
This file forms part of Version 0.8.79 of Facets, a framework demonstrating 
the Superficial approach to GUI-driven interactive applications.  
http://superficial.sourceforge.net 				
Copyright (C) 2011  David M Wright 
This library is free software; you can redistribute it and/or modify it under 
the terms of the GNU Lesser General Public License as published by the 
Free Software Foundation; either release 3 of the License, or (at your 
option) any later release.
This library is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR 
A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more 
details.
You should have received a copy of the GNU Lesser General Public License along 
with this library; if not, write to the Free Software Foundation, Inc., 
59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/
