package facets.util;
import static java.lang.Math.*;
import java.text.DecimalFormat;
import java.text.NumberFormat;
/**
Floating-point utilities. 
<p>{@link Doubles} extends its superclass by
 */
public final class Doubles{
	public static int DIGITS_SF=3,DECIMALS_FX=3;
	private final double vals[];
	static void main(String[]args){
		DIGITS_SF=3;
		for(int range=3,shift=range;shift>-range;shift--){
			double val=3.45678*pow(10,shift)*(true?-1:1);
			Util.println("sf" +DIGITS_SF+"("+val+")="+sf(val));
		}
	}
	public static double sf(double val){
		return sigFigs(val,DIGITS_SF);
	}
	public static double sigFigs(double val,int digits){
		if(digits<0)throw new IllegalArgumentException("Digits <1="+digits);
		else if(Double.isInfinite(val))throw new IllegalArgumentException("Infinite value");
		else if(digits==0||val==0||val!=val)return val;
		double ceiling=pow(10,digits),floor=ceiling/10,signum=signum(val),sf=abs(val);
		if(sf<1E-3)return 0;
		boolean shiftUp=sf<floor;
		double factor=shiftUp?10:0.1;
		int shifted=0;
		for(;sf>ceiling||sf<floor;shifted+=shiftUp?1:-1)sf*=factor;
		double exp=pow(10,shifted);
		if(false)Util.println("Doubles.sigFigs: shiftUp="+shiftUp+" shifted="+shifted);
		sf=rint(sf)/exp;
		if(true||!shiftUp)return sf*signum;
		NumberFormat trimmer=DecimalFormat.getInstance();
		trimmer.setMaximumFractionDigits(shifted);
		double trim=Double.valueOf(trimmer.format(sf));
		if(trim!=sf)Util.println("Doubles.sigFigs: val="+sf+" trim="+trim);
		return trim*signum;
	}
	Doubles(double a,double b,double c){
		this(new double[]{a,b,c});
	}
	public Doubles(double[]vals){
		this.vals=vals;
	}
	double mean(){
		double all=0;
		for(int i=0;i<vals.length;i++)
			all+=vals[i];
		return all/vals.length;
	}
	double meanOfAbsolutes(){
		double all=0;
		for(int i=0;i<vals.length;i++)
			all+=abs(vals[i]);
		return all/vals.length;
	}
	public double rms(){
		double all=0,floor=0;
		for(int i=0;i<vals.length;i++)if(vals[i]<floor)floor=vals[i];
		if(floor<0)for(int i=0;i<vals.length;i++)vals[i]-=floor;
		for(int i=0;i<vals.length;i++)all+=sqrt(vals[i]);
		all/=vals.length;
		return all*all+floor;
	}
	double weightedMean(double[]weights){
		if(weights.length!=vals.length)throw new IllegalArgumentException(Debug.info(this));;
		double all=0,allWeights=0;
		for(int i=0;i<vals.length;i++){
			all+=vals[i]*weights[i];
			allWeights+=weights[i];
		}
		return all/allWeights;
	}
	public static Double[]toObjects(double[]array){
		if(array==null) return null;
		Double[]doubles=new Double[array.length];
		for(int i=0;i<doubles.length;i++)
			doubles[i]=new Double(array[i]);
		return doubles;
	}
	public static double[]toArray(Object[]objects){
		double[]array=new double[objects.length];
		for(int i=0;i<array.length;i++)
			array[i]=((Double)objects[i]).doubleValue();
		return array;
	}
	public static double hypotenuse(double a,double b){
		return sqrt(a*a+b*b);
	}
	public String toString(){
		return fx(mean())+", "+fx(rms())+", "+fx(mean()/rms());
	}
	public static double stepped(double[]steps,double[]stepped,double val){
		int at=0;for(double step:steps)
			if(val<step)return stepped[at];
			else at++;
		return stepped[at];
	}
	static boolean equal(double a,double b,double tolerance){
		return abs(a-b)<tolerance;
	}
	static double interpolate(double from,double to,double ratio){
		return from+(to-from)*ratio;
	}
	static double[]join(double[]front,double[]back){
		if(front==null) return back;
		if(back==null) return front;
		double[]join=new double[front.length+back.length];
		System.arraycopy(front,0,join,0,front.length);
		System.arraycopy(back,0,join,front.length,back.length);
		return join;
	}
	static double maxAbs(double a,double b){
		if(abs(a)>abs(b)) return a;
		else return b;
	}
	static double sqr(double d){
		return d*d;
	}
	public static double fx(double val){
		return fixed(val,DECIMALS_FX);
	}
	public static double fixed(double val,int decimals){
		if(decimals<0)throw new IllegalArgumentException("Decimals <0="+decimals);
		double shift=pow(10,decimals),rint=rint(val*shift);
		rint/=shift;
		return rint;
	}
}
/*/*
This file forms part of Version 0.8.79 of Facets, a framework demonstrating 
the Superficial approach to GUI-driven interactive applications.  
http://superficial.sourceforge.net 				
Copyright (C) 2011  David M Wright 
This library is free software; you can redistribute it and/or modify it under 
the terms of the GNU Lesser General Public License as published by the 
Free Software Foundation; either release 3 of the License, or (at your 
option) any later release.
This library is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR 
A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more 
details.
You should have received a copy of the GNU Lesser General Public License along 
with this library; if not, write to the Free Software Foundation, Inc., 
59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

