package facets.util;
import facets.util.datatree.DataNode;
import facets.util.datatree.ExceptionNode;
import facets.util.datatree.TypedNode;
import java.io.File;
import java.io.FileOutputStream;
import java.io.FilenameFilter;
import java.io.IOException;
import java.io.InputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.OutputStream;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;
/**
{@link DataNode} that knows about {@link File}s. 
 */
public class FileNode extends DataNode{
	final public static String FLAG_LIST=StringFlags.newFlag("List");
	private transient StringFlags flags=StringFlags.EMPTY;
	final public void clearFlags(){
		flags=StringFlags.EMPTY;
	}
	final public StringFlags flags(){
		return flags;
	}
	final public boolean hasFlag(String flag){
		return flags.includeFlag(flag);
	}
	final public void addFlag(String flag){
		flags=flags.addFlag(flag);
	}
	final public void removeFlag(String flag){
		flags=flags.removeFlag(flag);
	}
	private void readObject(ObjectInputStream s) throws IOException, 
			ClassNotFoundException{
		s.defaultReadObject();
		flags=StringFlags.EMPTY;
	}
	public static final String TYPE_FOLDER="Folder",TYPE_FILE="File";
	public static boolean checkReadFolders,checkReadFiles;
	public static FileNode newLinesOutput(File file,String[]lines){
		FileNode node=new FileNode(file){
			protected boolean writeOnly(){return true;}
		};
		node.setContents(lines);
		return node;
	}
  protected final static String fileTitle(File file){
		String fileName=Strings.fileName(file.getAbsoluteFile());
		if(fileName==null||fileName.equals(""))throw new IllegalStateException(
				"Null or empty fileName in "+Debug.info(file));
		return fileName;
	}
  protected final static String fileType(File file){
    if(file==null)throw new IllegalArgumentException("Null file");
    if(file.isDirectory())return TYPE_FOLDER;
    String ext=Strings.fileExtension(file);
    return ext.equals("")?TYPE_FILE:ext;
  }
	public final File file;
	final transient FilenameFilter fileFilter=new FilenameFilter(){
		public boolean accept(File dir,String name){
		  File check=new File(dir.getAbsolutePath()+"/"+name);
		  return!check.isDirectory()&&filterAcceptFile(check);
		}
	},
	dirFilter=new FilenameFilter(){
		public boolean accept(File dir,String name){
		  File check=new File(dir.getAbsolutePath()+"/"+name);
		  return check.isDirectory()&&filterAcceptDir(check);
		}
	};
  private boolean hasReadContents;
  public FileNode(File file){
		super(fileType(file),fileTitle(file));
		TextLines.setDefaultEncoding(true);
		if(!(this.file=file).exists()&&!writeOnly())setTitle("No file found");
		else if(writeOnly())setTitle(file.getAbsolutePath());
		if(!file.isDirectory())return;
		String[]list=file.list();
		if(list!=null&&list.length>0)
			setChildren(new DataNode("Unread","children"));
	}
	public void writeLines()throws IOException{
		new TextLines(file).writeAllLines((String[])contents());
	}
	protected boolean filterAcceptFile(File file){
		return true;
	}
	protected boolean filterAcceptDir(File file){
		return true;
	}
	protected FileNode newChildFileNode(File file){
		if(file.isDirectory())return new FileNode(file); 
		FileNode node=new FileNode(file);
		if(checkReadFiles)node.checkReadContents();
		return node;
	}
	protected TypedNode[]orderedFolderContents(TypedNode[]contents){
		return contents;
	}
	protected void setDirContents(File dir){
		setChildren();
		List<TypedNode>children=new ArrayList();			
		String path=dir.getAbsolutePath()+"/",list[]=dir.list(dirFilter);
		if(list!=null)for(int i=0;i<list.length;i++){
			FileNode folder=newChildFileNode(new File(path+list[i]));
			children.add(folder);
			if(checkReadFolders)folder.checkReadContents();
		}
		list=dir.list(fileFilter);
		if(list!=null)for(int i=0;i<list.length;i++)
			children.add(newChildFileNode(new File(path+list[i])));
		setChildren(orderedFolderContents(children.toArray(new TypedNode[]{})));
  }
	protected void setFileContents(File file)throws IOException{
		setContents(new TextLines(file).readLines());
	}
	protected boolean writeOnly(){
		return false;
	}
	final public void checkReadContents(){
		if(writeOnly()||hasReadContents)return;
		try{
			if(file.isDirectory())setDirContents(file);
			else setFileContents(file);
		}catch(Exception e){
			setContents(new ExceptionNode(e));
		}
		hasReadContents=true;
	}
	public static void deserializeTree(DataNode root,boolean saveNewTree){
		Object[]defaults=root.contents(),contents=null;
		String type=root.type(),title=root.title(),path=title+"."+type+".tree";
		while(contents==null)try{
			InputStream stream=FileNode.class.getClassLoader().getResource(path).openStream();
			contents=((TypedNode)new ObjectInputStream(stream).readObject()).contents();
			stream.close();
		}catch(Exception e){
			FileNode srcFolder=new FileNode(new File("_" +type));
			checkReadFolders=checkReadFiles=true;
			srcFolder.checkReadContents();
			checkReadFolders=checkReadFiles=false;
			contents=srcFolder.contents();
			if(saveNewTree)try{
				root.setContents(contents);
				serializeTree(root,Util.currentDir());
			}catch(IOException i){
				contents=defaults;
			}
		}
		if(contents[0]instanceof ExceptionNode)contents=defaults;
		root.setContents(contents);
	}
	public static void serializeTree(TypedNode root,File dirPath)throws IOException{
		class SerialNode extends TypedNode{
			Object[]contents;	
			SerialNode(TypedNode parent,TypedNode node){
				super(Object.class,node.type(),node.title());
				setParent(parent);
				Object[]contents=node.contents();
				ItemList<Object>list=new ItemList(Object.class);
				for(int i=0;i<contents.length;i++)
					list.addItem(contents[i]instanceof TypedNode?
						new SerialNode(this,(TypedNode)contents[i]):contents[i]);
				this.contents=list.items();
			}
			public Object[]contents(){return contents;}
		}
	  File tree=new File(dirPath,root.title()+"." +root.type()+".tree");
		OutputStream stream=new FileOutputStream(tree);
		root.setParent(null);
	  new ObjectOutputStream(stream).writeObject(
	  		true?root:new SerialNode(null,root));    
	  stream.close();
	  Util.println("Stored ",tree.getAbsolutePath());
	}
}
/*/*
This file forms part of Version 0.8.79 of Facets, a framework demonstrating 
the Superficial approach to GUI-driven interactive applications.  
http://superficial.sourceforge.net 				
Copyright (C) 2011  David M Wright 
This library is free software; you can redistribute it and/or modify it under 
the terms of the GNU Lesser General Public License as published by the 
Free Software Foundation; either release 3 of the License, or (at your 
option) any later release.
This library is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR 
A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more 
details.
You should have received a copy of the GNU Lesser General Public License along 
with this library; if not, write to the Free Software Foundation, Inc., 
59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

