package facets.util;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;
/**
Iteration with an index.
 */
public abstract class IndexingIterator<T>{
	final private List<T>items;
	public final int itemCount;
	public IndexingIterator(List<T>items){
		this.items=Collections.unmodifiableList(items);
		itemCount=items.size();
	}
	public IndexingIterator(T[]items){
		this(Arrays.asList(items));
	}
	public void iterateWithIndex(){
		int at=0,size=this.items.size();
		for(T src:this.items)itemIterated(src,at++);
	}
	protected abstract void itemIterated(T item,int at);
}
/*/*
This file forms part of Version 0.8.79 of Facets, a framework demonstrating 
the Superficial approach to GUI-driven interactive applications.  
http://superficial.sourceforge.net 				
Copyright (C) 2011  David M Wright 
This library is free software; you can redistribute it and/or modify it under 
the terms of the GNU Lesser General Public License as published by the 
Free Software Foundation; either release 3 of the License, or (at your 
option) any later release.
This library is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR 
A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more 
details.
You should have received a copy of the GNU Lesser General Public License along 
with this library; if not, write to the Free Software Foundation, Inc., 
59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

