package facets.util;
import java.lang.reflect.Array;
import java.util.Arrays;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
/**
Utility methods for arrays. 
 */
public final class Objects{
  private final static boolean debug=false;
	public static Object[]reverse(Object[]src){
		Class componentType=getMemberType(src),
			useType=componentType!=null?componentType:src[0].getClass();
		Object[]reverse=(Object[])Array.newInstance(useType,src.length);
		for(int i=0,srcFrom=src.length-1;i<reverse.length;i++)
			reverse[i]=src[srcFrom-i];
		return reverse;
	}
  static Class getMemberType(Object[]array){
  	String fullName=array.getClass().getName(),
  		trimSemiColon=fullName.substring(0,fullName.length()-1),
  		trimDimensions=
  			trimSemiColon.substring(fullName.indexOf("L")+1,trimSemiColon.length());
		return classForName(trimDimensions);
	}
	private static Class classForName(String className){
		Class useClass=null;
		try{useClass=Class.forName(className);}
		catch(ClassNotFoundException e){throw new RuntimeException(e+" for "+Debug.info(className));}
		return useClass;
	}
	/**
	Converts the type of an array. 
	@param type element type of the new array 
	@param src must contain only objects of a suitable type
	 */
	public static <T>T[]newTyped(Class<T>type,Object[]src){
    T[]array=(T[])Array.newInstance(type,src.length);
    if(src.length>0&&!type.isAssignableFrom(src[0].getClass()))
      throw new IllegalArgumentException(src[0]+" should be of type \n"+type);
    System.arraycopy(src,0,array,0,array.length);
    return array;
  }
  public static <T>T[]join(Class<T>type,T[]front,T[]back){
  	for(T t:front)if(t==null)throw new IllegalStateException(
  			"Null member in "+Debug.info(front));
  	for(T t:back)if(t==null)throw new IllegalStateException(
  			"Null member in "+Debug.info(back));
		if(front==null)return back;if(back==null)return front;
		T[]join=(T[])Array.newInstance(type,front.length+back.length);
		System.arraycopy(front,0,join,0,front.length);
		System.arraycopy(back,0,join,front.length,back.length);
		return join;
  }
  public static String toLines(Object...array){
    if(array==null)return "null";
    StringBuilder list=new StringBuilder(array.length*40);
    for(int i=0;i<array.length;i++)
      list.append((array[i]==null?"null"
      		:false?Debug.info(array[i]):array[i].toString())
      	+(i<array.length-1?"\n":""));
    return list.toString();
  }
  public static String toStringWithHeader(Object[]array){
		return Debug.info(array)+" ["+array.length+"] " +
				"{\n"+toLines(array)+"\n}";
	}
  public static String toString(int[]ints){
		return "{"+toString(ints,",")+"}";
	}
  public static String toString(int[]ints,String spacer){	  
    if(ints==null)return "null";
    StringBuilder list=new StringBuilder();
    for(int i=0;i<ints.length;i++)
      list.append(ints[i]+(i<ints.length-1?spacer:""));
    return list.toString();
  }
  public static String toString(Object...array){
		return toString(array,",");
	}
  public static String toString(Object[]array,String spacer){
    if(array==null)return "null";
    StringBuilder list=new StringBuilder();
    for(int i=0;i<array.length;i++)list.append(
      (array[i]==null?"null":spacer.equals("\n")?array[i]:array[i].toString().trim())
      	+(i<array.length-1?spacer:""));
    return list.toString();
  }
  public static int indexOf(Object[]array,Object member){
    for(int i=0;i<array.length;i++)if(array[i]==member)return i;
    throw new RuntimeException(Debug.info(member)+" not found in \n"+Debug.arrayInfo(array));
  }
  public static Object[]uniqued(Class type,Object[]objects){
    if(debug)Util.println("Objects.uniqued: ",objects);
    Set set=new HashSet(Arrays.asList(objects));
    if(debug)Util.println("",set.toArray());
    return newTyped(type,set.toArray());
  }
  public static Object[]sorted(Class type,Object[]array,Comparator sorter){
    List sortable=Arrays.asList(array);
    Collections.sort(sortable,sorter);
    return newTyped(type,sortable.toArray());
	}
}
/*/*
This file forms part of Version 0.8.79 of Facets, a framework demonstrating 
the Superficial approach to GUI-driven interactive applications.  
http://superficial.sourceforge.net 				
Copyright (C) 2011  David M Wright 
This library is free software; you can redistribute it and/or modify it under 
the terms of the GNU Lesser General Public License as published by the 
Free Software Foundation; either release 3 of the License, or (at your 
option) any later release.
This library is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR 
A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more 
details.
You should have received a copy of the GNU Lesser General Public License along 
with this library; if not, write to the Free Software Foundation, Inc., 
59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

