package facets.util;
import java.io.BufferedReader;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.FilterInputStream;
import java.io.FilterOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;
import java.io.UnsupportedEncodingException;
import java.net.URL;
import java.nio.channels.FileChannel;
import java.util.zip.DeflaterInputStream;
import java.util.zip.DeflaterOutputStream;
import java.util.zip.GZIPInputStream;
import java.util.zip.GZIPOutputStream;
import java.util.zip.ZipOutputStream;
/**
Reads from and where allowed writes lines to a wrapped text file reference. 
<p>Can be constructed either from a {@link File} or {@link URL};
 read-only in the latter case. 
 */
final public class TextLines extends Tracer{
	public static final String DEFAULT_ENCODING="UTF-8",EXT_ZIP=".zip",EXT_CLEAR=".clear";
	public long lines,chars;
	/**<pre>
[?Cp1252 Eclipse standard]
US-ASCII Seven-bit ASCII, a.k.a. ISO646-US, a.k.a. the Basic Latin block of the Unicode character set 
ISO-8859-1   ISO Latin Alphabet No. 1, a.k.a. ISO-LATIN-1 
UTF-8 Eight-bit UCS Transformation Format 
UTF-16BE Sixteen-bit UCS Transformation Format, big-endian byte order 
UTF-16LE Sixteen-bit UCS Transformation Format, little-endian byte order 
UTF-16 Sixteen-bit UCS Transformation Format, byte order identified by an optional byte-order mark 
	*/
	public static String encoding="No encoding set";
	private static final Object BUFFER="buffer";
	public static boolean trace=false;
	public final Object reference;
	private final boolean zip;
	private BufferedReader reader;
	private PrintWriter writer;
	private byte[]buffer;
	private String nextLine;
	private boolean nextLineRead;
	/**
	Wrap a file reference. 
	@param file must be non-<code>null</code>
	 */
	public TextLines(File file){
		this((Object)file);
	}
	/**
	Wrap a URL reference. 
	@param url must be non-<code>null</code>
	 */
	public TextLines(URL url){
		this((Object)url);
	}
	/**
	Wrap an output stream. 
	@param out must be non-<code>null</code>
	 */
	public TextLines(OutputStream out){
		this((Object)out);
	}
	/**
	Wrap an input stream. 
	@param in must be non-<code>null</code>
	 */
	public TextLines(InputStream in){
		this((Object)in);
	}
	public String getFileName(){
		return reference==BUFFER?BUFFER.toString()
			:reference instanceof File?((File)reference).getName()
					:((URL)reference).getFile();
	}
	@Override
	protected void traceOutput(String msg){
		if(trace)super.traceOutput(msg);
	}
	private TextLines(Object reference){
		if(!(reference instanceof URL)){
			String systemEncoding=System.getProperty("file.encoding");
			if(!encoding.equals(systemEncoding))throw new IllegalStateException(
					"System encoding="+systemEncoding+
					" not set to "+encoding);
		}
		if((this.reference=reference)==null)throw new IllegalArgumentException(
				"Null reference in "+Debug.info(this));
		String fileName=reference instanceof URL?((URL)reference).getFile()
				:reference instanceof File?((File)reference).getName():"";
		zip=fileName.endsWith(EXT_ZIP);
		if(reference!=BUFFER)trace(": zip=",zip +" reference=" +reference);
	}	
	/**
	Attempts to write text lines to the reference wrapped. 
	@throws IOException
	 */
	public void writeAllLines(String[]lines)throws IOException{
		if(false)trace(".writeAllLines: zip=",zip +" reference=" +reference);
		if(lines==null)throw new IllegalArgumentException("Null lines in "+Debug.info(this));
		for(int i=0;i<lines.length;i++)writeNextLine(lines[i]);
		closeLineWriter();
	}
	public void writeNextLine(String line)throws IOException{
		if(line==null)throw new IllegalArgumentException("Null line in "+Debug.info(this));
		if(writer==null)writer=new PrintWriter(new OutputStreamWriter(
				reference instanceof OutputStream?(OutputStream)reference:newOutputStream(),
						encoding)) ;
		if(false)trace("writeNextLine: "+Regex.replaceAll(line,"\n","\\\\n","\r","\\\\r"));
		writer.println(line.replaceAll("\\r",""));
		lines++;chars+=line.length();
	}
	public void closeLineWriter(){
		if(writer==null)throw new IllegalStateException("Null writer in "+Debug.info(this));
		if(false)trace(".closeLineWriter: zip=",zip +" reference=" +reference);
		writer.close();
		writer=null;
	}
	public OutputStream newOutputStream()throws IOException{
		return reference==BUFFER?new ByteArrayOutputStream(){
			public void close(){
				buffer=toByteArray();
				chars=buffer.length;
				try{
					String text=new String(buffer,encoding);
					trace(".newOutputStream: text=",text);
					lines+=text.split("\n").length;
				}catch(UnsupportedEncodingException e){
					throw new RuntimeException(e);
				}
			}
		}
		:!zip?new FileOutputStream((File)reference)
		:new GZIPOutputStream(new FileOutputStream((File)reference)){
			private final FileOutputStream clear;
			private final File zipped,unzipped;
			{
				zipped=(File)reference;
				unzipped=new File(zipped.getParentFile(),
						zipped.getName().replaceAll("(\\.\\w+)\\Q" +EXT_ZIP +"\\E",
								(false?EXT_CLEAR:"")+"$1"));
				clear=new FileOutputStream(unzipped);
			}
			@Override
			public void write(byte[] b,int off,int len)throws IOException{
				if(false)trace(".write: byte[] b,int off,int len="+len);
				super.write(b,off,len);
				clear.write(b,off,len);
			}
			@Override
			public void close()throws IOException{
				super.close();
				clear.close();
				trace(".close:" +
						" unzipped=" +unzipped.length()+
						" zipped="+zipped.length());
			}
		};
	}
	public InputStream newInputStream()throws IOException{
		chars=lines=0;
		InputStream in=reference==BUFFER?new ByteArrayInputStream(buffer)
				:reference instanceof URL?((URL)reference).openStream():
					reference instanceof InputStream?(InputStream)reference:
			new FileInputStream((File)reference);
		return zip?new GZIPInputStream(in) 
			:true?in:new FilterInputStream(in){
			public int read()throws IOException{
				int read=super.read();
				chars+=read;
				Util.println(".read: ",chars);
				return read;
			}
			public int read(byte[] b,int off,int len) throws IOException{
				int read=super.read(b,off,len);
				chars+=read;
				Util.println(".read b,off,len: ",chars);
				return read;
			}
		};
	}
	public boolean hasNextLine()throws IOException{
	  if(reader==null)reader=newReader();
	  nextLine=reader.readLine();
	  nextLineRead=true;
	  if(nextLine==null){
			reader.close();
			reader=null;
		}
		return nextLine!=null;
	}
	public BufferedReader newReader()throws IOException{
		return new BufferedReader(new InputStreamReader(
				newInputStream(),encoding));
	}
	public String nextLine()throws IOException{
		if(!nextLineRead)hasNextLine();
	  nextLineRead=false;
		if(nextLine==null)throw new IllegalStateException("Null next line in "+Debug.info(this));
		lines++;chars+=nextLine.length();
		return nextLine;
	}
	public String[]readLines()throws IOException{
		return readLinesString().split("\n");
	}
	/**
	Attempts to read text lines from the reference or buffer wrapped. 
	@throws IOException
	 */
	public String readLinesString()throws IOException{
		StringBuilder lines=new StringBuilder();
	  while(hasNextLine())lines.append(nextLine()+(true?"\n":""));
		return lines.toString().replaceAll("\n\\z","");
	}
	public static TextLines newBuffer(){
		return new TextLines(BUFFER);
	}
	public static TextLines newBuffer(String[]lines){
		TextLines tl=newBuffer();
		try{
			tl.writeAllLines(lines);
			return tl;
		}catch(IOException e){
			throw new RuntimeException(e);
		}
	}
	public String toString(){
		return //Debug.info(this)+" "+
			reference+" lines="+lines+" chars K="+chars/1024;
	}
	public void copyFile(String outName) throws IOException{
		File src=(File)reference,dst=new File(src.getParentFile()+"/"+outName);
		FileChannel in=new FileInputStream(src).getChannel(),
			out=new FileOutputStream(dst).getChannel();
		out.transferFrom(in,0,in.size());
		in.close();
		out.close();
	}
	public static void setDefaultEncoding(boolean systemAccess){
		encoding=!systemAccess?DEFAULT_ENCODING:System.getProperty("file.encoding");
	}
	public static void setEncoding(String encoding){
		System.setProperty("file.encoding",TextLines.encoding=encoding);
		if(false){
			Util.println("TextLines.setEncoding:" +
					" System=" +System.getProperty("file.encoding")+
					" TextLines="+encoding);
			if(false)Debug.printStackTrace(5);
		}
	}
	public static void createIfRequired(File file,String content) throws IOException{
		if(!file.exists())new TextLines(file).writeAllLines(content.split("\n"));
	}
}
/*/*
This file forms part of Version 0.8.79 of Facets, a framework demonstrating 
the Superficial approach to GUI-driven interactive applications.  
http://superficial.sourceforge.net 				
Copyright (C) 2011  David M Wright 
This library is free software; you can redistribute it and/or modify it under 
the terms of the GNU Lesser General Public License as published by the 
Free Software Foundation; either release 3 of the License, or (at your 
option) any later release.
This library is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR 
A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more 
details.
You should have received a copy of the GNU Lesser General Public License along 
with this library; if not, write to the Free Software Foundation, Inc., 
59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

