package facets.util;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.Serializable;
import java.nio.channels.FileChannel;
import java.util.Collection;
import java.util.HashMap;
import java.util.Map;
/**
General-purpose utility methods. 
trace(Debug)?\(("~?)[^."]*\.([^"]*")	
trace$1($2$.3
(trace(Debug)?\([^;+]+)\+
$1,
(println\("[^+"]+")\+
 */
public final class Util{
  public static final int MB=1<<20;
	public static <K,V>Map<K,V>newFilledMap(Object[][]pairs){
		Map<K,V>fill=new HashMap();
		for(Object[]p:pairs)fill.put((K)p[0],(V)p[1]);
		return fill;
	}
	public static int intPct(double num,double den){
		return(int)(num/den*100);
	}
	public static void println(String toPrint){
	  System.out.println(toPrint==null?"null":
	  	toPrint.toString());
	}
	public static void println(String msg,Object toPrint){
	  println(msg+toPrint);
	}
	public static void println(String msg,Collection c){
		println(msg+arrayPrintString(c.toArray()));		
	}
	public static void println(String msg,Object[]array){
		println(msg+arrayPrintString(array));		
	}
	public static String arrayPrintString(Object[]toPrint){
		String msg=toPrint==null?"null"
	  :false&&Objects.getMemberType(toPrint)==String.class?
			Objects.toString(toPrint)
		:Objects.toStringWithHeader(toPrint);
		return msg;
	}
	/**
	Encapsulates making a back-up copy of a file.
	 */
	public static class FileBackup{
		private final File src;
		public FileBackup(File src){
			this.src=src;
			if(src==null||!src.exists())throw new IllegalArgumentException(
					"Null or non-existing src="+src);
		}
		final public File doBackup()throws IOException{
			File bak=newBackupFile(src);
			if(!bak.exists()||overwriteExisting())copyFile(src,bak);
			return bak;
		}
		protected boolean overwriteExisting(){
			return false;
		}
		protected File newBackupFile(File src){
			return new File(src.getParent(),"_"+src.getName());
		}
	}
	/**
	Encapsulates simple (de)serialization. 
	 */
	public final static class SerializableBytes{
		public static Serializable deserialize(byte[]bytes){
			try{
				return(Serializable)new ObjectInputStream(
						new ByteArrayInputStream(bytes)).readObject();
			}catch(IOException e){
				throw new RuntimeException(e);
			}catch(ClassNotFoundException e){
				throw new RuntimeException(e);
			}
		}
		public static byte[]serialize(Serializable src){
		  ByteArrayOutputStream bytes=new ByteArrayOutputStream();
		  try{
				new ObjectOutputStream(bytes).writeObject(src);
			}catch(IOException e){
				throw new RuntimeException(e);
			}
		  return bytes.toByteArray();
		}
	}
	/**
	Determines size by counting serialized bytes. 
	 */
	public static int byteCount(Serializable src){
		try{
			return SerializableBytes.serialize(src).length;
		}catch(Exception e){
			e.printStackTrace();
			return -1;
		}
	}
	/**
	 Make a deep copy by serialization. 
	 @param src to be copied
	 */
	public static Serializable deserializedCopy(Serializable src){
		try{
			byte[]bytes=SerializableBytes.serialize(src);
			Serializable copy=SerializableBytes.deserialize(bytes);
			if(false)Util.println("Util.deserializedCopy: ",
					Debug.info(src)+"->"+Debug.info(copy)+" bytes="+bytes.length);
			return copy;
		}catch(Exception e){
			throw new RuntimeException(e);
		}
	}
	private static boolean endsWithDigit(String string){
		return Character.isDigit(string.charAt(string.length()-1));
	}
	public static void copyFile(File src,File dest)throws IOException{
  	Util.println("Util.copyFile: ",dest.getAbsolutePath());
  	if(src.equals(dest))throw new IllegalArgumentException(
				"Src=dest="+src);
    dest.createNewFile();
    FileChannel in=new FileInputStream(src).getChannel(),
  		out=new FileOutputStream(dest).getChannel();
  	in.transferTo(0,in.size(),out);
  	in.close();out.close();
  }
  public static double fx(double val){return Doubles.fx(val);}
	public static double sf(double val){return Doubles.sf(val);}
	public static String helpfulClassName(Object o){
		Class trueClass=o instanceof Class?(Class)o:o.getClass(),
				ancestor=trueClass;
		while(ancestor!=null&&endsWithDigit(ancestor.getName()))
			ancestor=ancestor.getSuperclass();
		String array=o instanceof Object[]?"[":"";
		return array+shortName(trueClass.getName())
				+(ancestor==trueClass?"":"/"+shortName(ancestor.getName()));
	}
  public static String shortTypeNameKey(Object o){
	  String[]fixes=new String[]{
	  		"\\..*","",
	  		"[\\$\\[\\]]","_",
	  		"[;]",""
	  	};
		String name=o instanceof Class?((Class)o).getName():
	    	o.getClass().getName(),
	    key=Strings.reverse(Regex.replaceAll(Strings.reverse(name),false,fixes));
	  if(false)Util.println("Util.keysafeClassName: ", name+"->"+key);
	  return key;
	}
  private static String shortName(String className){
		int semiColon=className.lastIndexOf(';'),stop=semiColon>0?semiColon
				:className.length();
		return false?className:className.substring(className.lastIndexOf('.')+1,
				stop);
	}
	private static int isgn(int val){
		return val<0?-1:1;
	}
	public static File currentDir(){
		return new File(".").getAbsoluteFile().getParentFile();
	}
}
/*/*
This file forms part of Version 0.8.79 of Facets, a framework demonstrating 
the Superficial approach to GUI-driven interactive applications.  
http://superficial.sourceforge.net 				
Copyright (C) 2011  David M Wright 
This library is free software; you can redistribute it and/or modify it under 
the terms of the GNU Lesser General Public License as published by the 
Free Software Foundation; either release 3 of the License, or (at your 
option) any later release.
This library is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR 
A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more 
details.
You should have received a copy of the GNU Lesser General Public License along 
with this library; if not, write to the Free Software Foundation, Inc., 
59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

