package facets.util.app;
import facets.util.Times;
import facets.util.Util;
import facets.util.datatree.Nodes;
import facets.util.datatree.TypedNode;
import facets.util.datatree.ValueNode;
import java.util.Timer;
import java.util.TimerTask;
/**
Event tracing. 
 */
public final class Events{
	public static boolean trace=false,events=true,buildTimes,restarting,canRestart=true;
	final public static String KEY_TRACE="trace",
		KEY_EVENTS="events",KEY_BUILD="buildTimes",KEY_TIMES="times",
		KEY_TIMES_RESET="timesReset",KEY_ESTIMATE="buildEstimate",KEY_MEM="memCheck",
		KEY_FILTERS="filters",
		FILTERS[]={"Cursor","Notified","Retargeting","Selection","Retargeted","Refreshing",
			"Retrieved","Created","Attached","Storing"};
	public static final ValueNode DEFAULT_FILTERS=new ValueNode(KEY_FILTERS,
			TypedNode.UNTITLED);
	static{
		Object[]keyPairs=new String[FILTERS.length];
		for(int i=0;i<keyPairs.length;i++)keyPairs[i]=FILTERS[i]+"=true";
		DEFAULT_FILTERS.setContents(keyPairs);
	}
	static Events.EventTracer buildMonitor;
	static Timer timer;

	/**
	Can be notified of build etc messages.
	<p>An {@link EventTracer} passed to {@link Events#setBuildMonitor(EventTracer)}
	will be be updated with any message starting with "\gt>"  
	created with {@link Events#traceEvent(String)}.
	 */
	public static abstract class EventTracer{
		protected abstract void update(String msg,int elapsed);
		protected abstract void setBuildEstimate(int millis);
	}
	public static void setBuildMonitor(EventTracer monitor){
		buildMonitor=monitor;
		monitor.setBuildEstimate(AppValues.stateDebug.getInt(KEY_ESTIMATE));
	}
	public static void traceEvent(String msg){
		String firstWord=msg.replaceAll("(?s)~?(\\w+).*","$1"),
			noDetail=true?null:msg.replaceAll(
				"([A-Z][^A-Z]+).*","$1").replaceAll("\\b for\\b","").replaceAll(
				"\\b in\\b","").replaceAll("\\b from\\b","")
				.replaceAll("\\b to\\b","").trim();
		if(noDetail!=null&&noDetail.startsWith(">")) {
			noDetail=noDetail.substring(1);
			if(timer!=null)timer.cancel();
			final int elapsed=true?0:(int)Times.elapsed();
			(timer=new Timer()).schedule(new TimerTask() {
				public void run(){
					AppValues.stateDebug.put(KEY_ESTIMATE,elapsed);
					timer.cancel();
				}			
			},2000);
			if(buildMonitor!=null)buildMonitor.update(noDetail,elapsed);
			if(trace&&buildTimes)Util.println(noDetail+" ",elapsed);
		}
		if(!trace)return;
		ValueNode filters=AppValues.stateDebug==null?DEFAULT_FILTERS:Nodes.guaranteedValueChild(
				AppValues.stateDebug,KEY_FILTERS);
		if(events&&filters!=null&&!filters.getBoolean(firstWord)){
			long elapsed=Times.elapsed();
			restarting=elapsed==0&&canRestart;
			Util.println((restarting?"\n":"")+
					(elapsed>10000?elapsed/1000+"s":elapsed)
					+" "+msg.replaceAll("^>(.+)","$1"));
			canRestart=elapsed>0;
		}
	}}
/*/*
This file forms part of Version 0.8.79 of Facets, a framework demonstrating 
the Superficial approach to GUI-driven interactive applications.  
http://superficial.sourceforge.net 				
Copyright (C) 2011  David M Wright 
This library is free software; you can redistribute it and/or modify it under 
the terms of the GNU Lesser General Public License as published by the 
Free Software Foundation; either release 3 of the License, or (at your 
option) any later release.
This library is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR 
A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more 
details.
You should have received a copy of the GNU Lesser General Public License along 
with this library; if not, write to the Free Software Foundation, Inc., 
59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

