package facets.util.app;
import static facets.util.Debug.*;
import static java.lang.Math.*;
import static java.lang.System.*;
import facets.util.Debug;
import facets.util.Doubles;
import facets.util.Objects;
import facets.util.PackedSerialized;
import facets.util.SizeEstimable;
import facets.util.Times;
import facets.util.Titled;
import facets.util.Tracer;
import facets.util.Util;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
/**
Cache that encapsulates the creation of items to be stored.
 */
public class ProvidingCache extends Tracer{
	public static final int PASS_THROUGH=Integer.MAX_VALUE;
	private static final boolean traceVerbose=false,timing=false;
	private static final long loadTime=currentTimeMillis();
	protected void traceOutput(String msg){
		if(true||!trace())return;
		msg=Debug.info(this)+": "+msg;
		if(timing)Times.printElapsed(msg);
		else System.out.println(msg);
	}
	protected boolean trace(){
		return false||timing;
	}
	private final class Key{
		final Object[]values;
		final long time;
		long memAdvance=-1,memFinal=-1,timeCost=-1;
		Key(ItemProvider request,Object[]values){
			if(values.length==0)throw new IllegalArgumentException(
					"Empty values in "+Debug.info(this));
			this.values=Objects.join(Object.class,
					new Object[]{request.source,request.getClass()},values);
			time=currentTimeMillis()-loadTime;
		}
		Double keepValue(){
			double recent=sqrt((double)time/(currentTimeMillis()-loadTime)),
				space=memFinal==0?0:(1/sqrt(memFinal/1024d)),create=sqrt(timeCost),
				trio[]={recent,space,create};
			for(int i=0;i<trio.length;i++)if(Double.isInfinite(trio[i]))
					throw new IllegalStateException(new String[]{"recent","space","create"}[i]+
							"=infinity in "+Debug.info(this));
			return recent*5+space*10+create/100;
		}
		boolean matches(Object[]values){
			boolean matches=true;
			for(int i=0;i<values.length;i++)
				matches&=values[i]==null||values[i].equals(this.values[i]);
			return matches;
		}
		public boolean equals(Object o){
			return Arrays.equals(values,(((Key)o).values));
		}
		public String toString(){
			String[]ids=new String[values.length-2];
			for(int i=0;i<ids.length;i++)ids[i]=Debug.id(values[i+2]);
			return (false?"":(" ["+Debug.info(values[0])+","+
			values[1].toString().replaceFirst("[^A-Z]+","")+"," +
				Objects.toString(ids)+"]"))+(timeCost>=0?(" advance="+mb(memAdvance)+
			" @"+time/100+" ''"+timeCost+" final="+mb(memFinal)+" keep="+Doubles.sf(keepValue())):"");
		}
		@Override
		public int hashCode(){
			return Arrays.hashCode(values);
		}
	}
	private final Map<Key,Object>items=new HashMap();
	private final int countMax;
	private final AppWatcher watcher;
	public ProvidingCache(int countMax,AppWatcher watcher){
		this.countMax=countMax;
		this.watcher=watcher;
	}
	private Object getItemForValues(final ItemProvider p,Object[]values){
		final Key key=new Key(p,values);
		if(traceVerbose)trace("GETTING " +key);
		processingRequest(true);
		final boolean passThrough=countMax==PASS_THROUGH||p.passThrough();
		Object stored=passThrough?null:items.get(key);
		if(stored!=null){
			for(Key itemKey:items.keySet())
				if(itemKey.equals(key)){
					key.memFinal=itemKey.memFinal;
					key.timeCost=itemKey.timeCost;
				}
			items.put(key,stored);
			if(traceVerbose)trace("GOT "+Debug.info(stored)+key);
			return p.packsItems()?p.unpackItem(stored):stored;
		}
		int itemCountStart=items.size(),maximumCount=false?5:countMax<0?0
				:p.maximumSharePct()*countMax/(true?countMax:100);
		if(maximumCount>0){
			List<Key>matches=keysMatching(p.source,p.getClass());
			int matchCount=matches.size();
			if(matchCount>=maximumCount){
				Collections.sort(matches,new Comparator<Key>(){
					@Override
					public int compare(Key o,Key p){
						return new Long(o.time).compareTo(new Long(p.time));
					}
				});
				if(false)trace(".getItemForValues: matches=",matches);
				Key remove=matches.get(0);
				traceDebug("REMOVED maximumCount="+maximumCount+" matchCount="+matchCount+
						"\n\t"+remove+" > ",items.remove(remove));
				itemCountStart=items.size();
			}
		}
		long itemMems=0;
		for(Key k:items.keySet())itemMems+=k.memFinal;
		final long memNeeded=checkItemBytes()?(key.memAdvance=p.advanceByteCount()):-1;
		long memAvailable=memNeeded>=0?getBytesAvailable(itemMems):memNeeded;
		trace("CHECK items="+itemCountStart +(memNeeded<0?"":(" itemMems="+mb(itemMems)+
				(true?"":(" available="+mb(memAvailable)))+" needed="+mb(memNeeded)))+" for "+key);
		if((memNeeded>=0&&memAvailable<=0)||memNeeded<0&&itemCountStart>countMax){
			List<Key>purgeSort=new ArrayList(items.keySet());
			Collections.sort(purgeSort,new Comparator<Key>(){
				@Override
				public int compare(Key o,Key p){
					return o.keepValue().compareTo(p.keepValue());
				}
			});
			trace("PURGING from ",purgeSort);
			long purged=0;
			List<Key>purge=purgeSort.subList(0,purgeSort.size()/4);
			for(Key r:purge){
				items.remove(r);
				purged+=r.memFinal;
			}
			Runtime.getRuntime().gc();
			trace("PURGED purged="+mb(purged)+" ",purge);
		}
		long beforeNew=currentTimeMillis();
		Object item=null;
		try{
			item=watcher==null?p.doReturnableOperation():watcher.runWatched(p);
		}
		catch(OutOfMemoryError e){
			MemoryChecks.checkStatus(e.getMessage()+": Cannot create item for "+p);
			System.exit(-1);
		}
		if(passThrough||item==null)return item;
		Object storeReady=p.packsItems()?p.packItem(item):item;
		key.timeCost=currentTimeMillis()-beforeNew;
		key.memFinal=memNeeded>=0?p.finalByteCount(storeReady):memNeeded;
		items.put(key,storeReady);
		traceDebug("PUT items="+items.size()+" "+key+" > ",storeReady);
		processingRequest(false);
		return item;
	}
	/**
	{@link WatchableOperation} that creates items for storage and return by 
	{@link ProvidingCache}.
	<p><i>Note: </i>In contrast to the superclass, {@link #cancelStyle()} returns <code>true</code>
	by default. 
	 */
	public abstract static class ItemProvider<T>extends WatchableOperation{
		private final ProvidingCache c;
		final Object source;
		/**
		Re-implementation. 
	@return <code>true</code> by default . 
		 */
		@Override
		public CancelStyle cancelStyle(){
			return CancelStyle.Timeout;
		}
		protected boolean passThrough(){
			return false;
		}
		public Object doReturnableOperation(){
			return newItem();
		}
		public ItemProvider(ProvidingCache c,Object source,String title){
			super(title);
			this.c=c;
			this.source=source;
		}
		final public T getForValues(Object...values){
			T t=(T)c.getItemForValues(this,values);
			return t;
		}
		final public T getNewForValues(Object...values){
			c.removeItemForValues(this,values);
			T t=(T)c.getItemForValues(this,values);
			return t;
		}
		final public boolean hasForValues(Object...values){
			return c.containsItemForValues(this,values);
		}
		public final void removeForValues(Object...values){
			c.removeItemForValues(this,values);
		}
		public final void removeAll(){
			c.removeMatchingItems(source,getClass());
		}
		protected int maximumSharePct(){
			return 0;
		}
		protected long advanceByteCount(){
			throw new RuntimeException("Not implemented in "+Debug.info(this));
		}
		protected abstract T newItem();
		protected long finalByteCount(Object storeReady){
			return storeReady instanceof SizeEstimable?
					((SizeEstimable)storeReady).estimateSize():advanceByteCount();
		}
		protected boolean packsItems(){
			return false;
		}
		protected Object packItem(T item){
			return new PackedSerialized((Serializable)item);
		}
		protected T unpackItem(Object packed){
			return(T)((PackedSerialized)packed).unpack();
		}
	}
	protected AppWatcher watcher(){
		return null;
	}
	protected void processingRequest(boolean starting){}
	protected boolean checkItemBytes(){
		return false;
	}
	protected long getBytesAvailable(long itemMems){
		return(countMax*1<<20)-itemMems;
	}
	private boolean containsItemForValues(ItemProvider r,Object[]values){
		return items.get(new Key(r,values))!=null;
	}
	private void removeItemForValues(ItemProvider r,Object[]values){
		items.remove(new Key(r,values));
	}
	private void removeMatchingItems(Object...values){
		processingRequest(true);
		int sizeThen=items.size();
		for(Object remove:keysMatching(values))
			traceDebug("Removing ",items.remove(remove));
		int sizeNow=items.size();
		if(sizeThen-sizeNow>0)trace("REMOVE MATCHING "+
						Debug.arrayInfo(values)+
					" sizeThen="+sizeThen+" sizeNow="+sizeNow);
		processingRequest(false);
	}
	private List<Key>keysMatching(Object...values){
		List<Key>removes=new ArrayList();
		for(Key k:items.keySet())if(k.matches(values))removes.add(k);
		return removes;
	}
	public final void clear(){
		items.clear();
		trace("CLEAR");
	}
}
/*/*
This file forms part of Version 0.8.79 of Facets, a framework demonstrating 
the Superficial approach to GUI-driven interactive applications.  
http://superficial.sourceforge.net 				
Copyright (C) 2011  David M Wright 
This library is free software; you can redistribute it and/or modify it under 
the terms of the GNU Lesser General Public License as published by the 
Free Software Foundation; either release 3 of the License, or (at your 
option) any later release.
This library is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR 
A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more 
details.
You should have received a copy of the GNU Lesser General Public License along 
with this library; if not, write to the Free Software Foundation, Inc., 
59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

