package facets.util.datatree;
import facets.util.Debug;
import facets.util.Strings;
import facets.util.Util;
/**
Wraps an exception in a {@link DataNode}.
<p>{@link ExceptionNode} enables the graceful handling of almost
any exception thrown during construction of a tree of {@link TypedNode}s. 
The exception can be caught by returning it wrapped in an {@link ExceptionNode},
allowing execution to continue while ensuring that the exception is recorded. 
<p>Typically the {@link ExceptionNode} is eventually displayed
in a tree viewer, facilitating debugging. 
 */
final public class ExceptionNode extends DataNode{
	public static final Object TYPE="Exception";
	/**
	 If set to <code>true</code>, the exception passed to the constructor
	 is rethrown.  
	 */
	public static boolean throwExceptions=true,alertExceptions=true;
	public ExceptionNode(Exception e){
		this(e,null);
	}
  public ExceptionNode(Exception e,TypedNode[]detail){
    super("Exception",e.getClass().getSimpleName());
    String[]trace=Debug.readTraceLines(e,20,"",0);
    if(throwExceptions)throw new RuntimeException
			("\n"+Strings.linesString(trace),e);
    else if(alertExceptions)
    	Util.println("ExceptionNode: ",trace);
    setContents(detail==null?(Object[])trace:detail);
  }
}
/*/*
This file forms part of Version 0.8.79 of Facets, a framework demonstrating 
the Superficial approach to GUI-driven interactive applications.  
http://superficial.sourceforge.net 				
Copyright (C) 2011  David M Wright 
This library is free software; you can redistribute it and/or modify it under 
the terms of the GNU Lesser General Public License as published by the 
Free Software Foundation; either release 3 of the License, or (at your 
option) any later release.
This library is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR 
A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more 
details.
You should have received a copy of the GNU Lesser General Public License along 
with this library; if not, write to the Free Software Foundation, Inc., 
59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

