package facets.util.datatree;
import static facets.util.datatree.Nodes.*;
import facets.util.Debug;
import facets.util.Objects;
import facets.util.Titled;
/**
Reads and writes contents of a {@link ValueNode}. 
 */
public abstract class ValueProxy implements Titled{
	@Override
	public String title(){
		return " source="+Debug.info(source);
	}
	public final ValueNode source;
	private Object[]values={};
	public ValueProxy(ValueNode source){
		if((this.source=source)==null)
			throw new IllegalArgumentException("Null source in "+Debug.info(this));
	}
	final public Object get(int at){
		if(values.length>0)return checkValues(at);
		values=lazyValues();
		return checkValues(at);
	}
	private Object checkValues(int at){
		for(Object value:values)if(value==null)throw new IllegalStateException(
				"Null value in "+Debug.info(this));
		if(at>=values.length)throw new IllegalStateException(
				"Can't get at=" +at+" from values="+values.length+
				" in "+Debug.info(this));
		else return values;
	}
	final public void put(int at,Object value){
		if(values.length==0)get(at);
		if(value==null)throw new IllegalArgumentException("Null value in "+Debug.info(this));
		else if(values[at].getClass()!=value.getClass())
			throw new IllegalArgumentException("Wrong value class in "+Debug.info(this));
		values[at]=value;
	}
	final public int valueCount(){
		return values.length;
	}
	final public void updateSource(){
		applyValuesToSource(values);
	}
	protected Object[]lazyValues(){
		throw new RuntimeException("Not implemented in "+Debug.info(this));
	}
	protected void applyValuesToSource(Object[]values){
		throw new RuntimeException("Not implemented in "+Debug.info(this));
	}
	public String toString(){
		return Debug.info(this)+"<-"+Debug.info(source);
	}
	public static ValueProxy sourceProxy(ValueProxy[]proxies,Object source){
		for(int i=0;i<proxies.length;i++)
			if(proxies[i].source==source)return proxies[i];
		throw new IllegalArgumentException(
				source+" not found in "+Objects.toStringWithHeader(proxies));
	}
	public static ValueNode[]getProxySources(Object[]proxies){
		ValueNode[]sources=new ValueNode[proxies.length];
		for(int i=0;i<sources.length;i++)sources[i]=((ValueProxy)proxies[i]).source;
		return sources;
	}
	public boolean equals(Object o){
		return stateString().equals(((ValueProxy)o).stateString());
	}
	public String stateString(){
		return treeString(source);
	}
	public static String stateString(ValueProxy[]array){
		StringBuilder str=new StringBuilder();
		for(ValueProxy each:array)str.append(each.stateString());
		return str.toString();
	}
}
/*/*
This file forms part of Version 0.8.79 of Facets, a framework demonstrating 
the Superficial approach to GUI-driven interactive applications.  
http://superficial.sourceforge.net 				
Copyright (C) 2011  David M Wright 
This library is free software; you can redistribute it and/or modify it under 
the terms of the GNU Lesser General Public License as published by the 
Free Software Foundation; either release 3 of the License, or (at your 
option) any later release.
This library is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR 
A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more 
details.
You should have received a copy of the GNU Lesser General Public License along 
with this library; if not, write to the Free Software Foundation, Inc., 
59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

