package facets.util.datatree;
import facets.util.Debug;
import facets.util.FileSpecifier;
import facets.util.TextLines;
import facets.util.datatree.Nodes.TreeRoot;
import java.io.File;
/**
{@link FileSpecifier} with attached {@link XmlPolicy}. 
 */
public class XmlSpecifier extends FileSpecifier{
	public final XmlPolicy policy;
	public final String dataType;
	public XmlSpecifier(String extension,String description,XmlPolicy policy){
		super(extension,description);
		this.policy=policy;
		if(!extension.contains(DataConstants.TYPE_XML))throw new IllegalArgumentException(
				"Non-xml extension " +extension+" in "+Debug.info(this));
		final String[]raw=extension.replace(TextLines.EXT_ZIP,"").split("\\.");
		StringBuilder safe=new StringBuilder(raw[0]);
		for(int i=1;i<raw.length;i++)
			raw[i]=raw[i].substring(0,1).toUpperCase()+raw[i].substring(1);
		dataType=safe.toString();
	}
	/**
	Convenience method creating a temporary node for use as {@link TreeRoot#tree}. 
	<p>Instances supplies {@link #dataType} and {@link XmlPolicy}
	@param data the data tree to be wrapped
	@return an {@link XmlDocRoot} with {@link TreeRoot#tree} of type {@link #dataType}, 
	titled from <code>data</code> which is set as its single child
	 */
	final public XmlDocRoot newRootParent(DataNode data){
		return new XmlDocRoot(policy.treeAsXmlRoot()?data
				:new DataNode(dataType,data.title(),new Object[]{data}),
				policy);
	}
	final public File newFile(File dir,String nameTop){
		return new File(dir,nameTop+dot+extension);
	}
	final public DataNode newDataNode(File file){
		return new DataNode(dataType,file.getName().replaceFirst(_dot+".+",""));
	}
}
/*/*
This file forms part of Version 0.8.79 of Facets, a framework demonstrating 
the Superficial approach to GUI-driven interactive applications.  
http://superficial.sourceforge.net 				
Copyright (C) 2011  David M Wright 
This library is free software; you can redistribute it and/or modify it under 
the terms of the GNU Lesser General Public License as published by the 
Free Software Foundation; either release 3 of the License, or (at your 
option) any later release.
This library is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR 
A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more 
details.
You should have received a copy of the GNU Lesser General Public License along 
with this library; if not, write to the Free Software Foundation, Inc., 
59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

